@@ -67,15 +67,15 @@
                             Include header files
 ******************************************************************************/
 #include "lcmaps_config.h"
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #ifdef LCMAPS_GSI_MODE
 #   include <gssapi.h>
 #endif
 
+#include <stdio.h>
+
 #include "lcmaps_modules.h"
 #include "lcmaps_arguments.h"
+#include "lcmaps_plugin_prototypes.h"
 
 /******************************************************************************
                                 Definitions
@@ -131,7 +131,7 @@ int plugin_introspect(
 
     *argv = argList;
     *argc = lcmaps_cntArgs(argList);
-    lcmaps_log_debug(1,"lcmaps_plugin_example-plugin_introspect(): address first argument: 0x%x\n",argList);
+    lcmaps_log_debug(1,"lcmaps_plugin_example-plugin_introspect(): address first argument: %p\n",(void*)argList);
 
     return LCMAPS_MOD_SUCCESS;
 }
@@ -261,24 +261,24 @@ int plugin_run(
     if ( ( value = lcmaps_getArgValue("user_cred", "gss_cred_id_t", argc, argv) ) )
     {
 	pcred = (gss_cred_id_t *) value;
-        lcmaps_log_debug(1,"lcmaps_plugin_example-plugin_run(): address user_cred: %p\n",pcred);
-        lcmaps_log_debug(1,"lcmaps_plugin_example-plugin_run(): value user_cred: %p\n",*pcred);
+        lcmaps_log_debug(1,"lcmaps_plugin_example-plugin_run(): address user_cred: %p\n",(void*)pcred);
+        lcmaps_log_debug(1,"lcmaps_plugin_example-plugin_run(): value user_cred: %p\n",(void*)*pcred);
         cred=*pcred;
         if (cred) {
-            lcmaps_log_debug(1,"lcmaps_plugin_example-plugin_run(): inside value user_cred: %p\n",*(int *)(cred));
+            lcmaps_log_debug(1,"lcmaps_plugin_example-plugin_run(): inside value user_cred: %d\n",*(int *)(cred));
         }
     }
     else
         lcmaps_log_debug(1,"lcmaps_plugin_example-plugin_run(): could not get address of user_cred !\n");
 #endif
 
-    lcmaps_log_debug(1,"lcmaps_plugin_example-plugin_run(): address first argument: 0x%x\n",argv);
+    lcmaps_log_debug(1,"lcmaps_plugin_example-plugin_run(): address first argument: %p\n",(void*)argv);
 
     /* succes */
     return LCMAPS_MOD_SUCCESS;
 
- fail_example:
-    return LCMAPS_MOD_FAIL;
+/* fail_example:
+    return LCMAPS_MOD_FAIL;*/
 }
 
 /******************************************************************************
@@ -297,7 +297,7 @@ Returns:
     \retval LCMAPS_MOD_SUCCESS success
     \retval LCMAPS_MOD_FAIL    failure (will result in an authorization failure)
 */
-int plugin_terminate()
+int plugin_terminate(void)
 {
     lcmaps_log_debug(1,"lcmaps_plugin_example-plugin_terminate(): terminating\n");
 
@@ -308,7 +308,7 @@ int plugin_terminate()
 /******************************************************************************
 CVS Information:
     $Source: /srv/home/dennisvd/svn/mw-security/lcmaps/examples/lcmaps_plugin_example.c,v $
-    $Date: 2013-02-22 12:34:39 +0100 (Fri, 22 Feb 2013) $
-    $Revision: 16862 $
+    $Date: 2014-07-07 21:28:25 +0200 (Mon, 07 Jul 2014) $
+    $Revision: 17839 $
     $Author: msalle $
 ******************************************************************************/
