@@ -50,44 +50,50 @@
  *
  *
  *  \author  G.M. Venekamp  (venekamp@nikhef.nl)
- *  \version $Revision: 16542 $
- *  \date    $Date: 2012-10-11 17:29:15 +0200 (Thu, 11 Oct 2012) $
+ *  \version $Revision: 18217 $
+ *  \date    $Date: 2015-02-02 12:00:49 +0100 (Mon, 02 Feb 2015) $
  *
  */
 
+#define _XOPEN_SOURCE	500
 
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
 
-#include "_lcmaps_pluginmanager.h"
 #include "lcmaps_log.h"
-#include "pdl.h"
-#include "pdl_variable.h"
-#include "pdl_policy.h"
-#include "pdl_rule.h"
 
-static char* script_name        = NULL;           //!<  If non NULL, the name of the configuration script.
-static char* path               = NULL;           //!<  Path where plugins can be found.
-static const char* undefined	= "(empty string)"; //!< String printed instead of (null)
-static int path_lineno          = 0;              //!<  Path line number.
-static plugin_t* top_plugin     = NULL;           //!<  First node of the list.
-static BOOL parse_error         = FALSE;          //!<  Tell if there have been any error during parsing.
-static char* level_str[PDL_SAME];                 //!<  When a message is printed, how do we spell warning in a given language.
+#include "evaluationmanager/pdl.h"
+#include "evaluationmanager/pdl_variable.h"
+#include "evaluationmanager/pdl_policy.h"
+#include "evaluationmanager/pdl_rule.h"
+
+
+#define WARNBUFLEN    ((size_t)2048L)
+
+
+static char* script_name        = NULL;           /* If non NULL, the name of the configuration script. */
+static char* path               = NULL;           /* Path where plugins can be found. */
+static const char* undefined	= "(empty string)"; /*String printed instead of (null) */
+static int path_lineno          = 0;              /* Path line number. */
+static plugin_t* top_plugin     = NULL;           /* First node of the list. */
+static BOOL parse_error         = FALSE;          /* Tell if there have been any error during parsing. */
+static const char* level_str[PDL_SAME];                 /* When a message is printed, how do we spell warning in a given language. */
 
 static       policy_t* current_policy = NULL;
 static const rule_t*   current_rule   = NULL;
 
-unsigned int lineno = 1;   //!<  The first line of a configuration sctipt is labeled 1.
+int lineno = 1;   /* The first line of a configuration sctipt is labeled 1. */
 
-void _lcmaps_set_path(const record_t* _path);
-record_t* _lcmaps_concat_strings(const record_t* s1, const record_t* s2, const char* extra);
+static void _lcmaps_set_path(const record_t* _path);
+static record_t* _lcmaps_concat_strings(const record_t* s1, const record_t* s2, const char* extra);
 /* void lcmaps_reduce_policies(void); */
-void lcmaps_init_name_args(plugin_t** plugin, const rule_t* rule, rule_type_t type);
-BOOL lcmaps_plugin_exists(const char* string);
-int lcmaps_find_first_space(const char* string);
-int lcmaps_free_plugins(plugin_t ** list);
+static int lcmaps_init_name_args(plugin_t** plugin, const rule_t* rule, rule_type_t type);
+static BOOL lcmaps_plugin_exists(const char* string);
+static size_t lcmaps_find_first_space(const char* string);
+static int lcmaps_free_plugins(plugin_t ** list);
 
 /*!
  *  Return the current policy.
@@ -126,7 +132,7 @@ int lcmaps_pdl_init(const char* name)
 
     script_name = strdup(name);
     if (script_name==NULL)  {
-	lcmaps_warning(PDL_ERROR, "Out of memory when trying to open '%s'.", name);
+	lcmaps_pdl_warning(PDL_ERROR, "Out of memory when trying to open '%s'.", name);
 	return -1;
     }
 
@@ -135,7 +141,7 @@ int lcmaps_pdl_init(const char* name)
       /* set_active_yyin (file); */
       yyin = file;
     else {
-      lcmaps_warning(PDL_ERROR, "Could not open file '%s'.", name);
+      lcmaps_pdl_warning(PDL_ERROR, "Could not open file '%s'.", name);
       return -1;
     }
   }
@@ -213,9 +219,12 @@ const plugin_t* lcmaps_get_plugins(void)
 	      rule->false_branch ? rule->false_branch : undefined);
 
       lcmaps_log_debug(4, "  get_plugins:  initializing...\n");
-      lcmaps_init_name_args(&plugin, rule, STATE);
-      lcmaps_init_name_args(&plugin, rule, TRUE_BRANCH);
-      lcmaps_init_name_args(&plugin, rule, FALSE_BRANCH);
+      if (lcmaps_init_name_args(&plugin, rule, STATE)<0 ||
+          lcmaps_init_name_args(&plugin, rule, TRUE_BRANCH)<0 ||
+          lcmaps_init_name_args(&plugin, rule, FALSE_BRANCH)<0)	{
+	      lcmaps_log(LOG_ERR,"Error initializing plugins.\n");
+	      return NULL;
+      }
       lcmaps_log_debug(4, "  get_plugins:  initializing done.\n");
 
       rule = rule->next;
@@ -236,7 +245,7 @@ const plugin_t* lcmaps_get_plugins(void)
  *  \retval 1 failure
  *
  */
-int lcmaps_free_plugins(
+static int lcmaps_free_plugins(
         plugin_t ** list
 )
 {
@@ -247,7 +256,7 @@ int lcmaps_free_plugins(
     while (p_list) {
         tmp_p_list = p_list->next;
 
-        lcmaps_log_debug(5, "freeing plugin %s at address %p\n", p_list->name, p_list);
+        lcmaps_log_debug(5, "freeing plugin %s at address %p\n", p_list->name, (void*)p_list);
         if (p_list->name){
             free(p_list->name);
             p_list->name = NULL;
@@ -272,19 +281,20 @@ int lcmaps_free_plugins(
  *  \return TRUE if the plugin exists, FALSE otherwise.
  *
  */
-BOOL lcmaps_plugin_exists(const char* string)
+static BOOL lcmaps_plugin_exists(const char* string)
 {
-  int space, remainder;
+  size_t space,string_len;
   plugin_t *plugin;
 
+  string_len= strlen(string);
   space     = lcmaps_find_first_space(string);
-  remainder = strlen(string) - space - 1;
   plugin    = top_plugin;
 
   while (plugin) {
     if (plugin->name && strncmp(plugin->name, string, space)==0) {
       if (plugin->args)
-        if (strncmp(plugin->args, string+space+1, remainder)!=0) {
+        if (string_len<=space+1 ||
+	    strncmp(plugin->args, string+space+1, string_len-space-1)!=0) {
           plugin = plugin->next;
           continue;
         }
@@ -300,58 +310,82 @@ BOOL lcmaps_plugin_exists(const char* string)
 /*!
  *
  */
-void lcmaps_init_name_args(plugin_t** plugin, const rule_t* rule, rule_type_t type)
+static int lcmaps_init_name_args(plugin_t** plugin, const rule_t* rule, rule_type_t type)
 {
-  int space, remainder;
+  size_t space, remainder, string_len;
   const char* string;
+  const char* branch;
 
   switch (type) {
   case STATE:
     string = rule->state;
+    branch = "STATE";
     break;
   case TRUE_BRANCH:
     string = rule->true_branch;
+    branch = "TRUE_BRANCH";
     break;
   case FALSE_BRANCH:
     string = rule->false_branch;
+    branch = "FALSE_BRANCH";
     break;
   default:
-    lcmaps_warning(PDL_ERROR, "init_name_args: unknown type!");
-    return;
+    lcmaps_pdl_warning(PDL_ERROR, "init_name_args: unknown type!");
+    return -1;
   }
 
-  lcmaps_log_debug(5, "  init_name_args: processing: %s\n", string ? string : undefined);
-
-  if (!string || lcmaps_plugin_exists(string)) {
-    lcmaps_log_debug(5, "  init_name_args: Either the plugin exists or string == 0.\n");
-    return;
+  if (!string)	{
+      lcmaps_log(LOG_DEBUG, "  init_name_args: no plugin on %s\n",branch);
+      return 0;
   }
 
-  lcmaps_log_debug(3, "  init_name_args: plugin does not exists.\n");
+  lcmaps_log_debug(5, "  init_name_args: processing %s: %s\n", branch,string);
+
+  if (lcmaps_plugin_exists(string)) {
+    lcmaps_log_debug(5, "  init_name_args: The plugin already exists.\n");
+    /* Not an error */
+    return 0;
+  }
+
+  lcmaps_log_debug(3, "  init_name_args: plugin does not yet exist.\n");
 
   if (!top_plugin) {
-    top_plugin = (plugin_t*)malloc(sizeof(plugin_t));
+    if ( (top_plugin = (plugin_t*)malloc(sizeof(plugin_t)))==NULL)  {
+	lcmaps_log(LOG_ERR,"%s: Out of memory\n",__func__);
+	return -1;
+    }
     *plugin = top_plugin;
   }
   else {
-    (*plugin)->next = (plugin_t*)malloc(sizeof(plugin_t));
+    if ( ((*plugin)->next = (plugin_t*)malloc(sizeof(plugin_t)))==NULL)	{
+	lcmaps_log(LOG_ERR,"%s: Out of memory\n",__func__);
+	return -1;
+    }
     *plugin = (*plugin)->next;
   }
 
   (*plugin)->name = (*plugin)->args = NULL;
   (*plugin)->next = NULL;
 
+  string_len=strlen(string);
   space = lcmaps_find_first_space(string);
 
-  lcmaps_log_debug(5, "  init_name_args: space found a pos: %d  strlen = %d.\n", space, strlen(string));
+  lcmaps_log_debug(5, "  init_name_args: space found a pos: %lu  strlen = %lu.\n",
+	  (long unsigned)space, (long unsigned)string_len);
 
-  (*plugin)->name = (char *)malloc(space+1);
+  if ( ((*plugin)->name = (char *)malloc(space+1))==NULL)   {
+	lcmaps_log(LOG_ERR,"%s: Out of memory\n",__func__);
+	return -1;
+  }
   strncpy((*plugin)->name, string, space);
   *((*plugin)->name+space) = '\0';
 
-  remainder = strlen(string)-space-1;
-  if (remainder > 0) {
-    (*plugin)->args   = (char *)malloc(remainder+1);
+  if (string_len>space+1)   {
+    remainder = string_len-space-1;
+    if ( ((*plugin)->args   = (char *)malloc(remainder+1))==NULL)   {
+	lcmaps_log(LOG_ERR,"%s: Out of memory\n",__func__);
+	return -1;
+    }
     strncpy((*plugin)->args, string+space+1, remainder);
     *((*plugin)->args+remainder) = '\0';
   } else
@@ -364,6 +398,8 @@ void lcmaps_init_name_args(plugin_t** plugin, const rule_t* rule, rule_type_t ty
 	  (*plugin)->name ? (*plugin)->name : undefined );
   lcmaps_log_debug(4, "  init_name_args: plugin->args = %s\n",
 	  (*plugin)->args ? (*plugin)->args : undefined );
+
+  return 0;
 }
 
 
@@ -376,9 +412,9 @@ void lcmaps_init_name_args(plugin_t** plugin, const rule_t* rule, rule_type_t ty
  *          string.
  *
  */
-int lcmaps_find_first_space(const char* string)
+static size_t lcmaps_find_first_space(const char* string)
 {
-  int space, max_length;
+  size_t space, max_length;
   space = 0;
   max_length = strlen(string);
 
@@ -396,7 +432,7 @@ int lcmaps_find_first_space(const char* string)
 char *lcmaps_pdl_path(void)
 {
   return path;
-};
+}
 
 
 /*!
@@ -410,7 +446,7 @@ int yyerror(const char* s)
 {
   /* lcmaps_log_debug(3, "%s\n", token_name()); */
 
-  lcmaps_warning(PDL_ERROR, s);
+  lcmaps_pdl_warning(PDL_ERROR, "%s", s);
 
   return 0;
 }
@@ -421,7 +457,7 @@ int yyerror(const char* s)
  *  reserved path word. This function acts as a wrapper for the
  *  _lcmaps_set_path() function.
  *
- *  \param path
+ *  \param mypath path to be set
  */
 void lcmaps_set_path(record_t* mypath)
 {
@@ -441,12 +477,12 @@ void lcmaps_set_path(record_t* mypath)
  *  \param _path The new path.
  *
  */
-void _lcmaps_set_path(const record_t* _path)
+static void _lcmaps_set_path(const record_t* _path)
 {
   /* lcmaps_log_debug(3, "lcmaps_set_path: '%s'.\n", _path); */
 
   if (path!=NULL) {
-    lcmaps_warning(PDL_WARNING, "path already defined as %s in line: %d; ignoring this instance.", path, path_lineno);
+    lcmaps_pdl_warning(PDL_WARNING, "path already defined as %s in line: %d; ignoring this instance.", path, path_lineno);
     return;
   }
   if (_path!=NULL)  {  /* Only act when value is provided */
@@ -454,14 +490,14 @@ void _lcmaps_set_path(const record_t* _path)
       if ((_path->string)[0]!='/') {	/* Relative path: prefix LCMAPS_LIB_HOME */
 	  path=calloc(strlen(LCMAPS_LIB_HOME)+strlen(_path->string)+2,sizeof(char));
 	  if (path==NULL)   {
-	      lcmaps_warning(PDL_ERROR, "Out of memory when setting path.");
+	      lcmaps_pdl_warning(PDL_ERROR, "Out of memory when setting path.");
 	      return;
 	  }
 	  sprintf(path,"%s/%s",LCMAPS_LIB_HOME,_path->string);
       } else {
 	  path = strdup(_path->string);
 	  if (path==NULL)   {
-	     lcmaps_warning(PDL_ERROR, "Out of memory when setting path.");
+	     lcmaps_pdl_warning(PDL_ERROR, "Out of memory when setting path.");
 	     return;
 	  }
       }
@@ -483,8 +519,8 @@ void lcmaps_free_path(void)
 
 
 /*!
- *  Concatenate two strings. The orginal two strings are freed. When
- *  the concatenation fails, the origial strings are still freed. The
+ *  Concatenate two strings. The original two strings are freed. When
+ *  the concatenation fails, the original strings are still freed. The
  *  actual concatenation is done by _lcmaps_concat_strings().
  *
  *  \param s1    first half of the string.
@@ -514,16 +550,17 @@ record_t* lcmaps_concat_strings(record_t* s1, record_t* s2)
  *  \return new string which is the concatenation of s1 and s2.
  *
  */
-record_t* _lcmaps_concat_strings(const record_t* s1, const record_t* s2,
+static record_t* _lcmaps_concat_strings(const record_t* s1, const record_t* s2,
                           const char* extra)
 {
-  int len       = strlen(s1->string);
-  int len_extra = extra ? strlen(extra) : 0;
+  size_t len       = strlen(s1->string);
+  size_t len_extra = extra ? strlen(extra) : 0;
 
   record_t* record = (record_t *)malloc(sizeof(record_t));
-
-  if (!(record->string = (char *)malloc(len + len_extra + strlen(s2->string)+1))) {
-    lcmaps_warning(PDL_ERROR, "out of memory");
+  if ( record==NULL ||
+       (record->string =
+	    (char *)malloc(len + len_extra + strlen(s2->string)+1))==NULL ) {
+    lcmaps_pdl_warning(PDL_ERROR, "out of memory");
     return NULL;
   }
 
@@ -537,10 +574,9 @@ record_t* _lcmaps_concat_strings(const record_t* s1, const record_t* s2,
   return record;
 }
 
-
 /*!
- *  Concatenate two strings. The orginal two strings are freed. When
- *  the concatenation fails, the origial strings are still freed. The
+ *  Concatenate two strings. The original two strings are freed. When
+ *  the concatenation fails, the original strings are still freed. The
  *  actual concatenation is done by _lcmaps_concat_strings().
  *
  *  \param  s1  First string.
@@ -573,8 +609,11 @@ record_t* lcmaps_concat_strings_with_space(record_t* s1, record_t* s2)
      *  Since there is nothing to concatenate, we simply copy the
      *  first string (s1) to the result.
      */
-    r = (record_t*)malloc(sizeof(r));
-    memcpy(r, s1, sizeof(r));
+    if ( (r = (record_t*)malloc(sizeof(record_t)))==NULL)   {
+	lcmaps_pdl_warning(PDL_ERROR, "out of memory");
+	return NULL;
+    }
+    memcpy(r, s1, sizeof(record_t));
   }
 
   return r;
@@ -691,7 +730,7 @@ char* lcmaps_pdl_next_plugin(plugin_status_t status)
   if (state) {
     string=strdup(state);
     if (string==NULL)   {
-	lcmaps_warning(PDL_ERROR, "Out of memory.");
+	lcmaps_pdl_warning(PDL_ERROR, "Out of memory.");
 	return NULL;
     }
     tmp = strchr(string,' ');
@@ -755,15 +794,16 @@ void lcmaps_free_resources(void)
  *  Display a warning message.
  *
  *  \param error Severity of the error.
- *  \param s     The text string.
- *  \param ...   Additional values; much like printf(char *, ...);
+ *  \param s     The text string followed by additional values; much like
+ *  printf(char *, ...);
  *
  */
-void lcmaps_warning(pdl_error_t error, const char* s, ...)
+void lcmaps_pdl_warning(pdl_error_t error, const char* s, ...)
 {
-  static char* level = NULL;
-  char buf[2048];
+  static const char* level = NULL;
+  char buf[WARNBUFLEN];
   int res;
+  size_t len;
 
   va_list args;
 
@@ -780,21 +820,40 @@ void lcmaps_warning(pdl_error_t error, const char* s, ...)
    *  Prepend a standard text before the message. This makes reading
    *  the errors/warnings a little easier.
    */
-  res = sprintf(buf, "%s:%d: [%s] ", script_name, lineno, level);
+  res = snprintf(buf, WARNBUFLEN, "%s:%d: [%s] ", script_name, lineno, level);
+  if (res<0)	{
+      lcmaps_log(LOG_ERR, "Cannot log message: %s\n",strerror(errno));
+      len=0;
+  } else    {
+      len=(size_t)res;
+      if (len>WARNBUFLEN-2)	{
+	  lcmaps_log(LOG_ERR, "Log message is too long\n");
+	  return;
+      }
+  }
 
   /*  Print (format) the actual message.  */
   va_start(args, s);
-  res += vsnprintf(buf+res, sizeof(buf)-res-2, s, args);
+  res = vsnprintf(buf+len, WARNBUFLEN-2-len, s, args);
   va_end(args);
+  if (res<0)	{
+      lcmaps_log(LOG_ERR, "Cannot log message: %s\n",strerror(errno));
+      return;
+  }
+  len+=(size_t)res;
+  if (len>WARNBUFLEN-2) {
+      lcmaps_log(LOG_ERR, "Log message is too long\n");
+      return;
+  }
 
   /*
    *  The string needs to end in a new line and of course strings are
    *  termintated by the end of string character. Make sure it happens
    *  and respect the buffer size!
    */
-  buf[res<sizeof(buf)-1 ? res++ : sizeof(buf)-2] = '\n';
-  buf[res<sizeof(buf)   ? res   : sizeof(buf)-1] = '\0';
+  buf[len<WARNBUFLEN-1 ? len++ : WARNBUFLEN-2] = '\n';
+  buf[len<WARNBUFLEN   ? len   : WARNBUFLEN-1] = '\0';
 
-  lcmaps_log(LOG_ERR, buf);
+  lcmaps_log(LOG_ERR, "%s", buf);
 }
 
