@@ -47,35 +47,36 @@
  *
  *
  *  \author  G.M. Venekamp  (venekamp@nikhef.nl)
- *  \version $Revision: 15701 $
- *  \date    $Date: 2011-12-13 12:06:42 +0100 (Tue, 13 Dec 2011) $
+ *  \version $Revision: 18216 $
+ *  \date    $Date: 2015-02-02 11:46:42 +0100 (Mon, 02 Feb 2015) $
  *
  */
 
 
-#include <stdarg.h>
+/* Needed for NULL */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "lcmaps_log.h"
+#include "pdl.h"
 #include "pdl_rule.h"
 #include "pdl_policy.h"
 #include "pdl_variable.h"
 
-rule_t *top_rule  = NULL;
-rule_t *last_rule = NULL;
+static rule_t *top_rule  = NULL;
+static rule_t *last_rule = NULL;
 static BOOL add_new_rules = TRUE;
 
-rule_t* _lcmaps_add_rule(record_t* state, record_t* true_branch, record_t* false_branch);
+static rule_t* _lcmaps_add_rule(record_t* state, record_t* true_branch, record_t* false_branch);
 /* const rule_t* lcmaps_find_state(const rule_t* rule, const char* state); */
-recursion_t lcmaps_has_recursion(rule_t* rule, unsigned int* list, unsigned int depth, unsigned int* seen_rules);
-int lcmaps_find_insert_position(unsigned int* list, unsigned int my_rule_number, unsigned int high);
-unsigned int lcmaps_rule_number(rule_t* rule);
-BOOL lcmaps_make_list(unsigned int* new_list, unsigned int* list, unsigned int my_rule_number, unsigned int depth);
-unsigned int lcmaps_count_rules(rule_t* rule);
-void lcmaps_update_list(unsigned int* rules, unsigned int rule);
-rule_t* lcmaps_get_rule_number(unsigned int rule_num);
+static recursion_t lcmaps_has_recursion(rule_t* rule, unsigned int* list, unsigned int depth, unsigned int* seen_rules);
+static unsigned int lcmaps_find_insert_position(unsigned int* list, unsigned int my_rule_number, unsigned int high);
+static unsigned int lcmaps_rule_number(rule_t* rule);
+static BOOL lcmaps_make_list(unsigned int* new_list, unsigned int* list, unsigned int my_rule_number, unsigned int depth);
+static unsigned int lcmaps_count_rules(rule_t* rule);
+static void lcmaps_update_list(unsigned int* rules, unsigned int rule);
+static rule_t* lcmaps_get_rule_number(unsigned int rule_num);
 
 
 /*!
@@ -167,26 +168,26 @@ rule_t* lcmaps_add_rule(record_t* state, record_t* true_branch,
  *  \return TRUE if the rule has been added successfully, FALSE otherwise.
  *
  */
-rule_t* _lcmaps_add_rule(record_t* state, record_t* true_branch,
+static rule_t* _lcmaps_add_rule(record_t* state, record_t* true_branch,
                   record_t* false_branch)
 {
   rule_t* rule     = NULL;
   policy_t* policy = NULL;
 
   if ((policy = lcmaps_find_policy(state->string))) {
-    lcmaps_warning(PDL_ERROR, "Left hand side of a rule cannot be a policy; see also line %d.", policy->lineno);
+    lcmaps_pdl_warning(PDL_ERROR, "Left hand side of a rule cannot be a policy; see also line %d.", policy->lineno);
     return 0;
   }
 
   /* cast away constness. */
   if ((rule = lcmaps_find_state(top_rule, state->string))) {
-    lcmaps_warning(PDL_ERROR, "State '%s' is already in use. See line %d.\n", state->string, rule->lineno);
+    lcmaps_pdl_warning(PDL_ERROR, "State '%s' is already in use. See line %d.\n", state->string, rule->lineno);
     return 0;
   }
 
   if ((true_branch && (policy = lcmaps_find_policy(true_branch->string)))
       || (false_branch && (policy = lcmaps_find_policy(false_branch->string))))
-    lcmaps_warning(PDL_ERROR, "Rule contians reference to a policy. This is currently not supported.");
+    lcmaps_pdl_warning(PDL_ERROR, "Rule contians reference to a policy. This is currently not supported.");
   /*  Do not return 0 if a warning has been issued. Ignore the fact that  *
    *  a policy rule has been used. Use its name as a module instead.      */
 
@@ -196,7 +197,7 @@ rule_t* _lcmaps_add_rule(record_t* state, record_t* true_branch,
     return 0;
 
   if (!(rule = (rule_t *)malloc(sizeof(rule_t)))) {
-    lcmaps_warning(PDL_ERROR, "out of memory.");
+    lcmaps_pdl_warning(PDL_ERROR, "out of memory.");
     return 0;
   }
 
@@ -249,20 +250,20 @@ BOOL lcmaps_check_rule_for_recursion(rule_t* rule)
   unsigned int num_rules = lcmaps_count_rules(rule);
   unsigned int *rules = NULL;
   recursion_t rc;
-  int i = 0;
+  unsigned int i = 0;
 
-  rules = (unsigned int*)calloc((num_rules+1), sizeof(*rules));
+  rules = (unsigned int*)calloc((size_t)(num_rules+1), sizeof(*rules));
 
   top_rule = rule;
 
-  rc = lcmaps_has_recursion(rule, 0, 0, rules);
+  rc = lcmaps_has_recursion(rule, NULL, 0, rules);
 
   if (rules[0] != num_rules) {
     int j=1;
     for (i=1; i<=num_rules; i++) {
       if (rules[j] != i) {
         lineno = lcmaps_get_rule_number(i-1)->lineno;
-        lcmaps_warning(PDL_WARNING, "rule is not part of the chain.");
+        lcmaps_pdl_warning(PDL_WARNING, "rule is not part of the chain.");
       } else
         j++;
     }
@@ -281,7 +282,7 @@ BOOL lcmaps_check_rule_for_recursion(rule_t* rule)
  *  \return Number of counted rules.
  *
  */
-unsigned int lcmaps_count_rules(rule_t* rule)
+static unsigned int lcmaps_count_rules(rule_t* rule)
 {
   unsigned int c=0;
 
@@ -302,7 +303,7 @@ unsigned int lcmaps_count_rules(rule_t* rule)
  *          rule cannot be found.
  *
  */
-rule_t* lcmaps_get_rule_number(unsigned int rule_num)
+static rule_t* lcmaps_get_rule_number(unsigned int rule_num)
 {
   unsigned int i = 0;
   rule_t* r = NULL;
@@ -342,7 +343,7 @@ rule_t* lcmaps_get_rule_number(unsigned int rule_num)
  *          has been reported.
  *
  */
-recursion_t lcmaps_has_recursion(rule_t* rule, unsigned int* list, unsigned int depth,
+static recursion_t lcmaps_has_recursion(rule_t* rule, unsigned int* list, unsigned int depth,
                           unsigned int* seen_rules)
 {
   unsigned int rule_num = 0;
@@ -396,9 +397,9 @@ recursion_t lcmaps_has_recursion(rule_t* rule, unsigned int* list, unsigned int
       if ((rcl&RECURSION) && !(rcl&RECURSION_HANDLED)) {
   lineno = rule->lineno;
   if (rule->false_branch)
-    lcmaps_warning(PDL_ERROR, "rule  %s -> %s | %s causes infinite loop on true transition %s.", rule->state, rule->true_branch, rule->false_branch, rule->true_branch);
+    lcmaps_pdl_warning(PDL_ERROR, "rule  %s -> %s | %s causes infinite loop on true transition %s.", rule->state, rule->true_branch, rule->false_branch, rule->true_branch);
   else
-    lcmaps_warning(PDL_ERROR, "rule  %s -> %s causes infinite loop on transition %s.", rule->state, rule->true_branch, rule->true_branch);
+    lcmaps_pdl_warning(PDL_ERROR, "rule  %s -> %s causes infinite loop on transition %s.", rule->state, rule->true_branch, rule->true_branch);
 
   /*
    *  Set the handled bit to indicate that the current recursion
@@ -415,9 +416,9 @@ recursion_t lcmaps_has_recursion(rule_t* rule, unsigned int* list, unsigned int
       if ((rcr&RECURSION) && !(rcr&RECURSION_HANDLED)) {
   lineno = rule->lineno;
   if (rule->true_branch)
-    lcmaps_warning(PDL_ERROR, "rule  %s -> %s | %s causes infinite loop on false transition %s.", rule->state, rule->true_branch, rule->false_branch, rule->false_branch);
+    lcmaps_pdl_warning(PDL_ERROR, "rule  %s -> %s | %s causes infinite loop on false transition %s.", rule->state, rule->true_branch, rule->false_branch, rule->false_branch);
   else
-    lcmaps_warning(PDL_ERROR, "rule ~%s -> %s causes infinite loop on transition %s.", rule->state, rule->false_branch, rule->false_branch);
+    lcmaps_pdl_warning(PDL_ERROR, "rule ~%s -> %s causes infinite loop on transition %s.", rule->state, rule->false_branch, rule->false_branch);
 
   rcr |= RECURSION_HANDLED;
       }
@@ -449,9 +450,9 @@ recursion_t lcmaps_has_recursion(rule_t* rule, unsigned int* list, unsigned int
  *  \return Position of the rule in the current policy.
  *
  */
-unsigned int lcmaps_rule_number(rule_t* rule)
+static unsigned int lcmaps_rule_number(rule_t* rule)
 {
-  int n = 0;
+  unsigned int n = 0;
   rule_t* t = NULL;
 
   for (n=0, t=top_rule; t; ++n, t=t->next) {
@@ -477,10 +478,10 @@ unsigned int lcmaps_rule_number(rule_t* rule)
  *  \param rule   rule to insert.
  *
  */
-void lcmaps_update_list(unsigned int* rules, unsigned int rule)
+static void lcmaps_update_list(unsigned int* rules, unsigned int rule)
 {
   /* Rules in the list start at 1. */
-  int p = 1 + lcmaps_find_insert_position(rules+1, rule, rules[0]);
+  unsigned int p = 1 + lcmaps_find_insert_position(rules+1, rule, rules[0]);
 
   /* Same here, rules in the list start at 1. */
   rule++;
@@ -506,15 +507,15 @@ void lcmaps_update_list(unsigned int* rules, unsigned int rule)
  *  is a binary search.
  *
  *  \param list         List of sorted numbers.
- *  \param lcmaps_rule_number  Number to be inserted.
+ *  \param my_rule_number  Number to be inserted.
  *  \param high         Element number of last element in the list.
  *
  *  \return Position of insertion.
  *
  */
-int lcmaps_find_insert_position(unsigned int* list, unsigned int my_rule_number, unsigned int high)
+static unsigned int lcmaps_find_insert_position(unsigned int* list, unsigned int my_rule_number, unsigned int high)
 {
-  int low=0, mid;
+  unsigned int low=0, mid;
 
   /*
    *  low and high specify the current part of the list to be
@@ -544,32 +545,38 @@ int lcmaps_find_insert_position(unsigned int* list, unsigned int my_rule_number,
  *
  *  \param new_list     New list after sorted insertion of new element.
  *  \param list         Old list.
- *  \param lcmaps_rule_number  Number to be inserted into the list.
+ *  \param my_rule_number  Number to be inserted into the list.
  *  \param depth        Current depth of the tree. It is used to detmine
  *                      the number of elements of the list.
  *
  *  \return TRUE if element has been added, FALSE otherwise
  *
  */
-BOOL lcmaps_make_list(unsigned int* new_list, unsigned int* list, unsigned int my_rule_number,
+static BOOL lcmaps_make_list(unsigned int* new_list, unsigned int* list, unsigned int my_rule_number,
                       unsigned int depth)
 {
-  int insert = 0;
+  unsigned int insert = 0;
 
   if (!list) {
     *new_list = my_rule_number;
     return TRUE;
   }
 
-  insert = lcmaps_find_insert_position(list, my_rule_number, depth-1);
+  /* depth should be at least 1, but protect in any case */
+  insert = lcmaps_find_insert_position(list, my_rule_number, depth>0 ? depth-1 : 0);
 
   if ((insert>0) && (list[insert-1] == my_rule_number))
     return FALSE;
 
-  memcpy(new_list, list, insert*sizeof(int));
-  if ((depth-insert-1)>0)
+  /* copy part before insert from list to new_list */
+  if (insert>0)
+    memcpy(new_list, list, insert*sizeof(int));
+
+  /* copy part after insert from list to new_list */
+  if (depth > insert+1)
     memcpy(new_list+insert+1, list+insert, (depth-insert-1)*sizeof(int));
 
+  /* add new member at insert */
   new_list[insert] = my_rule_number;
 
   return TRUE;
