@@ -1,24 +1,23 @@
-/* A Bison parser, made by GNU Bison 2.3.  */
+
+/* A Bison parser, made by GNU Bison 2.4.1.  */
 
 /* Skeleton implementation for Bison's Yacc-like parsers in C
-
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
+   
+      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
    Free Software Foundation, Inc.
-
-   This program is free software; you can redistribute it and/or modify
+   
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -29,7 +28,7 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-
+   
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
@@ -47,7 +46,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.3"
+#define YYBISON_VERSION "2.4.1"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -55,11 +54,67 @@
 /* Pure parsers.  */
 #define YYPURE 0
 
+/* Push parsers.  */
+#define YYPUSH 0
+
+/* Pull parsers.  */
+#define YYPULL 1
+
 /* Using locations.  */
 #define YYLSP_NEEDED 0
 
 
 
+/* Copy the first part of user declarations.  */
+
+/* Line 189 of yacc.c  */
+#line 24 "../../src/evaluationmanager/pdl_yacc.y"
+
+/*
+ *   Copyright (c) 2002 EU DataGrid        http://www.eu-datagrid.org/
+ *
+ *
+ *   Author:  G.M. Venekamp  (venekamp@nikhef.nl)
+ *
+ *
+ *   This software is distributed under a BSD-style open source
+ *   licence. For a complete description of the licence take a look
+ *   at: http://eu-datagrid.web.cern.ch/eu-datagrid/license.html
+ *
+ */
+
+
+#include <stdlib.h>
+#include <string.h>
+
+#include "evaluationmanager/pdl.h"
+#include "evaluationmanager/pdl_variable.h"
+#include "evaluationmanager/pdl_policy.h"
+#include "evaluationmanager/pdl_rule.h"
+
+
+/* Line 189 of yacc.c  */
+#line 98 "evaluationmanager/pdl_yacc.c"
+
+/* Enabling traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 0
+#endif
+
+/* Enabling verbose error messages.  */
+#ifdef YYERROR_VERBOSE
+# undef YYERROR_VERBOSE
+# define YYERROR_VERBOSE 1
+#else
+# define YYERROR_VERBOSE 0
+#endif
+
+/* Enabling the token table.  */
+#ifndef YYTOKEN_TABLE
+# define YYTOKEN_TABLE 1
+#endif
+
+
 /* Tokens.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
@@ -89,73 +144,32 @@
 
 
 
-/* Copy the first part of user declarations.  */
-#line 24 "pdl_yacc.y"
-
-/*
- *   Copyright (c) 2002 EU DataGrid        http://www.eu-datagrid.org/
- *
- *
- *   Author:  G.M. Venekamp  (venekamp@nikhef.nl)
- *
- *
- *   This software is distributed under a BSD-style open source
- *   licence. For a complete description of the licence take a look
- *   at: http://eu-datagrid.web.cern.ch/eu-datagrid/license.html
- *
- */
-
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
-#include "pdl.h"
-#include "pdl_variable.h"
-#include "pdl_policy.h"
-#include "pdl_rule.h"
-
-
-/* Enabling traces.  */
-#ifndef YYDEBUG
-# define YYDEBUG 0
-#endif
-
-/* Enabling verbose error messages.  */
-#ifdef YYERROR_VERBOSE
-# undef YYERROR_VERBOSE
-# define YYERROR_VERBOSE 1
-#else
-# define YYERROR_VERBOSE 0
-#endif
-
-/* Enabling the token table.  */
-#ifndef YYTOKEN_TABLE
-# define YYTOKEN_TABLE 1
-#endif
-
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 49 "pdl_yacc.y"
 {
+
+/* Line 214 of yacc.c  */
+#line 48 "../../src/evaluationmanager/pdl_yacc.y"
+
   record_t* record;
   rule_t*   rule;
-}
-/* Line 193 of yacc.c.  */
-#line 146 "evaluationmanager/pdl_yacc.c"
-	YYSTYPE;
+
+
+
+/* Line 214 of yacc.c  */
+#line 161 "evaluationmanager/pdl_yacc.c"
+} YYSTYPE;
+# define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
-# define YYSTYPE_IS_TRIVIAL 1
 #endif
 
 
-
 /* Copy the second part of user declarations.  */
 
 
-/* Line 216 of yacc.c.  */
-#line 159 "evaluationmanager/pdl_yacc.c"
+/* Line 264 of yacc.c  */
+#line 173 "evaluationmanager/pdl_yacc.c"
 
 #ifdef short
 # undef short
@@ -230,14 +244,14 @@ typedef short int yytype_int16;
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static int
-YYID (int i)
+YYID (int yyi)
 #else
 static int
-YYID (i)
-    int i;
+YYID (yyi)
+    int yyi;
 #endif
 {
-  return i;
+  return yyi;
 }
 #endif
 
@@ -318,9 +332,9 @@ void free (void *); /* INFRINGES ON USER NAME SPACE */
 /* A type that is properly aligned for any stack member.  */
 union yyalloc
 {
-  yytype_int16 yyss;
-  YYSTYPE yyvs;
-  };
+  yytype_int16 yyss_alloc;
+  YYSTYPE yyvs_alloc;
+};
 
 /* The size of the maximum gap between one aligned stack and the next.  */
 # define YYSTACK_GAP_MAXIMUM (sizeof (union yyalloc) - 1)
@@ -354,12 +368,12 @@ union yyalloc
    elements in the stack, and YYPTR gives the new location of the
    stack.  Advance YYPTR to a properly aligned location for the next
    stack.  */
-# define YYSTACK_RELOCATE(Stack)					\
+# define YYSTACK_RELOCATE(Stack_alloc, Stack)				\
     do									\
       {									\
 	YYSIZE_T yynewbytes;						\
-	YYCOPY (&yyptr->Stack, Stack, yysize);				\
-	Stack = &yyptr->Stack;						\
+	YYCOPY (&yyptr->Stack_alloc, Stack, yysize);			\
+	Stack = &yyptr->Stack_alloc;					\
 	yynewbytes = yystacksize * sizeof (*Stack) + YYSTACK_GAP_MAXIMUM; \
 	yyptr += yynewbytes / sizeof (*yyptr);				\
       }									\
@@ -445,9 +459,9 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint8 yyrline[] =
 {
-       0,    64,    64,    65,    66,    67,    70,    71,    74,    75,
-      77,    78,    81,    82,    85,    86,    87,    91,    97,    98,
-     101,   102,   103
+       0,    63,    63,    64,    65,    66,    69,    70,    73,    74,
+      76,    77,    80,    81,    84,    85,    86,    90,    96,    97,
+     100,   101,   102
 };
 #endif
 
@@ -735,17 +749,20 @@ yy_symbol_print (yyoutput, yytype, yyvaluep)
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static void
-yy_stack_print (yytype_int16 *bottom, yytype_int16 *top)
+yy_stack_print (yytype_int16 *yybottom, yytype_int16 *yytop)
 #else
 static void
-yy_stack_print (bottom, top)
-    yytype_int16 *bottom;
-    yytype_int16 *top;
+yy_stack_print (yybottom, yytop)
+    yytype_int16 *yybottom;
+    yytype_int16 *yytop;
 #endif
 {
   YYFPRINTF (stderr, "Stack now");
-  for (; bottom <= top; ++bottom)
-    YYFPRINTF (stderr, " %d", *bottom);
+  for (; yybottom <= yytop; yybottom++)
+    {
+      int yybot = *yybottom;
+      YYFPRINTF (stderr, " %d", yybot);
+    }
   YYFPRINTF (stderr, "\n");
 }
 
@@ -779,11 +796,11 @@ yy_reduce_print (yyvsp, yyrule)
   /* The symbols being reduced.  */
   for (yyi = 0; yyi < yynrhs; yyi++)
     {
-      fprintf (stderr, "   $%d = ", yyi + 1);
+      YYFPRINTF (stderr, "   $%d = ", yyi + 1);
       yy_symbol_print (stderr, yyrhs[yyprhs[yyrule] + yyi],
 		       &(yyvsp[(yyi + 1) - (yynrhs)])
 		       		       );
-      fprintf (stderr, "\n");
+      YYFPRINTF (stderr, "\n");
     }
 }
 
@@ -1063,10 +1080,8 @@ yydestruct (yymsg, yytype, yyvaluep)
 	break;
     }
 }
-
 
 /* Prevent warnings from -Wmissing-prototypes.  */
-
 #ifdef YYPARSE_PARAM
 #if defined __STDC__ || defined __cplusplus
 int yyparse (void *YYPARSE_PARAM);
@@ -1082,11 +1097,10 @@ int yyparse ();
 #endif /* ! YYPARSE_PARAM */
 
 
-
-/* The look-ahead symbol.  */
+/* The lookahead symbol.  */
 int yychar;
 
-/* The semantic value of the look-ahead symbol.  */
+/* The semantic value of the lookahead symbol.  */
 YYSTYPE yylval;
 
 /* Number of syntax errors so far.  */
@@ -1094,9 +1108,9 @@ int yynerrs;
 
 
 
-/*----------.
-| yyparse.  |
-`----------*/
+/*-------------------------.
+| yyparse or yypush_parse.  |
+`-------------------------*/
 
 #ifdef YYPARSE_PARAM
 #if (defined __STDC__ || defined __C99__FUNC__ \
@@ -1120,14 +1134,39 @@ yyparse ()
 #endif
 #endif
 {
-  
-  int yystate;
+
+
+    int yystate;
+    /* Number of tokens to shift before error messages enabled.  */
+    int yyerrstatus;
+
+    /* The stacks and their tools:
+       `yyss': related to states.
+       `yyvs': related to semantic values.
+
+       Refer to the stacks thru separate pointers, to allow yyoverflow
+       to reallocate them elsewhere.  */
+
+    /* The state stack.  */
+    yytype_int16 yyssa[YYINITDEPTH];
+    yytype_int16 *yyss;
+    yytype_int16 *yyssp;
+
+    /* The semantic value stack.  */
+    YYSTYPE yyvsa[YYINITDEPTH];
+    YYSTYPE *yyvs;
+    YYSTYPE *yyvsp;
+
+    YYSIZE_T yystacksize;
+
   int yyn;
   int yyresult;
-  /* Number of tokens to shift before error messages enabled.  */
-  int yyerrstatus;
-  /* Look-ahead token as an internal (translated) token number.  */
-  int yytoken = 0;
+  /* Lookahead token as an internal (translated) token number.  */
+  int yytoken;
+  /* The variables used to return semantic value and location from the
+     action routines.  */
+  YYSTYPE yyval;
+
 #if YYERROR_VERBOSE
   /* Buffer for error messages, and its allocated size.  */
   char yymsgbuf[128];
@@ -1135,51 +1174,28 @@ yyparse ()
   YYSIZE_T yymsg_alloc = sizeof yymsgbuf;
 #endif
 
-  /* Three stacks and their tools:
-     `yyss': related to states,
-     `yyvs': related to semantic values,
-     `yyls': related to locations.
-
-     Refer to the stacks thru separate pointers, to allow yyoverflow
-     to reallocate them elsewhere.  */
-
-  /* The state stack.  */
-  yytype_int16 yyssa[YYINITDEPTH];
-  yytype_int16 *yyss = yyssa;
-  yytype_int16 *yyssp;
-
-  /* The semantic value stack.  */
-  YYSTYPE yyvsa[YYINITDEPTH];
-  YYSTYPE *yyvs = yyvsa;
-  YYSTYPE *yyvsp;
-
-
-
 #define YYPOPSTACK(N)   (yyvsp -= (N), yyssp -= (N))
 
-  YYSIZE_T yystacksize = YYINITDEPTH;
-
-  /* The variables used to return semantic value and location from the
-     action routines.  */
-  YYSTYPE yyval;
-
-
   /* The number of symbols on the RHS of the reduced rule.
      Keep to zero when no symbol should be popped.  */
   int yylen = 0;
 
+  yytoken = 0;
+  yyss = yyssa;
+  yyvs = yyvsa;
+  yystacksize = YYINITDEPTH;
+
   YYDPRINTF ((stderr, "Starting parse\n"));
 
   yystate = 0;
   yyerrstatus = 0;
   yynerrs = 0;
-  yychar = YYEMPTY;		/* Cause a token to be read.  */
+  yychar = YYEMPTY; /* Cause a token to be read.  */
 
   /* Initialize stack pointers.
      Waste one element of value and location stack
      so that they stay on the same level as the state stack.
      The wasted elements are never initialized.  */
-
   yyssp = yyss;
   yyvsp = yyvs;
 
@@ -1209,7 +1225,6 @@ yyparse ()
 	YYSTYPE *yyvs1 = yyvs;
 	yytype_int16 *yyss1 = yyss;
 
-
 	/* Each stack pointer address is followed by the size of the
 	   data in use in that stack, in bytes.  This used to be a
 	   conditional around just the two extra args, but that might
@@ -1217,7 +1232,6 @@ yyparse ()
 	yyoverflow (YY_("memory exhausted"),
 		    &yyss1, yysize * sizeof (*yyssp),
 		    &yyvs1, yysize * sizeof (*yyvsp),
-
 		    &yystacksize);
 
 	yyss = yyss1;
@@ -1240,9 +1254,8 @@ yyparse ()
 	  (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
 	if (! yyptr)
 	  goto yyexhaustedlab;
-	YYSTACK_RELOCATE (yyss);
-	YYSTACK_RELOCATE (yyvs);
-
+	YYSTACK_RELOCATE (yyss_alloc, yyss);
+	YYSTACK_RELOCATE (yyvs_alloc, yyvs);
 #  undef YYSTACK_RELOCATE
 	if (yyss1 != yyssa)
 	  YYSTACK_FREE (yyss1);
@@ -1253,7 +1266,6 @@ yyparse ()
       yyssp = yyss + yysize - 1;
       yyvsp = yyvs + yysize - 1;
 
-
       YYDPRINTF ((stderr, "Stack size increased to %lu\n",
 		  (unsigned long int) yystacksize));
 
@@ -1263,6 +1275,9 @@ yyparse ()
 
   YYDPRINTF ((stderr, "Entering state %d\n", yystate));
 
+  if (yystate == YYFINAL)
+    YYACCEPT;
+
   goto yybackup;
 
 /*-----------.
@@ -1271,16 +1286,16 @@ yyparse ()
 yybackup:
 
   /* Do appropriate processing given the current state.  Read a
-     look-ahead token if we need one and don't already have one.  */
+     lookahead token if we need one and don't already have one.  */
 
-  /* First try to decide what to do without reference to look-ahead token.  */
+  /* First try to decide what to do without reference to lookahead token.  */
   yyn = yypact[yystate];
   if (yyn == YYPACT_NINF)
     goto yydefault;
 
-  /* Not known => get a look-ahead token if don't already have one.  */
+  /* Not known => get a lookahead token if don't already have one.  */
 
-  /* YYCHAR is either YYEMPTY or YYEOF or a valid look-ahead symbol.  */
+  /* YYCHAR is either YYEMPTY or YYEOF or a valid lookahead symbol.  */
   if (yychar == YYEMPTY)
     {
       YYDPRINTF ((stderr, "Reading a token: "));
@@ -1312,20 +1327,16 @@ yybackup:
       goto yyreduce;
     }
 
-  if (yyn == YYFINAL)
-    YYACCEPT;
-
   /* Count tokens shifted since error; after three, turn off error
      status.  */
   if (yyerrstatus)
     yyerrstatus--;
 
-  /* Shift the look-ahead token.  */
+  /* Shift the lookahead token.  */
   YY_SYMBOL_PRINT ("Shifting", yytoken, &yylval, &yylloc);
 
-  /* Discard the shifted token unless it is eof.  */
-  if (yychar != YYEOF)
-    yychar = YYEMPTY;
+  /* Discard the shifted token.  */
+  yychar = YYEMPTY;
 
   yystate = yyn;
   *++yyvsp = yylval;
@@ -1365,100 +1376,135 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 64 "pdl_yacc.y"
-    { lcmaps_warning(PDL_WARNING, "Config file is empty."); }
+
+/* Line 1455 of yacc.c  */
+#line 63 "../../src/evaluationmanager/pdl_yacc.y"
+    { lcmaps_pdl_warning(PDL_WARNING, "Config file is empty."); }
     break;
 
   case 5:
-#line 67 "pdl_yacc.y"
-    { lcmaps_warning(PDL_ERROR, "Config file contains no policy rules."); }
+
+/* Line 1455 of yacc.c  */
+#line 66 "../../src/evaluationmanager/pdl_yacc.y"
+    { lcmaps_pdl_warning(PDL_ERROR, "Config file contains no policy rules."); }
     break;
 
   case 8:
-#line 74 "pdl_yacc.y"
+
+/* Line 1455 of yacc.c  */
+#line 73 "../../src/evaluationmanager/pdl_yacc.y"
     { lcmaps_set_path((yyvsp[(3) - (3)].record));         }
     break;
 
   case 9:
-#line 75 "pdl_yacc.y"
-    { lcmaps_warning(PDL_INFO, "Empty value assigned to path: ignoring this line.");
+
+/* Line 1455 of yacc.c  */
+#line 74 "../../src/evaluationmanager/pdl_yacc.y"
+    { lcmaps_pdl_warning(PDL_INFO, "Empty value assigned to path: ignoring this line.");
                                             lcmaps_set_path(NULL);       }
     break;
 
   case 10:
-#line 77 "pdl_yacc.y"
+
+/* Line 1455 of yacc.c  */
+#line 76 "../../src/evaluationmanager/pdl_yacc.y"
     { lcmaps_add_variable((yyvsp[(1) - (3)].record), (yyvsp[(3) - (3)].record)); }
     break;
 
   case 11:
-#line 78 "pdl_yacc.y"
+
+/* Line 1455 of yacc.c  */
+#line 77 "../../src/evaluationmanager/pdl_yacc.y"
     { lcmaps_add_variable((yyvsp[(1) - (3)].record), (yyvsp[(3) - (3)].record)); }
     break;
 
   case 12:
-#line 81 "pdl_yacc.y"
+
+/* Line 1455 of yacc.c  */
+#line 80 "../../src/evaluationmanager/pdl_yacc.y"
     { (yyval.record) = (yyvsp[(1) - (1)].record);                     }
     break;
 
   case 13:
-#line 82 "pdl_yacc.y"
+
+/* Line 1455 of yacc.c  */
+#line 81 "../../src/evaluationmanager/pdl_yacc.y"
     { (yyval.record) = lcmaps_concat_strings_with_space((yyvsp[(1) - (2)].record), (yyvsp[(2) - (2)].record)); }
     break;
 
   case 14:
-#line 85 "pdl_yacc.y"
+
+/* Line 1455 of yacc.c  */
+#line 84 "../../src/evaluationmanager/pdl_yacc.y"
     { lcmaps_add_policy((yyvsp[(1) - (2)].record), (yyvsp[(2) - (2)].rule)); }
     break;
 
   case 15:
-#line 86 "pdl_yacc.y"
+
+/* Line 1455 of yacc.c  */
+#line 85 "../../src/evaluationmanager/pdl_yacc.y"
     { lcmaps_add_policy((yyvsp[(2) - (3)].record), (yyvsp[(3) - (3)].rule)); }
     break;
 
   case 16:
-#line 87 "pdl_yacc.y"
-    { lcmaps_warning(PDL_WARNING, "expecting rule definitions.");
-                                            lcmaps_warning(PDL_WARNING, "no rules specified for policy: '%s' at line %d.", (yyvsp[(2) - (2)].record)->string, (yyvsp[(2) - (2)].record)->lineno);
+
+/* Line 1455 of yacc.c  */
+#line 86 "../../src/evaluationmanager/pdl_yacc.y"
+    { lcmaps_pdl_warning(PDL_WARNING, "expecting rule definitions.");
+                                            lcmaps_pdl_warning(PDL_WARNING, "no rules specified for policy: '%s' at line %d.", (yyvsp[(2) - (2)].record)->string, (yyvsp[(2) - (2)].record)->lineno);
                                             lcmaps_remove_policy((yyvsp[(2) - (2)].record));
                                           }
     break;
 
   case 17:
-#line 91 "pdl_yacc.y"
-    { lcmaps_warning(PDL_WARNING, "expecting rule definitions.");
-                                            lcmaps_warning(PDL_WARNING, "no rules specified for policy: '%s' at line %d.", (yyvsp[(1) - (1)].record)->string, (yyvsp[(1) - (1)].record)->lineno);
+
+/* Line 1455 of yacc.c  */
+#line 90 "../../src/evaluationmanager/pdl_yacc.y"
+    { lcmaps_pdl_warning(PDL_WARNING, "expecting rule definitions.");
+                                            lcmaps_pdl_warning(PDL_WARNING, "no rules specified for policy: '%s' at line %d.", (yyvsp[(1) - (1)].record)->string, (yyvsp[(1) - (1)].record)->lineno);
                                             lcmaps_remove_policy((yyvsp[(1) - (1)].record));
 	                                  }
     break;
 
   case 18:
-#line 97 "pdl_yacc.y"
+
+/* Line 1455 of yacc.c  */
+#line 96 "../../src/evaluationmanager/pdl_yacc.y"
     { (yyval.rule) = (yyvsp[(1) - (1)].rule); }
     break;
 
   case 19:
-#line 98 "pdl_yacc.y"
+
+/* Line 1455 of yacc.c  */
+#line 97 "../../src/evaluationmanager/pdl_yacc.y"
     { (yyval.rule) = (yyvsp[(1) - (2)].rule); }
     break;
 
   case 20:
-#line 101 "pdl_yacc.y"
+
+/* Line 1455 of yacc.c  */
+#line 100 "../../src/evaluationmanager/pdl_yacc.y"
     { (yyval.rule) = lcmaps_add_rule((yyvsp[(1) - (3)].record), (yyvsp[(3) - (3)].record),  0); }
     break;
 
   case 21:
-#line 102 "pdl_yacc.y"
+
+/* Line 1455 of yacc.c  */
+#line 101 "../../src/evaluationmanager/pdl_yacc.y"
     { (yyval.rule) = lcmaps_add_rule((yyvsp[(1) - (5)].record), (yyvsp[(3) - (5)].record), (yyvsp[(5) - (5)].record)); }
     break;
 
   case 22:
-#line 103 "pdl_yacc.y"
+
+/* Line 1455 of yacc.c  */
+#line 102 "../../src/evaluationmanager/pdl_yacc.y"
     { (yyval.rule) = lcmaps_add_rule((yyvsp[(2) - (4)].record),  0, (yyvsp[(4) - (4)].record)); }
     break;
 
 
-/* Line 1267 of yacc.c.  */
-#line 1462 "evaluationmanager/pdl_yacc.c"
+
+/* Line 1455 of yacc.c  */
+#line 1508 "evaluationmanager/pdl_yacc.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -1469,7 +1515,6 @@ yyreduce:
 
   *++yyvsp = yyval;
 
-
   /* Now `shift' the result of the reduction.  Determine what state
      that goes to, based on the state we popped back to and the rule
      number reduced by.  */
@@ -1534,7 +1579,7 @@ yyerrlab:
 
   if (yyerrstatus == 3)
     {
-      /* If just tried and failed to reuse look-ahead token after an
+      /* If just tried and failed to reuse lookahead token after an
 	 error, discard it.  */
 
       if (yychar <= YYEOF)
@@ -1551,7 +1596,7 @@ yyerrlab:
 	}
     }
 
-  /* Else will try to reuse look-ahead token after shifting the error
+  /* Else will try to reuse lookahead token after shifting the error
      token.  */
   goto yyerrlab1;
 
@@ -1608,9 +1653,6 @@ yyerrlab1:
       YY_STACK_PRINT (yyss, yyssp);
     }
 
-  if (yyn == YYFINAL)
-    YYACCEPT;
-
   *++yyvsp = yylval;
 
 
@@ -1635,7 +1677,7 @@ yyabortlab:
   yyresult = 1;
   goto yyreturn;
 
-#ifndef yyoverflow
+#if !defined(yyoverflow) || YYERROR_VERBOSE
 /*-------------------------------------------------.
 | yyexhaustedlab -- memory exhaustion comes here.  |
 `-------------------------------------------------*/
@@ -1646,7 +1688,7 @@ yyexhaustedlab:
 #endif
 
 yyreturn:
-  if (yychar != YYEOF && yychar != YYEMPTY)
+  if (yychar != YYEMPTY)
      yydestruct ("Cleanup: discarding lookahead",
 		 yytoken, &yylval);
   /* Do not reclaim the symbols of the rule which action triggered
@@ -1672,10 +1714,17 @@ yyreturn:
 }
 
 
-#line 106 "pdl_yacc.y"
+
+/* Line 1675 of yacc.c  */
+#line 105 "../../src/evaluationmanager/pdl_yacc.y"
 
 
 void lcmaps_set_yylval(record_t* r) {
   yylval.record = r;
 }
 
+/* Our pdl_yyparse() is just a wrapper around yyparse() */
+int pdl_yyparse (void)	{
+    return yyparse();
+}
+
