@@ -75,6 +75,7 @@
 #include "lcmaps_account.h"
 #include "lcmaps_types.h"
 #include <grp.h>
+#include <sys/types.h>
 
 /******************************************************************************
                              Define constants
@@ -99,14 +100,10 @@
 
 /*!
     \struct lcmaps_cred_id_s
-    \brief structure representing an LCMAPS credential.
-*/
-/*!
     \typedef lcmaps_cred_id_t
-    \brief Type of LCMAPS credentials
+    \brief structure type representing an LCMAPS credential.
 */
 
-
 typedef struct lcmaps_cred_id_s
 {
 #ifdef LCMAPS_GSI_MODE
@@ -135,12 +132,6 @@ lcmaps_cred_id_t;
  *                 Module definition
  *****************************************************************************/
 
-#ifndef LCMAPS_CREDENTIAL_C
-#   define LCMAPS_CREDENTIAL_STORAGE_CLASS extern
-#else
-#   define LCMAPS_CREDENTIAL_STORAGE_CLASS
-#endif
-
 /*! \name CREDENTIAL FUNCTIONS */
 /*@{*/ /* Beginning of credential functions */
 
@@ -148,15 +139,15 @@ lcmaps_cred_id_t;
 
 
 /* TODO function textual headers */
-LCMAPS_CREDENTIAL_STORAGE_CLASS int lcmaps_credential_store_gss_cred_id_t_and_sub_elements(
+int lcmaps_credential_store_gss_cred_id_t_and_sub_elements(
         gss_cred_id_t gss_cred_id,
         lcmaps_cred_id_t * plcmaps_credential
 );
-LCMAPS_CREDENTIAL_STORAGE_CLASS int lcmaps_credential_store_pem_string_and_sub_elements(
+int lcmaps_credential_store_pem_string_and_sub_elements(
         char * pem_string,
         lcmaps_cred_id_t * plcmaps_credential
 );
-LCMAPS_CREDENTIAL_STORAGE_CLASS int lcmaps_credential_store_x509_and_sub_elements(
+int lcmaps_credential_store_x509_and_sub_elements(
         X509 * cert,
         STACK_OF(X509) * certchain,
         lcmaps_cred_id_t * plcmaps_credential
@@ -179,7 +170,7 @@ Returns:        LCMAPS_CRED_SUCCESS: succes
                 LCMAPS_CRED_INVOCATION_ERROR: failure, because lcmaps_credential
                                      does not exist
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS int lcmaps_credential_store_gss_cred_id_t(
+int lcmaps_credential_store_gss_cred_id_t(
         gss_cred_id_t      gss_credential,
         lcmaps_cred_id_t * plcmaps_credential
 );
@@ -191,7 +182,7 @@ Parameters:
                 lcmaps_credential: lcmaps_credential
 Returns:        gss credential
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS gss_cred_id_t lcmaps_credential_get_gss_cred_id_t(
+gss_cred_id_t lcmaps_credential_get_gss_cred_id_t(
         lcmaps_cred_id_t lcmaps_credential
 );
 /******************************************************************************
@@ -203,7 +194,7 @@ Parameters:
 Returns:        LCMAPS_CRED_SUCCESS: succes
                 LCMAPS_CRED_ERROR: failure
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS int lcmaps_credential_store_gss_ctx_id_t(
+int lcmaps_credential_store_gss_ctx_id_t(
         gss_ctx_id_t       gss_context,
         lcmaps_cred_id_t * plcmaps_credential
 );
@@ -215,7 +206,7 @@ Parameters:
                 lcmaps_credential: lcmaps_credential
 Returns:        gss security context
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS gss_ctx_id_t lcmaps_credential_get_gss_ctx_id_t(
+gss_ctx_id_t lcmaps_credential_get_gss_ctx_id_t(
         lcmaps_cred_id_t lcmaps_credential
 );
 
@@ -231,7 +222,7 @@ Returns:        LCMAPS_CRED_SUCCESS:  success
                 LCMAPS_CRED_NO_PEM_STRING: PEM-encoded string is empty
                 LCMAPS_CRED_INVOCATION_ERROR: failure, because lcmaps_credential does not exist
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS int lcmaps_credential_store_pem_string(
+int lcmaps_credential_store_pem_string(
         char *             pem_string,
         lcmaps_cred_id_t * plcmaps_credential
 );
@@ -243,7 +234,7 @@ Parameters:
                 lcmaps_credential: lcmaps_credential
 Returns:        PEM-encoded string
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS char * lcmaps_credential_get_pem_string(
+char * lcmaps_credential_get_pem_string(
         lcmaps_cred_id_t lcmaps_credential
 );
 
@@ -258,7 +249,7 @@ Returns:        LCMAPS_CRED_SUCCESS: succes
                 LCMAPS_CRED_INVOCATION_ERROR: failure, because lcmaps_credential
                                      does not exist
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS int lcmaps_credential_store_stack_of_x509(
+int lcmaps_credential_store_stack_of_x509(
         STACK_OF(X509)   * px509_chain,
         lcmaps_cred_id_t * plcmaps_credential
 );
@@ -276,7 +267,7 @@ Returns:        LCMAPS_CRED_SUCCESS: succes
                 LCMAPS_CRED_INVOCATION_ERROR: failure, because lcmaps_credential
                                      does not exist
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS int lcmaps_credential_store_x509(
+int lcmaps_credential_store_x509(
         X509 *             px509,
         lcmaps_cred_id_t * plcmaps_credential
 );
@@ -288,7 +279,7 @@ Parameters:
                 lcmaps_credential: lcmaps_credential
 Returns:        the X509 cert.
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS X509 * lcmaps_credential_get_x509_cred(
+X509 * lcmaps_credential_get_x509_cred(
         lcmaps_cred_id_t lcmaps_credential
 );
 
@@ -299,7 +290,7 @@ Parameters:
                 lcmaps_credential: lcmaps_credential
 Returns:        the X509 cert chain
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS STACK_OF(X509) * lcmaps_credential_get_x509_chain(
+STACK_OF(X509) * lcmaps_credential_get_x509_chain(
         lcmaps_cred_id_t lcmaps_credential
 );
 
@@ -316,7 +307,7 @@ Returns:        LCMAPS_CRED_SUCCESS: success
                 LCMAPS_CRED_NO_DN:   no DN to fill
                 LCMAPS_CRED_INVOCATION_ERROR: failure, because lcmaps_credential does not exist
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS int lcmaps_credential_store_dn(
+int lcmaps_credential_store_dn(
         char *             dn,
         lcmaps_cred_id_t * plcmaps_credential
 );
@@ -328,7 +319,7 @@ Parameters:
                 lcmaps_credential: lcmaps_credential
 Returns:        user dn
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS char * lcmaps_credential_get_dn(
+char * lcmaps_credential_get_dn(
         lcmaps_cred_id_t lcmaps_credential
 );
 
@@ -344,7 +335,7 @@ Returns:        LCMAPS_CRED_SUCCESS: success
                 LCMAPS_CRED_ERROR: failure, because of a malloc error or a malformed fqan list
                 LCMAPS_CRED_INVOCATION_ERROR: failure, because lcmaps_credential does not exist
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS int lcmaps_credential_store_lcmaps_vomsdata (
+int lcmaps_credential_store_lcmaps_vomsdata (
         lcmaps_vomsdata_t * lcmaps_vomsdata,
         lcmaps_cred_id_t * plcmaps_credential
 );
@@ -361,7 +352,7 @@ Returns:        LCMAPS_CRED_SUCCESS: success
                 LCMAPS_CRED_ERROR: failure, because of a malloc error or a malformed fqan list
                 LCMAPS_CRED_INVOCATION_ERROR: failure, because lcmaps_credential does not exist
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS int lcmaps_credential_store_fqan_list(
+int lcmaps_credential_store_fqan_list(
         int                nfqan,
         char **            fqan_list,
         lcmaps_cred_id_t * plcmaps_credential
@@ -375,7 +366,7 @@ Parameters:
                 lcmaps_credential: lcmaps_credential
 Returns:        list of FQANs
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS char ** lcmaps_credential_get_fqan_list(
+char ** lcmaps_credential_get_fqan_list(
         int *            pnfqan,
         lcmaps_cred_id_t lcmaps_credential
 );
@@ -389,7 +380,7 @@ Parameters:
 Returns:        LCMAPS_CRED_SUCCESS: success
                 LCMAPS_CRED_INVOCATION_ERROR: failure, because lcmaps_credential does not exist
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS int lcmaps_credential_store_mapcounter(
+int lcmaps_credential_store_mapcounter(
         int mapcounter,
         lcmaps_cred_id_t * plcmaps_credential
 );
@@ -401,7 +392,7 @@ Parameters:
                 lcmaps_credential: lcmaps_credential
 Returns:        mapcounter
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS int lcmaps_credential_get_mapcounter(
+int lcmaps_credential_get_mapcounter(
         lcmaps_cred_id_t lcmaps_credential
 );
 
@@ -421,7 +412,7 @@ Returns:        LCMAPS_CRED_SUCCESS: success
                 LCMAPS_CRED_ERROR:   failure, because error filling lcmaps_account_info_t
                 LCMAPS_CRED_INVOCATION_ERROR: failure, because lcmaps_credential does not exist
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS int lcmaps_credential_store_requested_account(
+int lcmaps_credential_store_requested_account(
         uid_t *                 puid,
         gid_t **                ppgid_list,
         int *                   pnpgid,
@@ -438,7 +429,7 @@ Parameters:
                 lcmaps_credential: lcmaps_credential
 Returns:        requested account
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS lcmaps_account_info_t lcmaps_credential_get_requested_account(
+lcmaps_account_info_t lcmaps_credential_get_requested_account(
         lcmaps_cred_id_t lcmaps_credential
 );
 
@@ -450,7 +441,7 @@ Parameters:
                 string_list: list of strings
 Returns:        LCMAPS_CRED_SUCCESS
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS int lcmaps_clean_list_of_strings(
+int lcmaps_clean_list_of_strings(
     int listlen, char ** string_list
 );
 
@@ -464,7 +455,7 @@ Returns:        LCMAPS_CRED_SUCCESS:  success
                 LCMAPS_CRED_ERROR:  failed to set the member values
                 LCMAPS_CRED_INVOCATION_ERROR: failure, because lcmaps_credential does not exist
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS int lcmaps_credential_init(
+int lcmaps_credential_init(
         lcmaps_cred_id_t * plcmaps_credential
 );
 
@@ -476,7 +467,7 @@ Parameters:
 Returns:        LCMAPS_CRED_SUCCESS: success
                 LCMAPS_CRED_ERROR: failure
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS int lcmaps_release_cred(
+int lcmaps_release_cred(
         lcmaps_cred_id_t * plcmaps_credential
 );
 
@@ -488,7 +479,7 @@ Parameters:
 Returns:        LCMAPS_CRED_SUCCESS: success
                 LCMAPS_CRED_ERROR: failure
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS int lcmaps_release_cred_leave_STACK_OF_X509(
+int lcmaps_release_cred_leave_STACK_OF_X509(
         lcmaps_cred_id_t * plcmaps_credential
 );
 
@@ -500,7 +491,7 @@ Parameters:
 Returns:        LCMAPS_CRED_SUCCESS: success
                 LCMAPS_CRED_ERROR: failure
 ******************************************************************************/
-LCMAPS_CREDENTIAL_STORAGE_CLASS int lcmaps_clean_vomsdata(
+int lcmaps_clean_vomsdata(
         lcmaps_vomsdata_t * vomsdata
 );
 
@@ -511,7 +502,7 @@ LCMAPS_CREDENTIAL_STORAGE_CLASS int lcmaps_clean_vomsdata(
 /******************************************************************************
 CVS Information:
     $Source: /srv/home/dennisvd/svn/mw-security/lcmaps/src/grid_credential_handling/_lcmaps_credential.h,v $
-    $Date: 2012-01-01 15:20:00 +0100 (Sun, 01 Jan 2012) $
-    $Revision: 15818 $
-    $Author: okoeroo $
+    $Date: 2015-02-02 11:46:42 +0100 (Mon, 02 Feb 2015) $
+    $Revision: 18216 $
+    $Author: msalle $
 ******************************************************************************/
