@@ -67,26 +67,20 @@
  *                 Module definition
  *****************************************************************************/
 
-#ifndef LCMAPS_RUNVARS_C
-#   define LCMAPS_RUNVARS_STORAGE_CLASS extern
-#else
-#   define LCMAPS_RUNVARS_STORAGE_CLASS
-#endif
-
-LCMAPS_RUNVARS_STORAGE_CLASS int lcmaps_extractRunVars(
+int lcmaps_extractRunVars(
         lcmaps_request_t request,
         lcmaps_cred_id_t lcmaps_cred,
 	char *requested_username
 );
 
-LCMAPS_RUNVARS_STORAGE_CLASS void * lcmaps_getRunVars(
-        char *argName,
-        char *argType
+void * lcmaps_getRunVars(
+        const char *argName,
+        const char *argType
 );
 
-LCMAPS_RUNVARS_STORAGE_CLASS int lcmaps_setRunVars(
-        char *argName,
-        char *argType,
+int lcmaps_setRunVars(
+        const char *argName,
+        const char *argType,
         void *value
 );
 
@@ -101,7 +95,7 @@ LCMAPS_RUNVARS_STORAGE_CLASS int lcmaps_setRunVars(
 /******************************************************************************
 CVS Information:
     $Source: /srv/home/dennisvd/svn/mw-security/lcmaps/src/grid_credential_handling/_lcmaps_runvars.h,v $
-    $Date: 2013-02-21 20:24:33 +0100 (Thu, 21 Feb 2013) $
-    $Revision: 16855 $
+    $Date: 2014-03-13 11:56:06 +0100 (Thu, 13 Mar 2014) $
+    $Revision: 17647 $
     $Author: msalle $
 ******************************************************************************/
