@@ -47,22 +47,30 @@
     Last change: support for VOMS' generic attributes  (Placi Flury flury@switch.ch)
 */
 
+#define _XOPEN_SOURCE	600
+
 /*****************************************************************************
                             Include header files
 ******************************************************************************/
 #include "lcmaps_config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <pwd.h>
+#include <sys/types.h>
+
+#include <errno.h>
+
 #include <openssl/x509.h>
 #include <openssl/x509v3.h>
-#include "gssapi.h"
+
+#include <openssl/asn1.h>
+#include <openssl/err.h>
 
 #include "lcmaps_log.h"
 #include "lcmaps_types.h"
-#include "_lcmaps_gsi_utils.h"
 #include "_lcmaps_voms_attributes.h"
+#include "lcmaps_basic.h"
 
 
 /*#include "voms_apic.h"*/
@@ -76,11 +84,20 @@
 #define DEFAULT_CERT_DIR "/etc/grid-security/certificates"
 #define DEFAULT_VOMS_DIR "/etc/grid-security/vomsdir"
 
+/* Define a VERIFY_FULL_KNOWN to a combination of all known flags, this makes
+ * the checks a lot easier */
+#define VERIFY_FULL_KNOWN    (VERIFY_DATE | VERIFY_NOTARGET | VERIFY_KEY | VERIFY_SIGN | VERIFY_ORDER | VERIFY_ID | VERIFY_CERTLIST)
+
+/* valid flags for lcmaps_set_voms_verification_time() */
+#define VERIFY_OFFSET_NOTBEFORE	    1
+#define VERIFY_OFFSET_NOTAFTER	    2
+
+
 /******************************************************************************
                           Module specific prototypes
 ******************************************************************************/
 
-static void set_vomsdir_certdir(void);
+static void set_vomsdir_certdir(char **vomsdir, char **certdir);
 static void print_lcmaps_vomsdata(lcmaps_vomsdata_t * lcmaps_vomsdata);
 #ifdef LCMAPS_DEPRECATED
 static void print_vomsdata(struct vomsdata *);
@@ -100,20 +117,29 @@ void lcmaps_print_stack_of_x509_to_string(STACK_OF(X509) * px509_chain,
 void lcmaps_print_x509_to_string(X509 * px509_cert, char *output_file);
 
 
+static time_t asn1TimeToTimeT(ASN1_TIME* asn1time);
+
 /******************************************************************************
                        Define module specific variables
 ******************************************************************************/
 
-static char *certdir = NULL;
-static char *vomsdir = NULL;
+/*
+ * Always verify all VOMS credentials by default
+ */
+static unsigned int verify_voms_flags = VERIFY_FULL;
 
 /*
- * Always verify VOMS credentials by default
- * as designed by the VOMS developers
+ * Verification time for proxy. In case voms_verify_time_flags==1 the time is
+ * relative to the notBefore time of the last proxy delegation and the actual
+ * time can only be determined once we have the proxy cert. In case flags==0
+ * it is the UNIX time.
  */
-static int   verify_voms_credentials = 1;
+static time_t voms_verify_time = 0;
+static int voms_verify_time_flags = 0;
 
+#ifdef GRST_ACTIVE
 
+#if 0
 /* Everything ok (= OpenSSL X509_V_OK) */
 #define GRST_RET_OK             0
 
@@ -126,11 +152,12 @@ static int   verify_voms_credentials = 1;
 #define GRST_VOMS_OID           "1.3.6.1.4.1.8005.100.100.5"
 #define GRST_VOMS_DIR           "/etc/grid-security/vomsdir"
 
-#define GRST_ASN1_MAXCOORDLEN   50
 #define GRST_ASN1_MAXTAGS       500
+#endif
+
+#define GRST_ASN1_MAXCOORDLEN   50
 
 
-#ifdef GRST_ACTIVE
 struct GRSTasn1TagList
 {
     char treecoords[GRST_ASN1_MAXCOORDLEN + 1];
@@ -145,7 +172,7 @@ int GRSTasn1GetX509Name(char *, int, char *, char *,
 #endif
 
 /* reverse error codes from voms_apic.h */
-static char *retmsg[] =
+static const char *retmsg[] =
     { "VERR_NONE", "VERR_NOSOCKET", "VERR_NOIDENT", "VERR_COMM",
     "VERR_PARAM", "VERR_NOEXT", "VERR_NOINIT",
     "VERR_TIME", "VERR_IDCHECK", "VERR_EXTRAINFO",
@@ -160,7 +187,8 @@ static char *retmsg[] =
 /******************************************************************************
 Function:   lcmaps_disable_voms_attributes_verification
 Description:
-            Disables the verification in the VOMS API
+            Disables the verification in the VOMS API, identical to
+	    lcmaps_set_voms_attributes_verification(VERIFY_NONE);
 Parameters:
             Void
 Returns:
@@ -168,14 +196,15 @@ Returns:
 ******************************************************************************/
 void lcmaps_disable_voms_attributes_verification (void)
 {
-    /* disable verification */
-    verify_voms_credentials = 0;
+    /* Set to verify no VOMS attributes */
+    verify_voms_flags=VERIFY_NONE;
 }
 
 /******************************************************************************
 Function:   lcmaps_enable_voms_attributes_verification
 Description:
-            Enables the verification in the VOMS API
+            Enables the verification in the VOMS API, identical to
+	    lcmaps_set_voms_attributes_verification(VERIFY_FULL);
 Parameters:
             Void
 Returns:
@@ -183,15 +212,18 @@ Returns:
 ******************************************************************************/
 void lcmaps_enable_voms_attributes_verification (void)
 {
-    /* enable verification */
-    verify_voms_credentials = 1;
+    /* Set to verify all VOMS attributes */
+    verify_voms_flags=VERIFY_FULL;
 }
 
 /******************************************************************************
 Function:   lcmaps_is_set_to_verify_voms_attributes
 Description:
             Will return the current setting to enable or disable the
-            verification of the VOMS credentials by the VOMS API
+	    verification of the VOMS credentials by the VOMS API. I.e. it will
+	    return 1 only when *all* known enabling flags are set. See
+	    lcmaps_get_voms_attributes_verification() when a more detailed
+	    answer is needed.
 Parameters:
             Void
 Returns:
@@ -200,11 +232,98 @@ Returns:
 ******************************************************************************/
 int lcmaps_is_set_to_verify_voms_attributes (void)
 {
-    /* Will verify VOMS credentials? */
-    return verify_voms_credentials;
+    /* When all (known) flags are set: return 1, otherwise 0 */
+    return (verify_voms_flags & VERIFY_FULL_KNOWN)==VERIFY_FULL_KNOWN ? 1 : 0;
 }
 
 
+/******************************************************************************
+Function:   lcmaps_set_voms_attributes_verification
+Description:
+	    Sets the verification level used by the VOMS API to a combination of
+	    flags as defined in voms_apic.h:
+	      VERIFY_FULL   VERIFY_NONE
+	      VERIFY_DATE   VERIFY_NOTARGET 
+	      VERIFY_KEY    VERIFY_SIGN
+	      VERIFY_ORDER  VERIFY_ID
+	      VERIFY_CERTLIST
+	    When all known simple enabling flags (i.e. all except FULL and NONE)
+	    are set, lcmaps_is_set_to_verify_voms_attributes() will return 1.
+Parameters:
+            int
+Returns:
+            Void
+******************************************************************************/
+void lcmaps_set_voms_attributes_verification (unsigned int verify_flags)
+{
+    /* set verification to specified flags */
+    verify_voms_flags=verify_flags;
+}
+
+
+/******************************************************************************
+Function:   lcmaps_get_voms_attributes_verification
+Description:
+	    Gets the verification level used by the VOMS API. The result is a
+	    combination of flags as defined in voms_apic.h:
+	      VERIFY_FULL   VERIFY_NONE
+	      VERIFY_DATE   VERIFY_NOTARGET 
+	      VERIFY_KEY    VERIFY_SIGN
+	      VERIFY_ORDER  VERIFY_ID
+	      VERIFY_CERTLIST
+Parameters:
+            void
+Returns:
+	    flags currently set
+******************************************************************************/
+unsigned int lcmaps_get_voms_attributes_verification (void)
+{
+    return verify_voms_flags;
+}
+
+
+/******************************************************************************
+Function:   lcmaps_set_voms_verification_time
+Description:
+	    Sets the verification time to use to the given time. When flags is
+	    equal to 1 the time is relative to the notBefore time of the last
+	    proxy delegation in the chain.
+Parameters:
+            time_t verification time
+	    int flags when 0 verification time is Unix time, otherwise it's
+	    number of seconds relative to notBefore.
+Returns:
+	    void
+******************************************************************************/
+void lcmaps_set_voms_verification_time(time_t verify_time, int flags)
+{
+    voms_verify_time = verify_time;
+    voms_verify_time_flags = flags;
+}
+
+/******************************************************************************
+Function:   lcmaps_get_voms_verification_time
+Description:
+	    Returns the absolute or relative verification time at which VOMS
+	    will verify the AC and signature chain.
+Parameters:
+            time_t *verification_time (output)
+	    int *flags (output)
+Returns:
+	    int: 0 when success, -1 on error (and sets errno to EINVAL)
+******************************************************************************/
+int lcmaps_get_voms_verification_time(time_t *verify_time, int *flags)
+{
+    if (verify_time==NULL || flags==NULL)   {
+	errno=EINVAL;
+	return -1;
+    }
+
+    *verify_time = voms_verify_time;
+    *flags = voms_verify_time_flags;
+    return 0;
+}
+
 /**************************************************************************
  * Read the generic attributes from the usercert.
  *
@@ -221,6 +340,7 @@ static int lcmaps_get_voms_generic_attributes(struct voms *vo, struct vomsdata *
                                               lcmaps_voms_generic_attr_t **attr_list, int *nattrs)
 {
     int handle, attrSrcNum;
+    int attr_number = 0;
     int errno_local = 0;
     int i, j, k;
     /* struct attribute *tmp_list; */
@@ -242,12 +362,19 @@ static int lcmaps_get_voms_generic_attributes(struct voms *vo, struct vomsdata *
             *nattrs = 0;
             return 1;
         }
-        *nattrs += VOMS_GetAttributesNumber(vo, handle, vd, &errno_local);
+        attr_number = VOMS_GetAttributesNumber(vo, handle, vd, &errno_local);
+        if (errno_local != 0 || attr_number<0) {
+            lcmaps_log(LOG_ERR, "%s Error: VOMS_GetAttributesNumber(), %s", __func__,
+		    (errno_local ? retmsg[errno_local] : "negative number of attributes"));
+            *nattrs = 0;
+            return 1;
+        }
+	*nattrs += attr_number;
     }
     lcmaps_log_debug(3, "%s: found %d generic attributes. \n", __func__, *nattrs);
 
     /* Allocating lcmaps_voms_generic_attr_t's */
-    *attr_list = (lcmaps_voms_generic_attr_t *) calloc (*nattrs, sizeof(lcmaps_voms_generic_attr_t));
+    *attr_list = (lcmaps_voms_generic_attr_t *) calloc ((size_t)(*nattrs), sizeof(lcmaps_voms_generic_attr_t));
     if (*attr_list == NULL) {
         lcmaps_log(LOG_ERR, "%s: Error: Could not allocate more memory: %s\n", __func__, strerror(errno));
         return 1;
@@ -330,6 +457,8 @@ char **lcmaps_x509_to_voms_fqans(X509 * px509_cred,
                                  lcmaps_vomsdata_t ** lcmaps_vomsdata,
                                  int *pnfqan)
 {
+    char *certdir = NULL;
+    char *vomsdir = NULL;
     struct vomsdata *vd = NULL;
     struct voms **volist = NULL;
     struct voms *vo = NULL;
@@ -343,6 +472,11 @@ char **lcmaps_x509_to_voms_fqans(X509 * px509_cred,
     int k,j;
     int nattrs = 0;
     const char *template_hint = "LCMAPS will ignore the VOMS extensions in the mapping sequence";
+    unsigned int verify_flags;
+    unsigned long code;
+    char *voms_errbuf=NULL;
+    time_t verify_time=0,verify_time_offset=0;
+
 
 #ifdef DEPRECATED_VOMS_VERSION_CHECK
     int vomsmajor = -1;
@@ -361,7 +495,7 @@ char **lcmaps_x509_to_voms_fqans(X509 * px509_cred,
 
     /* CA dir through API takes presedence over environment */
 
-    set_vomsdir_certdir();
+    set_vomsdir_certdir(&vomsdir, &certdir);
 
     if ((vd = VOMS_Init(vomsdir, certdir)) == NULL) {
         lcmaps_log(LOG_ERR, "%s(): failed to initialize voms data structure. Either the VOMSDIR directory (%s) or the CA certificate directory (%s) does not exist.\n", __func__, vomsdir, certdir);
@@ -369,6 +503,53 @@ char **lcmaps_x509_to_voms_fqans(X509 * px509_cred,
     }
     lcmaps_log_debug(5, "%s(): VOMS data structure initialized\n", __func__);
 
+    /* Do we want to override the verification time ? */
+    switch (voms_verify_time_flags) {
+	case 0:
+	    verify_time=voms_verify_time;
+	    break;
+	case VERIFY_OFFSET_NOTBEFORE:
+	    verify_time_offset=asn1TimeToTimeT(X509_get_notBefore(px509_cred));
+	    if (verify_time_offset<0)	{
+		lcmaps_log(LOG_ERR,
+			"Could not get notBefore time from leaf proxy: %s\n",
+			strerror(errno));
+		goto fail_lcmaps_x509_to_voms_fqans;
+	    }
+	    lcmaps_log(LOG_INFO,
+		    "Setting verification time relative to notBefore time of leaf proxy: %ld + %ld.\n",
+		    verify_time_offset, voms_verify_time);
+	    verify_time=verify_time_offset+voms_verify_time;
+	    break;
+	case VERIFY_OFFSET_NOTAFTER:
+	    verify_time_offset=asn1TimeToTimeT(X509_get_notAfter(px509_cred));
+	    if (verify_time_offset<0)	{
+		lcmaps_log(LOG_ERR,
+			"Could not get notAfter time from leaf proxy: %s\n",
+			strerror(errno));
+		goto fail_lcmaps_x509_to_voms_fqans;
+	    }
+	    lcmaps_log(LOG_INFO,
+		    "Setting verification time relative to notAfter time of leaf proxy: %ld + %ld.\n",
+		    verify_time_offset, voms_verify_time);
+	    verify_time=verify_time_offset+voms_verify_time;
+	    break;
+	default:
+	    lcmaps_log(LOG_ERR,"Unknown flag specified: %d\n",
+		    voms_verify_time_flags);
+	    goto fail_lcmaps_x509_to_voms_fqans;
+    }
+
+    /* Now set the actual time */
+    if (verify_time>0)	{
+	lcmaps_log(LOG_DEBUG,"Setting verification time to %ld seconds\n",
+		verify_time);
+	VOMS_SetVerificationTime(verify_time, vd, &errNo);
+    } else if (verify_time<0)	{
+	lcmaps_log(LOG_ERR,"Verify time is invalid.\n");
+	goto fail_lcmaps_x509_to_voms_fqans;
+    }
+
 #ifdef GUMSOSG
     /*
      * OSG GUMS - without verification
@@ -383,24 +564,36 @@ char **lcmaps_x509_to_voms_fqans(X509 * px509_cred,
     lcmaps_disable_voms_attributes_verification();
 #endif
 
-    /* Enables or disables the VOMS attribute verification */
+    /* Enables or disables part of the VOMS attribute verification */
     if (lcmaps_is_set_to_verify_voms_attributes() == 0)
     {
-        VOMS_SetVerificationType(VERIFY_NONE, vd, &errNo);
-        lcmaps_log_debug (1, "%s(): VOMS attribute verification is disabled.\n", __func__);
+	verify_flags=lcmaps_get_voms_attributes_verification();
+	/* Even though all flags are specified essentially as bitmasks and hence
+	 * better used unsigned, VOMS_SetVerificationType() wants an int, cast
+	 * it only here */
+        VOMS_SetVerificationType((int)verify_flags, vd, &errNo);
+	if (verify_flags==VERIFY_NONE)
+	    lcmaps_log_debug (1,
+		    "%s(): VOMS attribute verification is fully disabled.\n",
+		    __func__);
+	else
+	    lcmaps_log_debug (1,
+		    "%s(): VOMS attribute verification is partially disabled, "
+		    "Enabled set is: 0x%08x.\n", __func__, verify_flags);
     }
 
     /* Extract the VOMS attributes */
     if (VOMS_Retrieve(px509_cred, px509_chain, RECURSE_CHAIN, vd, &errNo))
     {
         volist = vd->data;
-        tmp_lcmaps_vomsdata =(lcmaps_vomsdata_t *) calloc(1, sizeof(lcmaps_vomsdata_t));
+        tmp_lcmaps_vomsdata =(lcmaps_vomsdata_t *) calloc((size_t)1, sizeof(lcmaps_vomsdata_t));
 
         for(k = 0, (vo = volist[k]); (vo = volist[k]); k++){
             lcmaps_log_debug(3, "%s(): setting voms data for VO == %s\n", __func__, vo->voname);
             lcmaps_log_debug(3, "%s(): setting voms data for VO server == %s\n", __func__, vo->server);
 
-            tmp_lcmaps_voms = (lcmaps_voms_t *) realloc(tmp_lcmaps_voms,sizeof(lcmaps_voms_t) * (k+1));
+	    /* can cast k(+1) to size_t as k>=0 */
+            tmp_lcmaps_voms = (lcmaps_voms_t *) realloc(tmp_lcmaps_voms,sizeof(lcmaps_voms_t) * (size_t)(k+1));
             tmp_lcmaps_voms[k].user_dn = strdup(vo->user);
             tmp_lcmaps_voms[k].user_ca = strdup(vo->userca);
             tmp_lcmaps_voms[k].voms_issuer_dn = strdup(vo->server);
@@ -415,8 +608,7 @@ char **lcmaps_x509_to_voms_fqans(X509 * px509_cred,
                 lcmaps_log_debug(2, "%s(): NO DATA TYPE \n", __func__);
                 break;
             case TYPE_CUSTOM:
-                lcmaps_log_debug(2, "%s(): CUSTOM TYPE, %s\n", __func__,
-                                 vo->datalen - 10, vo->custom);
+                lcmaps_log_debug(2, "%s(): CUSTOM TYPE, %s\n", __func__, vo->custom);
                 break;
             case TYPE_STD:
                 lcmaps_log_debug(2, "%s(): TYP_STD\n", __func__);
@@ -424,14 +616,15 @@ char **lcmaps_x509_to_voms_fqans(X509 * px509_cred,
                 {
                     for (j = 0, tmp_fqan = NULL; (vo->fqan)[j]; j++)
                     {
-                        tmp_fqan         = (lcmaps_fqan_unix_t *) realloc(tmp_fqan,sizeof(lcmaps_fqan_unix_t) * (j+1));
+			/* can cast j(+1) to size_t as j>=0 */
+                        tmp_fqan         = (lcmaps_fqan_unix_t *) realloc(tmp_fqan,sizeof(lcmaps_fqan_unix_t) * (size_t)(j+1));
                         tmp_fqan[j].fqan = strdup((vo->fqan)[j]);
-                        tmp_fqan[j].uid  = -1;
-                        tmp_fqan[j].gid  = -1;
+                        tmp_fqan[j].uid  = (uid_t)-1;
+                        tmp_fqan[j].gid  = (gid_t)-1;
                         /* make a copy for the char** list of fqans  */
                         if(fqans_list == NULL)
                         {
-                            fqans_list = (char **) calloc(1,sizeof(char*));
+                            fqans_list = (char **) calloc((size_t)1,sizeof(char*));
                             *fqans_list = NULL;
                         }
                         if(*fqans_list == NULL)
@@ -440,7 +633,8 @@ char **lcmaps_x509_to_voms_fqans(X509 * px509_cred,
                         }
                         else
                         {
-                            fqans_list = (char **) realloc(fqans_list, sizeof(char *) * (fqan_count + 1));
+			    /* can cast fqan_count to size_t as it's >=0 */
+                            fqans_list = (char **) realloc(fqans_list, sizeof(char *) * (size_t)(fqan_count + 1));
                             fqans_list[fqan_count] = strdup((vo->fqan)[j]);
                         }
                         fqan_count++;
@@ -507,92 +701,106 @@ char **lcmaps_x509_to_voms_fqans(X509 * px509_cred,
         goto success_lcmaps_x509_to_voms_fqans;
 
     }
-    else if (errNo == VERR_NOEXT) {     /* Non voms proxies will fall into this error */
-        /* Non voms proxies go here: set the number of fqans to zero. Not an error */
-        lcmaps_log_debug(2, "%s(): Debug: VOMS extensions not present in the certificate chain.\n", __func__);
-        goto fail_lcmaps_x509_to_voms_fqans;
-    }
-    else if (errNo == VERR_IDCHECK) {
-        lcmaps_log(LOG_ERR, "%s(): Warning: VOMS User data in extension different from the real ones. This could mean that the owner's VOMS credentials got transplanted/stolen from another. %s.\n", __func__, template_hint);
-        goto fail_lcmaps_x509_to_voms_fqans;
-    }
-    else if (errNo == VERR_TIME) {
-        lcmaps_log(LOG_ERR, "%s(): Warning: VOMS extensions expired for at least one of the VOs. %s.\n", __func__, template_hint);
-        goto fail_lcmaps_x509_to_voms_fqans;
-    }
-    else if (errNo == VERR_ORDER) {
-        lcmaps_log(LOG_ERR, "%s(): Warning: The ordering of the VOMS groups, as required by the client, was not delivered by VOMS. %s.\n", __func__, template_hint);
-        goto fail_lcmaps_x509_to_voms_fqans;
-    }
-    else if (errNo == VERR_NOSOCKET) {
-        lcmaps_log(LOG_ERR, "%s(): Warning: VOMS Socket problem. (an error that indicates an ABI change in VOMS). %s.\n", __func__, template_hint);
-        goto fail_lcmaps_x509_to_voms_fqans;
-    }
-    else if (errNo == VERR_NOIDENT) {
-        lcmaps_log(LOG_ERR, "%s(): Warning: VOMS Cannot identify itself (certificate problem). %s.\n", __func__, template_hint);
-        goto fail_lcmaps_x509_to_voms_fqans;
-    }
-    else if (errNo == VERR_COMM) {
-        lcmaps_log(LOG_ERR, "%s(): Warning: VOMS server problem %s.\n", __func__, template_hint);
-        goto fail_lcmaps_x509_to_voms_fqans;
-    }
-    else if (errNo == VERR_PARAM) {
-        lcmaps_log(LOG_ERR, "%s(): Warning: Wrong parameters for VOMS %s.\n", __func__, template_hint);
-        goto fail_lcmaps_x509_to_voms_fqans;
-    }
-    else if (errNo == VERR_NOINIT) {
-        lcmaps_log(LOG_ERR, "%s(): Warning: VOMS initialization error. Please check the VOMS directory (default: /etc/grid-security/vomsdir/) %s.\n", __func__, template_hint);
-        goto fail_lcmaps_x509_to_voms_fqans;
-    }
-    else if (errNo == VERR_EXTRAINFO) {
-        lcmaps_log(LOG_ERR, "%s(): Warning: VO name and URI missing making it an invalid VOMS extension. %s.\n", __func__, template_hint);
-        goto fail_lcmaps_x509_to_voms_fqans;
-    }
-    else if (errNo == VERR_FORMAT) {
-        lcmaps_log(LOG_ERR, "%s(): Warning: Wrong VOMS data format making it an invalid VOMS extension. %s.\n", __func__, template_hint);
-        goto fail_lcmaps_x509_to_voms_fqans;
-    }
-    else if (errNo == VERR_NODATA) {
-        lcmaps_log(LOG_ERR, "%s(): Warning: Empty VOMS extension making it an invalid VOMS extension. %s.\n", __func__, template_hint);
-        goto fail_lcmaps_x509_to_voms_fqans;
-    }
-    else if (errNo == VERR_PARSE) {
-        lcmaps_log(LOG_ERR, "%s(): Warning: VOMS parse error making it an invalid VOMS extension. %s.\n", __func__, template_hint);
-        goto fail_lcmaps_x509_to_voms_fqans;
-    }
-    else if (errNo == VERR_DIR) {
-        lcmaps_log(LOG_ERR, "%s(): Warning: VOMS directory error. %s.\n", __func__, template_hint);
-        goto fail_lcmaps_x509_to_voms_fqans;
-    }
-    else if (errNo == VERR_SIGN) {
-        lcmaps_log(LOG_ERR, "%s(): Warning: VOMS Signature error making it an unverifyable VOMS extension. Hint: check the VOMS directory (default: /etc/grid-security/vomsdir/) and the .lsc files. Also you could fallback on using installed VOMS host certificates. If the problem persists, contact the user. %s.\n", __func__, template_hint);
-        goto fail_lcmaps_x509_to_voms_fqans;
-    }
-    else if (errNo == VERR_SERVER) {
-        lcmaps_log(LOG_ERR, "%s(): Warning: Unidentifiable VOMS server. %s.\n", __func__, template_hint);
-        goto fail_lcmaps_x509_to_voms_fqans;
-    }
-    else if (errNo == VERR_MEM) {
-        lcmaps_log(LOG_ERR, "%s(): Warning: Memory problems in VOMS_Retrieve() %s.\n", __func__, template_hint);
-        goto fail_lcmaps_x509_to_voms_fqans;
-    }
-    else if (errNo == VERR_VERIFY) {
-        char *err_buffer = VOMS_ErrorMessage(vd, VERR_VERIFY, NULL, 0);
-        lcmaps_log(LOG_ERR, "%s(): Warning: Generic verification error for VOMS: \"%s\". %s.\n", __func__, err_buffer, template_hint);
-        free(err_buffer);
-        goto fail_lcmaps_x509_to_voms_fqans;
-    }
-    else if (errNo == VERR_TYPE) {
-        lcmaps_log(LOG_ERR, "%s(): Warning: Returned VOMS data of unknown type. %s.\n", __func__, template_hint);
-        goto fail_lcmaps_x509_to_voms_fqans;
-    }
-    else {
-        lcmaps_log(LOG_ERR, "%s(): VOMS_Retrieve() error --> %d %s.\n", __func__, errNo, template_hint);
-        goto fail_lcmaps_x509_to_voms_fqans;
+    else    {
+	/* Non voms proxies will fall into this error: handle separately */
+	if (errNo == VERR_NOEXT) {
+	    /* Non voms proxies go here: set the number of fqans to zero. Not an error */
+	    lcmaps_log_debug(2, "%s(): Debug: VOMS extensions not present in the certificate chain.\n", __func__);
+	    goto fail_lcmaps_x509_to_voms_fqans;
+	}
+
+	/* Dump OpenSSL stack, even though we don't have the VOMS error strings,
+	 * this can give useful feedback as we can look them up in the VOMS
+	 * source */
+	while ( (code=ERR_get_error() ) )
+	    lcmaps_log(LOG_NOTICE,"OpenSSL error stack: %s\n",
+		ERR_error_string(code,NULL));
+	/* Dump VOMS log message */
+	voms_errbuf = VOMS_ErrorMessage(vd, errNo, NULL, 0);
+
+	/* Now parse the rest */
+	if (errNo == VERR_IDCHECK) {
+	    lcmaps_log(LOG_ERR, "%s(): Warning: VOMS User data in extension different from the real ones: %s. This could mean that the owner's VOMS credentials got transplanted/stolen from another. %s.\n", __func__, voms_errbuf, template_hint);
+	    goto fail_lcmaps_x509_to_voms_fqans;
+	}
+	else if (errNo == VERR_TIME) {
+	    lcmaps_log(LOG_ERR, "%s(): Warning: VOMS extensions expired for at least one of the VOs: %s. %s.\n", __func__, voms_errbuf, template_hint);
+	    goto fail_lcmaps_x509_to_voms_fqans;
+	}
+	else if (errNo == VERR_ORDER) {
+	    lcmaps_log(LOG_ERR, "%s(): Warning: The ordering of the VOMS groups, as required by the client, was not delivered by VOMS: %s. %s.\n", __func__, voms_errbuf, template_hint);
+	    goto fail_lcmaps_x509_to_voms_fqans;
+	}
+	else if (errNo == VERR_NOSOCKET) {
+	    lcmaps_log(LOG_ERR, "%s(): Warning: VOMS Socket problem: %s. (an error that indicates an ABI change in VOMS). %s.\n", __func__, voms_errbuf, template_hint);
+	    goto fail_lcmaps_x509_to_voms_fqans;
+	}
+	else if (errNo == VERR_NOIDENT) {
+	    lcmaps_log(LOG_ERR, "%s(): Warning: VOMS Cannot identify itself (certificate problem): %s. %s.\n", __func__, voms_errbuf, template_hint);
+	    goto fail_lcmaps_x509_to_voms_fqans;
+	}
+	else if (errNo == VERR_COMM) {
+	    lcmaps_log(LOG_ERR, "%s(): Warning: VOMS server problem: %s. %s.\n", __func__, voms_errbuf, template_hint);
+	    goto fail_lcmaps_x509_to_voms_fqans;
+	}
+	else if (errNo == VERR_PARAM) {
+	    lcmaps_log(LOG_ERR, "%s(): Warning: Wrong parameters for VOMS: %s. %s.\n", __func__, voms_errbuf, template_hint);
+	    goto fail_lcmaps_x509_to_voms_fqans;
+	}
+	else if (errNo == VERR_NOINIT) {
+	    lcmaps_log(LOG_ERR, "%s(): Warning: VOMS initialization error: %s. Please check the VOMS directory (default: /etc/grid-security/vomsdir/) %s.\n", __func__, voms_errbuf, template_hint);
+	    goto fail_lcmaps_x509_to_voms_fqans;
+	}
+	else if (errNo == VERR_EXTRAINFO) {
+	    lcmaps_log(LOG_ERR, "%s(): Warning: VO name and URI missing making it an invalid VOMS extension: %s. %s.\n", __func__, voms_errbuf, template_hint);
+	    goto fail_lcmaps_x509_to_voms_fqans;
+	}
+	else if (errNo == VERR_FORMAT) {
+	    lcmaps_log(LOG_ERR, "%s(): Warning: Wrong VOMS data format making it an invalid VOMS extension: %s. %s.\n", __func__, voms_errbuf, template_hint);
+	    goto fail_lcmaps_x509_to_voms_fqans;
+	}
+	else if (errNo == VERR_NODATA) {
+	    lcmaps_log(LOG_ERR, "%s(): Warning: Empty VOMS extension making it an invalid VOMS extension: %s. %s.\n", __func__, voms_errbuf, template_hint);
+	    goto fail_lcmaps_x509_to_voms_fqans;
+	}
+	else if (errNo == VERR_PARSE) {
+	    lcmaps_log(LOG_ERR, "%s(): Warning: VOMS parse error making it an invalid VOMS extension: %s. %s.\n", __func__, voms_errbuf, template_hint);
+	    goto fail_lcmaps_x509_to_voms_fqans;
+	}
+	else if (errNo == VERR_DIR) {
+	    lcmaps_log(LOG_ERR, "%s(): Warning: VOMS directory error: %s. %s.\n", __func__, voms_errbuf, template_hint);
+	    goto fail_lcmaps_x509_to_voms_fqans;
+	}
+	else if (errNo == VERR_SIGN) {
+	    lcmaps_log(LOG_ERR, "%s(): Warning: VOMS Signature error making it an unverifiable VOMS extension: %s. Hint: check the VOMS directory (default: /etc/grid-security/vomsdir/) and the .lsc files. Also you could fall back on using installed VOMS host certificates. If the problem persists, contact the user. %s.\n", __func__, voms_errbuf, template_hint);
+	    goto fail_lcmaps_x509_to_voms_fqans;
+	}
+	else if (errNo == VERR_SERVER) {
+	    lcmaps_log(LOG_ERR, "%s(): Warning: Unidentifiable VOMS server: %s. %s.\n", __func__, voms_errbuf, template_hint);
+	    goto fail_lcmaps_x509_to_voms_fqans;
+	}
+	else if (errNo == VERR_MEM) {
+	    lcmaps_log(LOG_ERR, "%s(): Warning: Memory problems in VOMS_Retrieve(): %s. %s.\n", __func__, voms_errbuf, template_hint);
+	    goto fail_lcmaps_x509_to_voms_fqans;
+	}
+	else if (errNo == VERR_VERIFY) {
+	    lcmaps_log(LOG_ERR, "%s(): Warning: Generic verification error for VOMS: %s. %s.\n", __func__, voms_errbuf, template_hint);
+	    goto fail_lcmaps_x509_to_voms_fqans;
+	}
+	else if (errNo == VERR_TYPE) {
+	    lcmaps_log(LOG_ERR, "%s(): Warning: Returned VOMS data of unknown type: %s. %s.\n", __func__, voms_errbuf, template_hint);
+	    goto fail_lcmaps_x509_to_voms_fqans;
+	}
+	else {
+	    lcmaps_log(LOG_ERR, "%s(): VOMS_Retrieve() error: %d --> %s %s.\n", __func__, errNo, voms_errbuf, template_hint);
+	    goto fail_lcmaps_x509_to_voms_fqans;
+	}
     }
 
 
 success_lcmaps_x509_to_voms_fqans:
+    free(vomsdir);
+    free(certdir);
     lcmaps_log_debug(3, "%s(): Success, VOMS destroy\n", __func__);
     if (vd)
         VOMS_Destroy(vd);
@@ -602,6 +810,9 @@ success_lcmaps_x509_to_voms_fqans:
     return fqans_list;
 
 fail_lcmaps_x509_to_voms_fqans:
+    free(vomsdir);
+    free(certdir);
+    free(voms_errbuf);
     lcmaps_log_debug(3, "%s(): Failure, VOMS destroy\n", __func__);
     VOMS_Destroy(vd);
     *pnfqan = fqan_count; /* NOTE: invalid FQAN found only if this is != 0 */
@@ -609,16 +820,22 @@ fail_lcmaps_x509_to_voms_fqans:
 
 }
 
-static void set_vomsdir_certdir(void)
+static void set_vomsdir_certdir(char **vomsdir, char **certdir)
 {
-    /* Retrieve the vomsdir and certdir */
-    vomsdir = getenv("LCMAPS_X509_VOMS_DIR");
-    vomsdir = (vomsdir ? vomsdir : getenv("X509_VOMS_DIR"));
-    vomsdir = (vomsdir ? vomsdir : DEFAULT_VOMS_DIR);
+    const char *tmp;
 
-    certdir = getenv("LCMAPS_X509_CERT_DIR");
-    certdir = (certdir ? certdir : getenv("X509_CERT_DIR"));
-    certdir = (certdir ? certdir : DEFAULT_CERT_DIR);
+    /* Retrieve the vomsdir and certdir */
+    if ( (tmp=getenv("LCMAPS_X509_VOMS_DIR"))==NULL )	{
+	if ( (tmp=getenv("X509_VOMS_DIR"))==NULL )
+	    tmp=DEFAULT_VOMS_DIR;
+    }
+    *vomsdir=strdup(tmp);
+
+    if ( (tmp=getenv("LCMAPS_X509_CERT_DIR"))==NULL )	{
+	if ( (tmp=getenv("X509_CERT_DIR"))==NULL )
+	    tmp=DEFAULT_CERT_DIR;
+    }
+    *certdir=strdup(tmp); 
 }
 
 
@@ -633,7 +850,7 @@ Returns:
 ******************************************************************************/
 static void print_lcmaps_vomsdata(lcmaps_vomsdata_t * lcmaps_vomsdata)
 {
-    char *logstr = "    print_lcmaps_vomsdata()";
+    const char *logstr = "    print_lcmaps_vomsdata()";
     int i = 0;
     int j = 0;
 
@@ -714,7 +931,71 @@ static void print_lcmaps_vomsdata(lcmaps_vomsdata_t * lcmaps_vomsdata)
     }
 }
 
+/******************************************************************************
+Function:    asn1TimeToTimeT
+Description:
+    Converts the asn1time to a time_t
+Parameters:
+    asn1time:    ASN1_TIME to convert
+Returns:
+    time_t corresponding to the given ASN1_TIME or -1 on error
+ ******************************************************************************/
+static time_t asn1TimeToTimeT(ASN1_TIME* asn1time)  {
+   ASN1_GENERALIZEDTIME *asn1time_generalized=NULL;
+   char  *asn1time_string=NULL, *tz;
+   char   zone;
+   struct tm time_tm;
+   int year;
+   time_t mytime;
 
+   /* Convert to ASN1_GENERALIZEDTIME, which has a 4 digit year */
+   asn1time_generalized=ASN1_TIME_to_generalizedtime(asn1time, NULL);
+   if (asn1time_generalized==NULL)  {	/* error */
+       errno=EINVAL;
+       return -1;
+   }
+   
+   /* Convert to a char * */
+   asn1time_string=(char*)ASN1_STRING_data(asn1time_generalized);
+
+   /* Make sure to start with an empty struct */
+   memset(&time_tm,0,sizeof(struct tm));
+
+   if (strlen(asn1time_string)!=15) {	/* dont understand */
+       errno=EINVAL;
+       return -1;
+   }
+
+   if ((sscanf(asn1time_string, "%04d%02d%02d%02d%02d%02d%c",
+         &year,
+         &(time_tm.tm_mon),
+         &(time_tm.tm_mday),
+         &(time_tm.tm_hour),
+         &(time_tm.tm_min),
+         &(time_tm.tm_sec),
+         &zone) != 7) || (zone != 'Z'))	{ /* dont understand */
+       errno=EINVAL;
+       return -1;
+   }
+
+   /* time format fixups: tm_year is years since 1900 */
+   time_tm.tm_year=year-1900;
+   /* month should be 0-11 */
+   --(time_tm.tm_mon);
+
+   /* Convert to time_t: make sure TZ is undefined */
+   tz = getenv("TZ");
+   setenv("TZ", "", 1);
+   tzset();
+   mytime = mktime(&time_tm);
+   if (tz)
+       setenv("TZ", tz, 1);
+   else
+       unsetenv("TZ");
+   tzset();
+
+   return mytime;
+}
 
 
 /******************************************************************************
@@ -787,7 +1068,7 @@ void lcmaps_print_x509_to_string(X509 * px509, char *output_file)
 /******************************************************************************
 CVS Information:
     $Source: /srv/home/dennisvd/svn/mw-security/lcmaps/src/grid_credential_handling/gsi_handling/lcmaps_voms_attributes.c,v $
-    $Date: 2012-11-01 21:04:45 +0100 (Thu, 01 Nov 2012) $
-    $Revision: 16744 $
-    $Author: okoeroo $
+    $Date: 2015-01-13 12:28:19 +0100 (Tue, 13 Jan 2015) $
+    $Revision: 18158 $
+    $Author: msalle $
 ******************************************************************************/
