@@ -49,19 +49,24 @@
     DN, GSI (gss_cred_id_t), security context (gss_ctx_id_t), or parsed voms attributes (FQANs)
 */
 
-#ifndef LCMAPS_CREDENTIAL_C
-#define LCMAPS_CREDENTIAL_C
+#define _XOPEN_SOURCE	500
 
 /******************************************************************************
                              Include header files
 ******************************************************************************/
 #include "lcmaps_config.h"
+
+/* Needed for NULL */
 #include <stdio.h>
+
 #include <stdlib.h>
 #include <string.h>
 
+#include <sys/types.h>
+
 #ifdef LCMAPS_GSI_MODE
 #   include <gssapi.h>
+#   include <openssl/x509.h>
 #   include "x509_handling/_lcmaps_x509_utils.h"
 #   include "gsi_handling/_lcmaps_gsi_utils.h"
 #   include "gsi_handling/_lcmaps_voms_attributes.h"
@@ -70,11 +75,14 @@
 
 /* LCMAPS includes */
 #include "lcmaps_log.h"
+#include "lcmaps_account.h"
 
 #ifdef LCMAPS_GSI_MODE
 
+#if 0
 #define TRUE  1
 #define FALSE 0
+#endif
 
 
 /******************************************************************************
@@ -169,7 +177,9 @@ int lcmaps_credential_store_pem_string_and_sub_elements(
     int                 retval          = LCMAPS_CRED_SUCCESS;
     STACK_OF(X509) *    certchain       = NULL;
     X509 *              cert            = NULL;
+#if 0
     X509 *              certPrime       = NULL;
+#endif
 
     /* Store the PEM string */
     retval = lcmaps_credential_store_pem_string(pem_string, plcmaps_credential);
@@ -188,6 +198,7 @@ int lcmaps_credential_store_pem_string_and_sub_elements(
         return LCMAPS_CRED_NO_X509_CHAIN;
     }
 
+#if 0
     /* Select the leaf certificate of the chain as the individual/last certificate */
     certPrime = sk_X509_value(certchain, 0);
     if (!certPrime) {
@@ -200,6 +211,12 @@ int lcmaps_credential_store_pem_string_and_sub_elements(
         lcmaps_log(LOG_ERR, "%s: could not duplicate an individual X509 structure. (fatal error).\n", __func__);
         return LCMAPS_CRED_NO_X509_CRED;
     }
+#endif
+    cert=sk_X509_value(certchain, 0);
+    if (!cert) {
+        lcmaps_log(LOG_ERR, "%s: could not select an individual X509 structure. (fatal error).\n", __func__);
+        return LCMAPS_CRED_NO_X509_CRED;
+    }
 
     /* Store and extract credential derivatives from the X.509 based credentials */
     retval |= lcmaps_credential_store_x509_and_sub_elements(cert, certchain, plcmaps_credential);
@@ -646,11 +663,18 @@ int lcmaps_credential_store_lcmaps_vomsdata (
         return LCMAPS_CRED_INVOCATION_ERROR;
     }
 
+    /* Check if the nvoms itself is >0 */
+    if (lcmaps_vomsdata->nvoms <= 0)
+    {
+        lcmaps_log_debug(1,"lcmaps_credential_store_lcmaps_vomsdata(): nvoms <= 0!\n");
+        return LCMAPS_CRED_INVOCATION_ERROR;
+    }
 
     if (plcmaps_credential->voms_data_list == NULL)
     {
         plcmaps_credential->voms_data_list = (lcmaps_vomsdata_t *) malloc (sizeof (lcmaps_vomsdata_t));
-        plcmaps_credential->voms_data_list->voms = (lcmaps_voms_t *) malloc (sizeof (lcmaps_voms_t) * lcmaps_vomsdata->nvoms);
+	/* lcmaps_vomsdata->nvoms >0 by now */
+        plcmaps_credential->voms_data_list->voms = (lcmaps_voms_t *) malloc (sizeof (lcmaps_voms_t) * (size_t)(lcmaps_vomsdata->nvoms));
 
         for (i = 0; i < lcmaps_vomsdata->nvoms; i++)
         {
@@ -665,25 +689,32 @@ int lcmaps_credential_store_lcmaps_vomsdata (
             plcmaps_credential->voms_data_list->voms[i].date2          = strdup(lcmaps_vomsdata->voms[i].date2);
             plcmaps_credential->voms_data_list->voms[i].voname         = strdup(lcmaps_vomsdata->voms[i].voname);
 
-            plcmaps_credential->voms_data_list->voms[i].fqan_unix      = (lcmaps_fqan_unix_t *) malloc (sizeof(lcmaps_fqan_unix_t) * lcmaps_vomsdata->voms[i].nfqan);
-            plcmaps_credential->voms_data_list->voms[i].nfqan          = lcmaps_vomsdata->voms[i].nfqan;
+	    plcmaps_credential->voms_data_list->voms[i].nfqan          = lcmaps_vomsdata->voms[i].nfqan;
+	    if (lcmaps_vomsdata->voms[i].nfqan > 0) {
+		plcmaps_credential->voms_data_list->voms[i].fqan_unix      = (lcmaps_fqan_unix_t *) malloc (sizeof(lcmaps_fqan_unix_t) * (size_t)(lcmaps_vomsdata->voms[i].nfqan));
 
-            for (j = 0; j < lcmaps_vomsdata->voms[i].nfqan; j++)
-            {
-                plcmaps_credential->voms_data_list->voms[i].fqan_unix[j].fqan = strdup(lcmaps_vomsdata->voms[i].fqan_unix[j].fqan);
-                plcmaps_credential->voms_data_list->voms[i].fqan_unix[j].uid  = lcmaps_vomsdata->voms[i].fqan_unix[j].uid;
-                plcmaps_credential->voms_data_list->voms[i].fqan_unix[j].gid  = lcmaps_vomsdata->voms[i].fqan_unix[j].gid;
-            }
+		for (j = 0; j < lcmaps_vomsdata->voms[i].nfqan; j++)
+		{
+		    plcmaps_credential->voms_data_list->voms[i].fqan_unix[j].fqan = strdup(lcmaps_vomsdata->voms[i].fqan_unix[j].fqan);
+		    plcmaps_credential->voms_data_list->voms[i].fqan_unix[j].uid  = lcmaps_vomsdata->voms[i].fqan_unix[j].uid;
+		    plcmaps_credential->voms_data_list->voms[i].fqan_unix[j].gid  = lcmaps_vomsdata->voms[i].fqan_unix[j].gid;
+		}
+	    } else
+		plcmaps_credential->voms_data_list->voms[i].fqan_unix      = NULL;
 
             plcmaps_credential->voms_data_list->voms[i].nattr = lcmaps_vomsdata->voms[i].nattr;
-            plcmaps_credential->voms_data_list->voms[i].attr_list = (lcmaps_voms_generic_attr_t *) calloc(lcmaps_vomsdata->voms[i].nattr,sizeof(lcmaps_voms_generic_attr_t));
+	    if (lcmaps_vomsdata->voms[i].nattr > 0) {
+		plcmaps_credential->voms_data_list->voms[i].attr_list = (lcmaps_voms_generic_attr_t *) calloc((size_t)(lcmaps_vomsdata->voms[i].nattr),sizeof(lcmaps_voms_generic_attr_t));
+
+		lcmaps_log_debug (3, "-- total # of generic attributes in VO: (%d) \n", plcmaps_credential->voms_data_list->voms[i].nattr);
+		for(j = 0; j < plcmaps_credential->voms_data_list->voms[i].nattr; j++){
+		    plcmaps_credential->voms_data_list->voms[i].attr_list[j].name      = strdup(lcmaps_vomsdata->voms[i].attr_list[j].name);
+		    plcmaps_credential->voms_data_list->voms[i].attr_list[j].value     = strdup(lcmaps_vomsdata->voms[i].attr_list[j].value);
+		    plcmaps_credential->voms_data_list->voms[i].attr_list[j].qualifier = strdup(lcmaps_vomsdata->voms[i].attr_list[j].qualifier);
+		}
+	    } else
+		plcmaps_credential->voms_data_list->voms[i].attr_list = NULL;
 
-            lcmaps_log_debug (3, "-- total # of generic attributes in VO: (%d) \n", plcmaps_credential->voms_data_list->voms[i].nattr);
-            for(j = 0; j < plcmaps_credential->voms_data_list->voms[i].nattr; j++){
-                plcmaps_credential->voms_data_list->voms[i].attr_list[j].name      = strdup(lcmaps_vomsdata->voms[i].attr_list[j].name);
-                plcmaps_credential->voms_data_list->voms[i].attr_list[j].value     = strdup(lcmaps_vomsdata->voms[i].attr_list[j].value);
-                plcmaps_credential->voms_data_list->voms[i].attr_list[j].qualifier = strdup(lcmaps_vomsdata->voms[i].attr_list[j].qualifier);
-            }
 
             plcmaps_credential->voms_data_list->workvo                 = strdup(lcmaps_vomsdata->workvo);
             plcmaps_credential->voms_data_list->extra_data             = strdup(lcmaps_vomsdata->extra_data);
@@ -730,7 +761,8 @@ int lcmaps_credential_store_fqan_list(
     if (plcmaps_credential->fqan == NULL)
     {
         plcmaps_credential->nfqan = nfqan;
-        plcmaps_credential->fqan = (char **) malloc(nfqan * sizeof(char *));
+	/* nfqan >= 1 */
+        plcmaps_credential->fqan = (char **) malloc((size_t)nfqan * sizeof(char *));
         if (plcmaps_credential->fqan == NULL) /* malloc error */
         {
             lcmaps_log_debug(1,"lcmaps.mod-lcmaps_credential_store_fqan_list(): Malloc error!\n");
@@ -1035,20 +1067,18 @@ int lcmaps_release_cred(
     if (plcmaps_credential == NULL)
         return LCMAPS_CRED_SUCCESS;
 
-    if (plcmaps_credential->dn != NULL) {
-        free(plcmaps_credential->dn);
-    }
+    free(plcmaps_credential->dn);
 
     lcmaps_clean_list_of_strings(plcmaps_credential->nfqan, plcmaps_credential->fqan);
     lcmaps_account_info_clean(&(plcmaps_credential->requested_account));
     lcmaps_clean_vomsdata(plcmaps_credential->voms_data_list);
 
 #ifdef LCMAPS_GSI_MODE
+    free(plcmaps_credential->pem_string);
     /* The px509_cred is since 25 januari a reference into the first element of the px509_chain
      * if (plcmaps_credential->px509_cred) X509_free(plcmaps_credential->px509_cred); */
-    if (plcmaps_credential->px509_chain) {
+    if (plcmaps_credential->px509_chain)
         lcmaps_x509_free_chain(&(plcmaps_credential->px509_chain));
-    }
 #endif
 
     return LCMAPS_CRED_SUCCESS;
@@ -1089,12 +1119,10 @@ int lcmaps_release_cred_leave_STACK_OF_X509(
     return LCMAPS_CRED_SUCCESS;
 }
 
-#endif /* LCMAPS_CREDENTIAL_C */
-
 /******************************************************************************
 CVS Information:
     $Source: /srv/home/dennisvd/svn/mw-security/lcmaps/src/grid_credential_handling/lcmaps_credential.c,v $
-    $Date: 2012-03-14 14:00:33 +0100 (Wed, 14 Mar 2012) $
-    $Revision: 16139 $
+    $Date: 2015-02-02 11:46:42 +0100 (Mon, 02 Feb 2015) $
+    $Revision: 18216 $
     $Author: msalle $
 ******************************************************************************/
