@@ -53,27 +53,18 @@
     \internal
 */
 
-#ifndef LCMAPS_RUNVARS_C
-#define LCMAPS_RUNVARS_C
-
 /*****************************************************************************
                             Include header files
 ******************************************************************************/
 #include "lcmaps_config.h"
+
+/* Needed for NULL */
 #include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <pwd.h>
-#ifdef LCMAPS_GSI_MODE
-#   include <gssapi.h>
-#endif
 
 /* LCMAPS includes */
 #include "lcmaps_log.h"
-#include "lcmaps_types.h"
-#include "lcmaps_utils.h"
 #include "lcmaps_arguments.h"
-#include "lcmaps_account.h"
+#include "lcmaps_types.h"
 #include "_lcmaps_runvars.h"
 #include "_lcmaps_credential.h"
 
@@ -179,7 +170,7 @@ int lcmaps_extractRunVars(
     static char *requested_username;
 
     int number_of_runvars=0;
-    char *  logstr = "lcmaps.mod-lcmaps_extractRunVars()";
+    const char * logstr = "lcmaps.mod-lcmaps_extractRunVars()";
 
     /* Get the number of variables
      */
@@ -445,8 +436,8 @@ Returns:        void pointer to the value or NULL
 ******************************************************************************/
 /*!
     \fn lcmaps_getRunVars(
-        char *argName,
-        char *argType
+        const char *argName,
+        const char *argType
         )
     \brief returns a void pointer to the requested value
     
@@ -461,8 +452,8 @@ Returns:        void pointer to the value or NULL
     \internal
 */
 void * lcmaps_getRunVars(
-        char *argName,
-        char *argType
+        const char *argName,
+        const char *argType
 )
 {
     /* do lcmaps_getArgValue with runvars_list and NUMBER_OF_RUNVARS
@@ -483,8 +474,8 @@ Returns:        0 succes
 ******************************************************************************/
 /*!
     \fn lcmaps_setRunVars(
-        char *argName,
-        char *argType,
+        const char *argName,
+        const char *argType,
         void *value
         )
     \brief fill the runvars_list with a value for argName and argType
@@ -502,14 +493,14 @@ Returns:        0 succes
     \internal
 */
 int lcmaps_setRunVars(
-        char *argName,
-        char *argType,
+        const char *argName,
+        const char *argType,
         void *value
 )
 {
     lcmaps_argument_t *pargument=NULL;
 #ifdef LCMAPS_DEBUG
-    char *  logstr = "lcmaps.mod-lcmaps_setRunVars()";
+    const char *  logstr = "lcmaps.mod-lcmaps_setRunVars()";
 #endif
 
     /* store address of 1st element of runvars_list in pargument,
@@ -526,12 +517,10 @@ int lcmaps_setRunVars(
     return lcmaps_setArgValue(argName, argType, value, NUMBER_OF_RUNVARS, &pargument);
 }
 
-#endif /* LCMAPS_RUNVARS_C */
-
 /******************************************************************************
 CVS Information:
     $Source: /srv/home/dennisvd/svn/mw-security/lcmaps/src/grid_credential_handling/lcmaps_runvars.c,v $
-    $Date: 2013-02-21 20:24:33 +0100 (Thu, 21 Feb 2013) $
-    $Revision: 16855 $
+    $Date: 2014-03-13 11:56:06 +0100 (Thu, 13 Mar 2014) $
+    $Revision: 17647 $
     $Author: msalle $
 ******************************************************************************/
