@@ -87,14 +87,18 @@
     -# lcmaps_term(): stop the PluginManager
 */
 
-#ifndef LCMAPS_C
-#define LCMAPS_C
+#define _XOPEN_SOURCE	500
 
 #include "lcmaps_config.h"
+
 #include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
+#include <sys/types.h>
+#include <pwd.h>
+
 #ifdef LCMAPS_GSI_MODE
+#   include <grp.h>
+#   include <stdlib.h>
 #   include <openssl/x509.h>
 #   include <openssl/err.h>
 #   include <gssapi.h>
@@ -105,12 +109,10 @@
 /* #include "lcmaps_account.h" */
 #include "lcmaps.h"
 #include "lcmaps_types.h"
-#include "lcmaps_utils.h"
 #include "lcmaps_cred_data.h"
-#include "_lcmaps_utils.h"
-#include "_lcmaps_pluginmanager.h"
-#include "_lcmaps_log.h"
-#include "_lcmaps_credential.h"
+#include "pluginmanager/_lcmaps_pluginmanager.h"
+#include "pluginmanager/_lcmaps_log.h"
+#include "grid_credential_handling/_lcmaps_credential.h"
 
 /******************************************************************************
                        Define module specific variables
@@ -434,7 +436,7 @@ int lcmaps_run_without_credentials_and_return_username(
         }
         (*usernamep) = strdup(user_info->pw_name);
 	if (*usernamep == NULL)	{
-	    lcmaps_log(LOG_ERR, "%s: Out of memory\n");
+	    lcmaps_log(LOG_ERR, "%s: Out of memory\n", __func__);
 	    goto fail_lcmaps_run_without_credentials_and_return_username;
 	}
     }
@@ -625,7 +627,7 @@ int lcmaps_run_with_fqans_mapcounter_and_return_account(
     if (poolindex_list && (cntpoolindex > 0))
     {
         poolindex = (char *)(*(poolindex_list));
-        lcmaps_log_debug(5, "%s: found %d poolindices starting at address = %p\n", __func__, cntpoolindex, poolindex_list);
+        lcmaps_log_debug(5, "%s: found %d poolindices starting at address = %p\n", __func__, cntpoolindex, (void*)poolindex_list);
         lcmaps_log_debug(5, "%s(): found this poolindex %s\n", __func__, poolindex);
         *poolindexp = poolindex;
     }
@@ -947,7 +949,7 @@ int lcmaps_run_and_return_username(
         }
         (*usernamep) = strdup(user_info->pw_name);
 	if (*usernamep == NULL)	{
-	    lcmaps_log(LOG_ERR, "%s: Out of memory\n");
+	    lcmaps_log(LOG_ERR, "%s: Out of memory\n", __func__);
 	    goto fail_lcmaps_run_and_return_username;
 	}
     }
@@ -1099,11 +1101,11 @@ int lcmaps_run_and_return_poolindex(
     if (tmpptr && (cntpoolindex > 0))
     {
         poolindex = (char *)(*(tmpptr));
-        lcmaps_log_debug(5, "%s(): found %d poolindeces at address = %p\n", __func__, cntpoolindex, tmpptr);
+        lcmaps_log_debug(5, "%s(): found %d poolindeces at address = %p\n", __func__, cntpoolindex, (void*)tmpptr);
         lcmaps_log_debug(5, "lcmaps_run_and_return_poolindex(): found this poolindex %s\n", poolindex);
         *poolindexp = strdup(poolindex);
 	if ( *poolindexp ==NULL)    {
-	    lcmaps_log(LOG_ERR, "%s: Out of memory\n");
+	    lcmaps_log(LOG_ERR, "%s: Out of memory\n", __func__);
 	    goto fail_lcmaps_run_and_return_poolindex;
 	}
     }
@@ -1294,7 +1296,7 @@ int lcmaps_run_with_pem_and_return_account(
         lcmaps_log_debug(5, "%s(): found %d poolindeces at address = %p\n", __func__, cntpoolindex, poolindex);
         *poolindexp = strdup(poolindex);
 	if ( *poolindexp ==NULL)    {
-	    lcmaps_log(LOG_ERR, "%s: Out of memory\n");
+	    lcmaps_log(LOG_ERR, "%s: Out of memory\n", __func__);
 	    goto fail_lcmaps_run_with_pem_and_return_account;
 	}
     }
@@ -1364,7 +1366,7 @@ int lcmaps_run_and_verify_account_from_pem(
     int                 nuid_found;
     gid_t *             pgid_list_found     = NULL;
     int                 npgid_found;
-    gid_t *             sgid_list_found     = NULL;
+/*    gid_t *             sgid_list_found     = NULL;*/
     int                 nsgid_found;
     char *              poolindex_found     = NULL;
     char **             poolindex_list      = NULL;
@@ -1491,17 +1493,17 @@ int lcmaps_run_and_verify_account_from_pem(
     {
         lcmaps_log_debug(1, "%s(): LCMAPS found no secondary groups\n", __func__);
     }
-    else
+/*    else
     {
         sgid_list_found = secGid;
-    }
+    }*/
 
     /* And finally the poolindex */
     poolindex_list = (char **) getCredentialData(POOL_INDEX, &cntpoolindex);
     if (poolindex_list && (cntpoolindex > 0))
     {
         poolindex_found = (char *)(*(poolindex_list));
-        lcmaps_log_debug(5, "%s(): found %d poolindeces starting at address = %p\n", __func__, cntpoolindex, poolindex_list);
+        lcmaps_log_debug(5, "%s(): found %d poolindeces starting at address = %p\n", __func__, cntpoolindex, (void*)poolindex_list);
         lcmaps_log_debug(5, "%s(): found this poolindex %s\n", __func__, poolindex_found);
     }
     else
@@ -1703,7 +1705,7 @@ int lcmaps_run_with_stack_of_x509_and_return_account(
     }
     /* Then the primary gids */
     priGid           = (gid_t *) getCredentialData(PRI_GID, &cntPriGid);
-    if (priGid == NULL)
+    if (priGid == NULL || cntPriGid<=0)
     {
         lcmaps_log_debug(1, "%s() error: LCMAPS could not find any gid, at least one required!\n", __func__);
         goto fail_lcmaps_run_with_stack_of_x509_and_return_account;
@@ -1711,7 +1713,8 @@ int lcmaps_run_with_stack_of_x509_and_return_account(
     else
     {
         *pnpgid = cntPriGid;
-        *ppgid_list = malloc(sizeof(gid_t) * cntPriGid);
+	/* Note: cntPriGid>0 */
+        *ppgid_list = malloc(sizeof(gid_t) * (size_t)cntPriGid);
 	if (*ppgid_list == NULL)    {
 	    lcmaps_log(LOG_ERR, "%s: Out of memory\n",__func__);
 	    goto fail_lcmaps_run_with_stack_of_x509_and_return_account;
@@ -1724,14 +1727,15 @@ int lcmaps_run_with_stack_of_x509_and_return_account(
     }
     /* Then the secondary gids */
     secGid           = (gid_t *) getCredentialData(SEC_GID, &cntSecGid);
-    if (secGid == NULL)
+    if (secGid == NULL || cntSecGid<=0)
     {
         lcmaps_log_debug(1, "%s(): LCMAPS found no secondary groups\n", __func__);
     }
     else
     {
         *pnsgid = cntSecGid;
-        *psgid_list = malloc(sizeof(gid_t) * cntSecGid);
+	/* Note: cntPriGid>0 */
+        *psgid_list = malloc(sizeof(gid_t) * (size_t)cntSecGid);
 	if (*psgid_list == NULL)    {
 	    lcmaps_log(LOG_ERR, "%s: Out of memory\n",__func__);
 	    goto fail_lcmaps_run_with_stack_of_x509_and_return_account;
@@ -1812,14 +1816,10 @@ int lcmaps_get_patch_version (void)
 }
 
 
-
-
-#endif /* LCMAPS_C */
-
 /******************************************************************************
 CVS Information:
     $Source: /srv/home/dennisvd/svn/mw-security/lcmaps/src/lcmaps.c,v $
-    $Date: 2013-02-21 20:24:33 +0100 (Thu, 21 Feb 2013) $
-    $Revision: 16855 $
+    $Date: 2014-07-07 21:28:25 +0200 (Mon, 07 Jul 2014) $
+    $Revision: 17839 $
     $Author: msalle $
 ******************************************************************************/
