@@ -50,40 +50,39 @@
     -# lcmaps_return_account_from_pem: interface function
 */
 
-#ifndef LCMAPS_RETURN_ACCOUNT_FROM_PEM_C
-#define LCMAPS_RETURN_ACCOUNT_FROM_PEM_C
+#define _XOPEN_SOURCE	600
 
 /******************************************************************************
                              Include header files
 ******************************************************************************/
 #include "lcmaps_config.h"
+
+/* Needed for NULL */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
-#include <time.h>
-
+#include <sys/types.h>
 
 /* LCMAPS includes */
 #include "lcmaps.h"
 #include "lcmaps_log.h"
+#include "lcmaps_account.h"
 #include "lcmaps_return_account_from_pem.h"
 
+#include "pluginmanager/_lcmaps_utils.h"
+#include "pluginmanager/_lcmaps_log.h"
+
 /******************************************************************************
                              Module specific defines
 ******************************************************************************/
 /* Default settings */
 #define LCMAPS_MAX_POLICIES 10
-#define LCMAPS_LOG_FILE "/var/log/lcmaps-suexec.log"
-/* #define LCMAPS_LOG_STRING "return_account_from_pem" */
-#define LCMAPS_BUFSIZ 8192
-#define LCMAPS_LOG_BUFSIZ 2048
 #define LCMAPS_DEFAULT_DB_FILE "lcmaps.db"
-#define LCMAPS_REQUIRED_DB_PATH LCMAPS_ETC_HOME
-
-/* Flags to indicate which type of logging is required */
-#define DO_USRLOG           ((unsigned short)0x0001)
-#define DO_SYSLOG           ((unsigned short)0x0002)
+#if defined(DONT_ALLOW_ABSOLUTE_LCMAPS_DB_FILE) ||\
+    defined(CHECK_LCMAPS_DB_PERMISSIONS)
+# define LCMAPS_REQUIRED_DB_PATH LCMAPS_ETC_HOME
+#endif
 
 /*
  * If the following is set, LCMAPS will not allow the environment variable
@@ -97,29 +96,6 @@
  */
 #define CHECK_LCMAPS_DB_PERMISSIONS 0
 
-/******************************************************************************
-                       Define module specific variables
-******************************************************************************/
-#if 0
-static FILE *           lcmaps_log_stream = NULL;      /* stream for LCMAPS logging */
-#endif
-#if 0
-static char *           lcmaps_log_string = NULL;      /* (constant) log string to be prepended to log information */
-#endif
-static char *           lcmaps_log_file = NULL;
-
-/******************************************************************************
-                          Module specific prototypes
-******************************************************************************/
-/*
-static int           start_logging();
-static int           log_line(int , char * , ...);
-static int           log_a_string(int , char * , char *);
-static int           end_logging();
-*/
-extern int lcmaps_tokenize( const char * command, char ** args, int * n, char * sep);
-
-
 /******************************************************************************
 Function:   lcmaps_return_account_from_pem_va
 Description:
@@ -150,10 +126,10 @@ static int lcmaps_return_account_from_pem_va(
     char *                              lcmaps_policy_string = NULL;
     char *                              lcmaps_policies[LCMAPS_MAX_POLICIES];
     int                                 lcmaps_npols = LCMAPS_MAX_POLICIES;
-    char *                              lcmaps_db_file_env = NULL;
+    const char *                        lcmaps_db_file_env = NULL;
     char *                              lcmaps_db_file = NULL;
     int                                 rc = -1;
-    uid_t                               uid = -1;
+    uid_t                               uid = (uid_t)-1;
     gid_t *                             pgid_list = NULL;
     int                                 npgid = 0;
     gid_t *                             sgid_list = NULL;
@@ -161,74 +137,42 @@ static int lcmaps_return_account_from_pem_va(
     char *                              poolindex = NULL;
     int                                 jj = 0;
 
-#if 0
-    time_t                              my_clock;
-    struct tm  *                        timestamp = NULL;
-    size_t                              string_size;
-    char *                              lcmaps_log_string_default = NULL;
-#endif
 #if CHECK_LCMAPS_DB_PERMISSIONS
-    struct stat                         prg_info;   /* program info holder       */
-    struct stat                         dir_info;   /* directory info holder     */
+    struct stat                         prg_info;   /* program info holder   */
+    struct stat                         dir_info;   /* directory info holder */
 #endif
 
-    /*
-     * Take log file from environment variables.
-     * If not defined take the default ones
-     */
-    lcmaps_log_file = getenv("LCMAPS_LOG_FILE");
-    lcmaps_log_file = (lcmaps_log_file ? lcmaps_log_file : LCMAPS_LOG_FILE );
-
-    /*
-     * The other environment variables should in principle not be touched,
-     * because reasonably defaults are provided by the LCMAPS core.
-     * An exception is made for LCMAPS_LOG_STRING, because we want to enforce
-     * that it is logged which interface is used.
-     * For traceability also add a timestamp.
-     * setenv with overwrite=0 only overwrites if the variable is non-existant
-     */
-#if 0
-    time(&my_clock);
-    /* timestamp = localtime(&my_clock); */
-    timestamp = gmtime(&my_clock);
-    string_size = strlen(LCMAPS_LOG_STRING) + strlen(": ") + 19 + 1;
-    lcmaps_log_string_default = (char *) malloc(string_size * sizeof(char));
-
-    snprintf(lcmaps_log_string_default, string_size, "%s: %04d-%02d-%02d.%02d:%02d:%02d",
-        LCMAPS_LOG_STRING,
-        timestamp->tm_year + 1900, timestamp->tm_mon + 1, timestamp->tm_mday,
-        timestamp->tm_hour, timestamp->tm_min, timestamp->tm_sec);
-
-    setenv("LCMAPS_LOG_STRING", lcmaps_log_string_default, 0);
-    lcmaps_log_string = getenv("LCMAPS_LOG_STRING");
-
-    if (lcmaps_log_string_default != NULL)
+    /* First initialize LCMAPS, initialize without file pointer or name which
+     * will result in getting env var LCMAPS_LOG_FILE. Specify DO_USRLOG to try
+     * that first, if that fails we will go automatically to syslog */
+    if (lcmaps_init_and_logfile(NULL, NULL, (unsigned short)(DO_USRLOG)))
     {
-        free(lcmaps_log_string_default);
-        lcmaps_log_string_default = NULL;
+        lcmaps_log(LOG_ERR, "%s: LCMAPS initialization failure\n", __func__);
+        goto return_account_from_pem_error;
     }
-#endif
 
     /*
-     * Set the lcmaps_db_file. If we're very paranoid we set DONT_ALLOW_ABSOLUTE_LCMAPS_DB_FILE, but normally
-     * this is not needed, because the calling application (glexec) should take care that the
+     * Set the lcmaps_db_file. If we're very paranoid we set
+     * DONT_ALLOW_ABSOLUTE_LCMAPS_DB_FILE, but normally this is not needed,
+     * because the calling application (glexec) should take care that the
      * environment variables are set correctly
      */
 #if DONT_ALLOW_ABSOLUTE_LCMAPS_DB_FILE
     /*
-     * "Another exception is made for the lcmaps.db file: This interface is used by
-     *  VO services (i.e. not site-controlled services), so the VO should not be able
-     *  to determine which lcmaps.db file (or in the future a general lcmaps config file)
-     *  they take."
+     * "Another exception is made for the lcmaps.db file: This interface is used
+     * by VO services (i.e. not site-controlled services), so the VO should not
+     * be able to determine which lcmaps.db file (or in the future a general
+     * lcmaps config file) they take."
      */
     if ( (lcmaps_db_file_env = getenv("LCMAPS_DB_FILE")) != NULL)
     {
         /*
          * OK, one tries to set the location of the lcmaps.db file.
-         * We don't allow paths to be included in the filename. LCMAPS will add the
-         * site-controlled path to it.
+	 * We don't allow paths to be included in the filename. LCMAPS will add
+	 * the site-controlled path to it.
          * If a path is found: error
-         * If no path is found: add the default path to it and performs various checks
+	 * If no path is found: add the default path to it and performs various
+	 * checks
          */
         if (strchr(lcmaps_db_file_env, '/') != NULL)
         {
@@ -262,8 +206,9 @@ static int lcmaps_return_account_from_pem_va(
 
     /*
      * Check the lcmaps_db_file.
-     * Again, if we're very paranoid we set CHECK_LCMAPS_DB_PERMISSIONS, but normally
-     * this is not needed, because the calling application (glexec) should do that.
+     * Again, if we're very paranoid we set CHECK_LCMAPS_DB_PERMISSIONS, but
+     * normally this is not needed, because the calling application (glexec)
+     * should do that.
      */
 #if CHECK_LCMAPS_DB_PERMISSIONS
     /*
@@ -273,7 +218,8 @@ static int lcmaps_return_account_from_pem_va(
      * If checks are ok reset environment variable
      */
     /*
-     * Stat the directory of lcmaps.db and verify it is a directory, or error out.
+     * Stat the directory of lcmaps.db and verify it is a directory, or error
+     * out.
      */
     if (((lstat(LCMAPS_REQUIRED_DB_PATH, &dir_info)) != 0) || !(S_ISDIR(dir_info.st_mode)))
     {
@@ -351,14 +297,6 @@ static int lcmaps_return_account_from_pem_va(
     }
     va_end(ap);
 
-#if 0
-    if (start_logging())
-    {
-        fprintf(stderr, "%s: Cannot start logging\n", lcmaps_log_string);
-        return 1;
-    }
-#endif
-
     /* Parse policy environment variable */
     for (jj=0; jj < LCMAPS_MAX_POLICIES; jj++) lcmaps_policies[jj] = 0;
     lcmaps_policy_string = getenv("LCMAPS_POLICY_NAME");
@@ -387,15 +325,6 @@ static int lcmaps_return_account_from_pem_va(
         goto return_account_from_pem_error;
     }
 
-    /* First initialize LCMAPS (used to use lcmaps_log_stream but that's always
-     * NULL */
-    if (lcmaps_init_and_log(NULL, DO_USRLOG|DO_SYSLOG))
-    {
-        lcmaps_log(LOG_ERR, "%s: LCMAPS initialization failure\n", __func__);
-        goto return_account_from_pem_error;
-    }
-
-
     /*
      * Now that we have the credential let's run (and terminate) LCMAPS !
      */
@@ -415,7 +344,6 @@ static int lcmaps_return_account_from_pem_va(
     );
     if (rc != 0)
     {
-        /* lcmaps_log (1, "LCMAPS failed to do mapping and return account information\n"); */
         lcmaps_log (LOG_ERR, "LCMAPS failed to do mapping and return account information\n");
         if (lcmaps_term())
         {
@@ -457,14 +385,6 @@ static int lcmaps_return_account_from_pem_va(
         goto return_account_from_pem_error;
     }
 
-#if 0
-    if (end_logging())
-    {
-        fprintf(stderr, "%s: Cannot cleanly end lcmaps logging\n", lcmaps_log_string);
-        goto return_account_from_pem_error;
-    }
-#endif
-
     /* Clean policies */
     for (jj=0; jj < LCMAPS_MAX_POLICIES; jj++)
     {
@@ -487,13 +407,6 @@ static int lcmaps_return_account_from_pem_va(
             lcmaps_policies[jj] = NULL;
         }
     }
-#if 0
-    if (end_logging())
-    {
-        fprintf(stderr, "%s: Cannot cleanly end lcmaps logging\n", lcmaps_log_string);
-    }
-#endif
-
     return 1;
 }
 
@@ -524,12 +437,10 @@ int lcmaps_return_account_from_pem(
     return (lcmaps_return_account_from_pem_va(3, pem_string, mapcounter, plcmaps_account));
 }
 
-#endif /* LCMAPS_RETURN_ACCOUNT_FROM_PEM_C */
-
 /******************************************************************************
 CVS Information:
     $Source: /srv/home/dennisvd/svn/mw-security/lcmaps/src/lcmaps_return_account_from_pem.c,v $
-    $Date: 2012-03-16 16:16:23 +0100 (Fri, 16 Mar 2012) $
-    $Revision: 16177 $
+    $Date: 2014-03-04 16:17:33 +0100 (Tue, 04 Mar 2014) $
+    $Revision: 17597 $
     $Author: msalle $
 ******************************************************************************/
