@@ -1,18 +1,18 @@
 /**
- * Copyright (c) Members of the EGEE Collaboration. 2004-2010. 
+ * Copyright (c) Members of the EGEE Collaboration. 2004-2010.
  * See http://www.eu-egee.org/partners/ for details on the copyright
- * holders.  
- * 
- * Licensed under the Apache License, Version 2.0 (the "License"); 
- * you may not use this file except in compliance with the License. 
- * You may obtain a copy of the License at 
- * 
- *     http://www.apache.org/licenses/LICENSE-2.0 
- * 
- * Unless required by applicable law or agreed to in writing, software 
- * distributed under the License is distributed on an "AS IS" BASIS, 
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
- * See the License for the specific language governing permissions and 
+ * holders.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
  * limitations under the License.
  *
  *
@@ -22,7 +22,7 @@
  *     Mischa Sall\'e <msalle@nikhef.nl>
  *     David Groep <davidg@nikhef.nl>
  *     NIKHEF Amsterdam, the Netherlands
- *     <grid-mw-security@nikhef.nl> 
+ *     <grid-mw-security@nikhef.nl>
  *
  *  2007-2009
  *     Oscar Koeroo <okoeroo@nikhef.nl>
@@ -56,15 +56,13 @@
                 gss credential, buffer and buffer length
 */
 
-#ifndef LCMAPS_RETURN_POOLINDEX_C
-#define LCMAPS_RETURN_POOLINDEX_C
-
 #include "lcmaps_config.h"
 
+/* Needed for NULL */
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include <string.h>
-#include <errno.h>
 
 /* globus headers */
 #include <gssapi.h>
@@ -75,6 +73,9 @@
 #include "lcmaps_log.h"
 #include "lcmaps_return_poolindex.h"
 
+#include "pluginmanager/_lcmaps_log.h"
+#include "pluginmanager/_lcmaps_utils.h"
+
 /* Next header defines functions needed to dig into the internal globus structs
  * Actual structs are defined in globus_internal.h */
 #include "grid_credential_handling/gsi_handling/_lcmaps_gsi_utils.h"
@@ -84,27 +85,12 @@
 ******************************************************************************/
 /* Default settings */
 #define LCMAPS_MAX_POLICIES 10
-/* #define LCMAPS_LOG_FILE "/var/log/lcmaps.log" */
-/* #define LCMAPS_LOG_STRING "poolindex" */
-/* #define LCMAPS_BUFSIZ 8192 */
-/* #define LCMAPS_LOG_BUFSIZ 2048 */
-
-/* Flags to indicate which type of logging is required */
-#define DO_USRLOG           ((unsigned short)0x0001)
-#define DO_SYSLOG           ((unsigned short)0x0002)
-
-/******************************************************************************
-                       Define module specific variables
-******************************************************************************/
-/* static FILE *           lcmaps_log_stream = NULL; */ /* stream for LCMAPS logging */
-/* static char *           lcmaps_log_file = NULL; */
 
 /******************************************************************************
                           Module specific prototypes
 ******************************************************************************/
 static gss_cred_id_t dig_up_gss_cred_from_context(gss_ctx_id_t);
 static int           lcmaps_return_poolindex_va(int , ...);
-extern int lcmaps_tokenize( const char * command, char ** args, int * n, char * sep);
 
 /******************************************************************************
 Function:       Dig up the user gss credential from the security context
@@ -190,7 +176,9 @@ static gss_cred_id_t dig_up_gss_cred_from_context(
 
     gss_release_name(&minor_status, &peer);
 
-    lcmaps_log (LOG_INFO, "Found name of peer: %s\n", peer_name_buffer.value);
+    /* peer_name_buffer.value is a void* */
+    lcmaps_log (LOG_INFO, "Found name of peer: %s\n",
+		(char*)(peer_name_buffer.value));
     /*
      * Sometimes the delegated credential returned by
      * globus_gss_assist_accept_sec_context()/gss_accept_sec_context() is empty.
@@ -245,7 +233,7 @@ Returns:
     The arguments of this function are passed as a variable argument list.
 
     \param narg  the number of arguments that follow
-    \param ap    va_arg list. This list currently contains:
+        This list currently contains:
         -# The GSS Security context established during invocation of the calling
            service. This parameter is of type gss_ctx_id_t.
         -# A pointer to a buffer. This buffer will contain the poolindex (NUL
@@ -265,7 +253,7 @@ static int lcmaps_return_poolindex_va(
     va_list                             ap;
     gss_ctx_id_t                        context_handle = GSS_C_NO_CONTEXT;
     char *                              poolindex_buffer = NULL;
-    unsigned int                        buffer_length = 0;
+    size_t                              buffer_length = 0;
     char *                              lcmaps_policy_string = NULL;
     char *                              lcmaps_policies[LCMAPS_MAX_POLICIES];
     int                                 lcmaps_npols = LCMAPS_MAX_POLICIES;
@@ -274,19 +262,29 @@ static int lcmaps_return_poolindex_va(
     char *                              poolindex = NULL;
     int                                 jj = 0;
 
+    /* First initialize LCMAPS, initialize without file pointer or name which
+     * will result in getting env var LCMAPS_LOG_FILE. Specify DO_USRLOG to try
+     * that first, if that fails we will go automatically to syslog */
+    if (lcmaps_init_and_logfile(NULL, NULL, (unsigned short)(DO_USRLOG)))
+    {
+        lcmaps_log(LOG_ERR, "%s: LCMAPS initialization failure\n", __func__);
+        return 1;
+    }
+
+
     /* Parse arguments from va_list */
     va_start(ap, narg);
     if (narg == 3)
     {
         context_handle = va_arg(ap, gss_ctx_id_t);
         poolindex_buffer = va_arg(ap, char *);
-        buffer_length = va_arg(ap, unsigned int);
+        buffer_length = (size_t)va_arg(ap, unsigned int);
     }
     else if (narg == 4)
     {
         context_handle = va_arg(ap, gss_ctx_id_t);
         poolindex_buffer = va_arg(ap, char *);
-        buffer_length = va_arg(ap, unsigned int);
+        buffer_length = (size_t)va_arg(ap, unsigned int);
         user_cred_handle = va_arg(ap, gss_cred_id_t);
     }
     else
@@ -315,7 +313,7 @@ static int lcmaps_return_poolindex_va(
                 lcmaps_log(LOG_ERR, "%s: of a non-matching quote\n", __func__);
                 break;
             case -4:
-                lcmaps_log(LOG_ERR, "%s: invalid input\n", __func__);
+                lcmaps_log(LOG_ERR, "%s: of invalid input\n", __func__);
                 break;
             default:
                 lcmaps_log(LOG_ERR, "%s: of an unknown error\n", __func__);
@@ -324,13 +322,6 @@ static int lcmaps_return_poolindex_va(
         goto return_poolindex_error;
     }
 
-    /* First initialize LCMAPS */
-    if (lcmaps_init_and_log(NULL, DO_USRLOG|DO_SYSLOG))
-    {
-        lcmaps_log(LOG_ERR, "%s: LCMAPS initialization failure\n", __func__);
-        return 1;
-    }
-
     /* 
      * Retrieve clients (delegated) credential
      * Check if by any chance the gss credential was directly given to us and
@@ -386,15 +377,15 @@ static int lcmaps_return_poolindex_va(
     {
         if(strlen(poolindex) + 1 > buffer_length)
         {
-            lcmaps_log (LOG_ERR, "Buffer (length=%d) too small for poolindex (length=%d)\n",
-                       buffer_length-1, poolindex);
+            lcmaps_log (LOG_ERR, "Buffer (length=%lu) too small for poolindex (length=%lu)\n",
+                       (long unsigned)(buffer_length-1), (long unsigned)strlen(poolindex));
             free(poolindex);
             goto return_poolindex_error;
         }
         else
         {
             strncpy(poolindex_buffer, poolindex, buffer_length);
-            lcmaps_log_a_string(LOG_DEBUG, "lcmaps_return_poolindex(): Returning poolindex %s\n", poolindex);
+            lcmaps_log(LOG_DEBUG, "lcmaps_return_poolindex(): Returning poolindex %s\n", poolindex);
         }
         free(poolindex);
     }
@@ -487,12 +478,10 @@ int lcmaps_return_poolindex_from_gss_cred(
            );
 }
 
-#endif /* LCMAPS_RETURN_POOLINDEX_C */
-
 /******************************************************************************
 CVS Information:
     $Source: /srv/home/dennisvd/svn/mw-security/lcmaps/src/lcmaps_return_poolindex.c,v $
-    $Date: 2012-01-26 12:07:08 +0100 (Thu, 26 Jan 2012) $
-    $Revision: 15876 $
+    $Date: 2015-02-02 11:46:42 +0100 (Mon, 02 Feb 2015) $
+    $Revision: 18216 $
     $Author: msalle $
 ******************************************************************************/
