@@ -1,18 +1,18 @@
 /**
- * Copyright (c) Members of the EGEE Collaboration. 2004-2010. 
+ * Copyright (c) Members of the EGEE Collaboration. 2004-2010.
  * See http://www.eu-egee.org/partners/ for details on the copyright
- * holders.  
- * 
- * Licensed under the Apache License, Version 2.0 (the "License"); 
- * you may not use this file except in compliance with the License. 
- * You may obtain a copy of the License at 
- * 
- *     http://www.apache.org/licenses/LICENSE-2.0 
- * 
- * Unless required by applicable law or agreed to in writing, software 
- * distributed under the License is distributed on an "AS IS" BASIS, 
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
- * See the License for the specific language governing permissions and 
+ * holders.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
  * limitations under the License.
  *
  *
@@ -22,7 +22,7 @@
  *     Mischa Sall\'e <msalle@nikhef.nl>
  *     David Groep <davidg@nikhef.nl>
  *     NIKHEF Amsterdam, the Netherlands
- *     <grid-mw-security@nikhef.nl> 
+ *     <grid-mw-security@nikhef.nl>
  *
  *  2007-2009
  *     Oscar Koeroo <okoeroo@nikhef.nl>
@@ -54,57 +54,39 @@
                 security context, buffer and buffer length
 */
 
-#ifndef LCMAPS_RETURN_POOLINDEX_WITHOUT_GSI_C
-#define LCMAPS_RETURN_POOLINDEX_WITHOUT_GSI_C
-
 /******************************************************************************
                              Include header files
 ******************************************************************************/
 #include "lcmaps_config.h"
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
 #include <stdarg.h>
-#include <errno.h>
-#include <time.h>
-#include "lcmaps_log.h"
+#include <sys/types.h>
 
 /* LCMAPS includes */
 #include "lcmaps.h"
+#include "lcmaps_account.h"
+#include "lcmaps_log.h"
 #include "lcmaps_return_poolindex_without_gsi.h"
 
+#include "pluginmanager/_lcmaps_log.h"
+#include "pluginmanager/_lcmaps_utils.h"
+
 /******************************************************************************
                              Module specific defines
 ******************************************************************************/
 /* Default settings */
 #define LCMAPS_MAX_POLICIES 10
-#define LCMAPS_LOG_FILE "/var/log/lcmaps.log"
-/* #define LCMAPS_LOG_STRING "poolindex" */
-#define LCMAPS_BUFSIZ 8192
-#define LCMAPS_LOG_BUFSIZ 2048
-
-/* Flags to indicate which type of logging is required */
-#define DO_USRLOG              ((unsigned short)0x0001)
-#define DO_SYSLOG              ((unsigned short)0x0002)
 
 /* Flags to influence behaviour of the interface */
 #define DONT_REQUIRE_POOLINDEX ((unsigned short)0x0000)
 #define REQUIRE_POOLINDEX      ((unsigned short)0x0001)
 
-/******************************************************************************
-                       Define module specific variables
-******************************************************************************/
-static int              do_syslog = 1;                 /* use syslog for LCMAPS logging */
-static int              lcmaps_log_stream_exists = 0;  /* 1 if log_stream already exists at startup */
-static FILE *           lcmaps_log_stream = NULL;      /* stream for LCMAPS logging */
-static char *           lcmaps_log_string = NULL;      /* (constant) log string to be prepended to log information */
-static char *           lcmaps_log_file = NULL;
 
 /******************************************************************************
                           Module specific prototypes
 ******************************************************************************/
 static int           lcmaps_return_poolindex_without_gsi_va(int , ...);
-extern int lcmaps_tokenize( const char * command, char ** args, int * n, char * sep);
 
 /******************************************************************************
 Function:   lcmaps_return_poolindex_without_gsi_va
@@ -165,7 +147,7 @@ static int lcmaps_return_poolindex_without_gsi_va(
     char *                              lcmaps_policies[LCMAPS_MAX_POLICIES];
     int                                 lcmaps_npols = LCMAPS_MAX_POLICIES;
     int                                 rc = -1;
-    uid_t                               uid = -1;
+    uid_t                               uid = (uid_t)-1;
     gid_t *                             pgid_list = NULL;
     int                                 npgid = 0;
     gid_t *                             sgid_list = NULL;
@@ -173,49 +155,15 @@ static int lcmaps_return_poolindex_without_gsi_va(
     char *                              poolindex = NULL;
     int                                 jj = 0;
 
-#if 0
-    time_t                              clock;
-    struct tm  *                        timestamp = NULL;
-    size_t                              string_size; 
-    char *                              lcmaps_log_string_default = NULL; 
-#endif
-
-    /*
-     * Take log file from environment variables.
-     * If not defined take the default ones
-     */
-    lcmaps_log_file = getenv("LCMAPS_LOG_FILE");
-    lcmaps_log_file = (lcmaps_log_file ? lcmaps_log_file : LCMAPS_LOG_FILE );
-
-    /*
-     * The other environment variables should in principle not be touched,
-     * because reasonably defaults are provided by the LCMAPS core.
-     * An exception is made for LCMAPS_LOG_STRING, because we want to enforce
-     * that it is logged which interface is used.
-     * For traceability also add a timestamp.
-     * setenv with overwrite=0 only overwrites if the variable is non-existant
-     */
-#if 0
-    time(&clock);
-    /* timestamp = localtime(&clock); */
-    timestamp = gmtime(&clock); 
-    string_size = strlen(LCMAPS_LOG_STRING) + strlen(": ") + 19 + 1;
-    lcmaps_log_string_default = (char *) malloc(string_size * sizeof(char));
-
-    snprintf(lcmaps_log_string_default, string_size, "%s: %04d-%02d-%02d.%02d:%02d:%02d",
-        LCMAPS_LOG_STRING,
-        timestamp->tm_year + 1900, timestamp->tm_mon + 1, timestamp->tm_mday,
-        timestamp->tm_hour, timestamp->tm_min, timestamp->tm_sec);
-
-    setenv("LCMAPS_LOG_STRING", lcmaps_log_string_default, 0);
-    lcmaps_log_string = getenv("LCMAPS_LOG_STRING");
-
-    if (lcmaps_log_string_default != NULL)
+    /* First initialize LCMAPS, initialize without file pointer or name which
+     * will result in getting env var LCMAPS_LOG_FILE. Specify DO_USRLOG to try
+     * that first, if that fails we will go automatically to syslog */
+    if (lcmaps_init_and_logfile(NULL, NULL, (unsigned short)(DO_USRLOG)))
     {
-        free(lcmaps_log_string_default);
-        lcmaps_log_string_default = NULL;
+        lcmaps_log(LOG_ERR, "%s: LCMAPS initialization failure\n", __func__);
+        return 1;
     }
-#endif
+
 
     /* Parse arguments from va_list */
     va_start(ap, narg);
@@ -231,7 +179,7 @@ static int lcmaps_return_poolindex_without_gsi_va(
         }
         if (narg >=6)
         {
-            flags = va_arg(ap, unsigned int);
+            flags = (unsigned short)va_arg(ap, unsigned int);
         }
     }
     else
@@ -241,50 +189,34 @@ static int lcmaps_return_poolindex_without_gsi_va(
     }
     va_end(ap);
 
-#if 0
-    if (start_logging())
-    {
-        fprintf(stderr, "%s: Cannot start logging\n", lcmaps_log_string);
-        return 1;
-    }
-#endif
-
     /* Parse policy environment variable */
     for (jj=0; jj < LCMAPS_MAX_POLICIES; jj++) lcmaps_policies[jj] = 0;
     lcmaps_policy_string = getenv("LCMAPS_POLICY_NAME");
     if ((rc = lcmaps_tokenize(lcmaps_policy_string, lcmaps_policies, &lcmaps_npols, ":")) != 0)
     {
-        lcmaps_log(LOG_ERR, "%s: Cannot parse LCMAPS_POLICY_NAME environment variable, because\n", lcmaps_log_string);
+        lcmaps_log(LOG_ERR, "%s: Cannot parse LCMAPS_POLICY_NAME environment variable, because\n", __func__);
         switch (rc)
         {
             case -1:
-                lcmaps_log(LOG_ERR, "%s: of a malloc error\n", lcmaps_log_string);
+                lcmaps_log(LOG_ERR, "%s: of a malloc error\n", __func__);
                 break;
             case -2:
                 lcmaps_log(LOG_ERR, "%s: the policy list is too large (max = %d)\n",
-                    lcmaps_log_string, LCMAPS_MAX_POLICIES);
+                    __func__, LCMAPS_MAX_POLICIES);
                 break;
             case -3:
-                lcmaps_log(LOG_ERR, "%s: of a non-matching quote\n", lcmaps_log_string);
+                lcmaps_log(LOG_ERR, "%s: of a non-matching quote\n", __func__);
                 break;
             case -4:
-                lcmaps_log(LOG_ERR, "%s: of invalid input\n", lcmaps_log_string);
+                lcmaps_log(LOG_ERR, "%s: of invalid input\n", __func__);
                 break;
             default:
-                lcmaps_log(LOG_ERR, "%s: of an unknown error\n", lcmaps_log_string);
+                lcmaps_log(LOG_ERR, "%s: of an unknown error\n", __func__);
                 break;
         }
         goto return_poolindex_without_gsi_error;
     }
 
-    /* First initialize LCMAPS */
-    if (lcmaps_init_and_log(lcmaps_log_stream, DO_USRLOG|DO_SYSLOG))
-    {
-        lcmaps_log(LOG_ERR, "%s: LCMAPS initialization failure\n", lcmaps_log_string);
-        goto return_poolindex_without_gsi_error;
-    }
-
-
     /*
      * Now that we have the credential let's run (and terminate) LCMAPS !
      */
@@ -325,17 +257,18 @@ static int lcmaps_return_poolindex_without_gsi_va(
     }
     if (rc != 0)
     {
-        lcmaps_log (LOG_NOTICE, "LCMAPS failed to do mapping and return account information\n");
+        lcmaps_log (LOG_ERR, "LCMAPS failed to do mapping and return account information\n");
         if (lcmaps_term())
         {
-            lcmaps_log (LOG_ERR, "LCMAPS termination failure\n"); 
+            lcmaps_log (LOG_ERR, "LCMAPS termination failure\n");
             goto return_poolindex_without_gsi_error;
         }
         goto return_poolindex_without_gsi_error;
     }
     /*
      *  Check if a poolindex has been found.
-     *  If this is not the case and the flag REQUIRE_POOLINDEX has been set: Error.
+     *  If this is not the case and the flag REQUIRE_POOLINDEX has been set:
+     *  Error.
      */
     if ( (poolindex == NULL) && ((flags & REQUIRE_POOLINDEX) == REQUIRE_POOLINDEX) )
     {
@@ -526,12 +459,10 @@ int lcmaps_return_account_without_gsi(
                 DONT_REQUIRE_POOLINDEX));
 }
 
-#endif /* LCMAPS_RETURN_POOLINDEX_WITHOUT_GSI_C */
-
 /******************************************************************************
 CVS Information:
     $Source: /srv/home/dennisvd/svn/mw-security/lcmaps/src/lcmaps_return_poolindex_without_gsi.c,v $
-    $Date: 2012-01-04 15:32:25 +0100 (Wed, 04 Jan 2012) $
-    $Revision: 15826 $
+    $Date: 2014-03-13 11:56:06 +0100 (Thu, 13 Mar 2014) $
+    $Revision: 17647 $
     $Author: msalle $
 ******************************************************************************/
