@@ -55,16 +55,11 @@
 #ifndef _LCMAPS_PLUGINMANAGER_H
 #define _LCMAPS_PLUGINMANAGER_H
 
-#ifndef LCMAPS_PLUGIN_MANAGER_C
-#   define LCMAPS_PLUGIN_MANAGER_C_STORAGE_CLASS extern
-#else
-#   define LCMAPS_PLUGIN_MANAGER_C_STORAGE_CLASS
-#endif
 /******************************************************************************
                              Include header files
 ******************************************************************************/
 #include "lcmaps_types.h"
-#include "_lcmaps_credential.h"
+#include "grid_credential_handling/_lcmaps_credential.h"
 
 /******************************************************************************
  *                 Define constants
@@ -88,7 +83,7 @@ Returns:
     0: succes
     1: failure
 ******************************************************************************/
-LCMAPS_PLUGIN_MANAGER_C_STORAGE_CLASS int lcmaps_startPluginManager(void);
+int lcmaps_startPluginManager(void);
 
 /******************************************************************************
 Function:   lcmaps_stopPluginManager
@@ -100,7 +95,7 @@ Returns:
     0: termination succeeded
     1: termination failed
 ******************************************************************************/
-LCMAPS_PLUGIN_MANAGER_C_STORAGE_CLASS int lcmaps_stopPluginManager(void);
+int lcmaps_stopPluginManager(void);
 
 /******************************************************************************
 Function:   lcmaps_runPluginManager
@@ -122,7 +117,7 @@ Returns:
     0: user mapping or mapping verification succeeded
     1: user mapping or mapping verification failed
 ******************************************************************************/
-LCMAPS_PLUGIN_MANAGER_C_STORAGE_CLASS int lcmaps_runPluginManager(
+int lcmaps_runPluginManager(
         lcmaps_request_t request,
         lcmaps_cred_id_t lcmaps_cred,
 	char *requested_username,
@@ -143,7 +138,7 @@ Returns:
     0: plugin run succeeded
     1: plugin run failed
 ******************************************************************************/
-LCMAPS_PLUGIN_MANAGER_C_STORAGE_CLASS int lcmaps_runPlugin(
+int lcmaps_runPlugin(
         const char * pluginname
 );
 
@@ -160,14 +155,14 @@ Parameters:
 Returns:
     the return value of lcmaps_cleanCredentialData()
 ******************************************************************************/
-LCMAPS_PLUGIN_MANAGER_C_STORAGE_CLASS int lcmaps_resetCredentialData(void);
+int lcmaps_resetCredentialData(void);
 
 #endif /* _LCMAPS_PLUGINMANAGER_H */
 
 /******************************************************************************
 CVS Information:
     $Source: /srv/home/dennisvd/svn/mw-security/lcmaps/src/pluginmanager/_lcmaps_pluginmanager.h,v $
-    $Date: 2013-02-21 20:24:33 +0100 (Thu, 21 Feb 2013) $
-    $Revision: 16855 $
+    $Date: 2014-03-13 11:56:06 +0100 (Thu, 13 Mar 2014) $
+    $Revision: 17647 $
     $Author: msalle $
 ******************************************************************************/
