@@ -45,23 +45,21 @@
     \author Oscar Koeroo and Martijn Steenbakkers for the EU DataGrid.
 */
 
+#define _XOPEN_SOURCE	500
 
 /*****************************************************************************
                             Include header files
 ******************************************************************************/
 #include <stdio.h>
 #include <stdlib.h>
-/* #include <malloc.h> */
 #include <string.h>
+#include <errno.h>
+#include <sys/types.h>
 
 #include "_lcmaps_cred_data.h"
 #include "lcmaps_log.h"
 #include "lcmaps_vo_data.h"
 
-
-#include "_lcmaps_credential.h"
-#include "_lcmaps_runvars.h"
-
 /******************************************************************************
                           Module specific prototypes
 ******************************************************************************/
@@ -133,7 +131,9 @@ int addCredentialData(
         case UID                   :
                           if (data)
                           {
-			      if ( (credData.uid = (uid_t *) realloc(credData.uid, ((credData.cntUid+1) * sizeof(uid_t)))) == NULL)
+			      if ( (credData.cntUid+1) <= 0)
+				  return -1;
+			      if ( (credData.uid = (uid_t *) realloc(credData.uid, ((size_t)(credData.cntUid+1) * sizeof(uid_t)))) == NULL)
 				  return -1;
                               credData.uid[credData.cntUid] = *((uid_t *)data);
                               credData.cntUid++;
@@ -144,7 +144,9 @@ int addCredentialData(
                           if (data)
                           {
                               credData.cntPriGid++;
-                              if ( (credData.priGid = (gid_t *) realloc(credData.priGid, (credData.cntPriGid * sizeof(gid_t)))) == NULL)
+			      if ( credData.cntPriGid <= 0 )
+				  return -1;
+                              if ( (credData.priGid = (gid_t *) realloc(credData.priGid, ((size_t)(credData.cntPriGid) * sizeof(gid_t)))) == NULL)
 				  return -1;
                               credData.priGid[credData.cntPriGid - 1] = *((gid_t *)data);
                           }
@@ -173,7 +175,9 @@ int addCredentialData(
 
                               /* New gid so increase cntSecGid by 1, reallocate the array and sort it */
                               credData.cntSecGid++;
-			      if ( (credData.secGid = (gid_t *) realloc(credData.secGid, (credData.cntSecGid * sizeof(gid_t)))) == NULL)
+			      if ( credData.cntSecGid <=0 )
+				  return -1;
+			      if ( (credData.secGid = (gid_t *) realloc(credData.secGid, ((size_t)(credData.cntSecGid) * sizeof(gid_t)))) == NULL)
 				  return -1;
                               credData.secGid[credData.cntSecGid - 1] = newgid;
 
@@ -182,7 +186,7 @@ int addCredentialData(
                               {
                                   qsort(
                                       credData.secGid,
-                                      credData.cntSecGid,
+                                      (size_t)credData.cntSecGid,
                                       sizeof(gid_t),
                                       (int (*)(const void *,const void *))lcmaps_compare_gids
                                   );
@@ -193,7 +197,9 @@ int addCredentialData(
         case LCMAPS_VO_CRED        :
                           if (data)
                           {
-			      if ( (credData.VoCred = (lcmaps_vo_data_t *) realloc(credData.VoCred, ((credData.cntVoCred+1) * sizeof(lcmaps_vo_data_t)))) == NULL)
+			      if ( (credData.cntVoCred+1) <= 0 )
+				  return -1;
+			      if ( (credData.VoCred = (lcmaps_vo_data_t *) realloc(credData.VoCred, ((size_t)(credData.cntVoCred+1) * sizeof(lcmaps_vo_data_t)))) == NULL)
 				  return -1;
                               lcmaps_copyVoData(&(credData.VoCred[credData.cntVoCred]), (lcmaps_vo_data_t *) data);
                               credData.cntVoCred++;
@@ -203,7 +209,9 @@ int addCredentialData(
         case LCMAPS_VO_CRED_STRING :
                           if (data)
                           {
-			      if ( (credData.VoCredString = (char **) realloc(credData.VoCredString, ((credData.cntVoCredString+1) * sizeof(char *)))) == NULL)
+			      if ( (credData.cntVoCredString+1) <= 0 )
+				  return -1;
+			      if ( (credData.VoCredString = (char **) realloc(credData.VoCredString, ((size_t)(credData.cntVoCredString+1) * sizeof(char *)))) == NULL)
 				  return -1;
 			      if ( (credData.VoCredString[credData.cntVoCredString] = strdup(*((char **)data))) == NULL)
 				  return -1;
@@ -214,7 +222,9 @@ int addCredentialData(
         case LCMAPS_VO_CRED_MAPPING:
                           if (data)
                           {
-			      if ( (credData.VoCredMapping = (lcmaps_vo_mapping_t *) realloc(credData.VoCredMapping, ((credData.cntVoCredMapping+1) * sizeof(lcmaps_vo_mapping_t)))) == NULL )
+			      if ( (credData.cntVoCredMapping+1) <= 0)
+				  return -1;
+			      if ( (credData.VoCredMapping = (lcmaps_vo_mapping_t *) realloc(credData.VoCredMapping, ((size_t)(credData.cntVoCredMapping+1) * sizeof(lcmaps_vo_mapping_t)))) == NULL )
 				  return -1;
                               lcmaps_copyVoMapping(&(credData.VoCredMapping[credData.cntVoCredMapping]), (lcmaps_vo_mapping_t *) data);
                               credData.cntVoCredMapping++;
@@ -376,14 +386,13 @@ Returns:
     \brief Get pointer to a list of credential data of a certain type
 
     \param debug_level  the debug level
-
-    \return nothing
 */
 void lcmaps_printCredData(int debug_level)
 {
     int i = 0;
-    int bufsize = 1500;
+    size_t buflen, bufsize = 1500;
     char * buffer = NULL;
+    int rc;
 
     /* 2048 is the max log line for sys log, 1500 is very safe */
     if ( (buffer = calloc (sizeof(char), (bufsize + 1))) == NULL)   {
@@ -395,30 +404,43 @@ void lcmaps_printCredData(int debug_level)
 
     if (credData.dn != NULL) 
     {
-        if ((bufsize - strlen(buffer)) <= snprintf (&buffer[strlen(buffer)], bufsize - strlen(buffer), "DN:\"%s\"%s", 
-                                                                                                       credData.dn,
-                                                                                                       (credData.cntUid > 0) || (credData.cntPriGid > 0) || (credData.cntSecGid > 0) ? "->" : "")) 
+	buflen=strlen(buffer);
+	rc=snprintf (&buffer[buflen], bufsize - buflen, "DN:\"%s\"%s",
+		credData.dn,
+		(credData.cntUid > 0) || (credData.cntPriGid > 0) || (credData.cntSecGid > 0) ? "->" : "");
+	if (rc<0)
+	    lcmaps_log(LOG_INFO, "LCMAPS: Warning: error printing DN: %s\n",strerror(errno));
+	else if ((size_t)rc>=bufsize-buflen)
             lcmaps_log(LOG_INFO, "LCMAPS: Warning: output truncated for DN.\n");
         /* lcmaps_log(debug_level, "dn: %s\n", credData.dn); */
     }
     for (i = 0; i < credData.cntUid; i++)
     {
-        if ((bufsize - strlen(buffer)) <= snprintf (&buffer[strlen(buffer)], bufsize - strlen(buffer), "mapped uid:\'%d\'", 
-                                                                                                       credData.uid[i]))
-            lcmaps_log(LOG_INFO, "LCMAPS: Warning: output truncated for uid. %d\n", bufsize);
+	buflen=strlen(buffer);
+	rc=snprintf (&buffer[buflen], bufsize - buflen, "mapped uid:\'%d\'", credData.uid[i]);
+	if (rc<0)
+	    lcmaps_log(LOG_INFO, "LCMAPS: Warning: error printing uid: %s\n",strerror(errno));
+	else if ((size_t)rc>=bufsize-buflen)
+            lcmaps_log(LOG_INFO, "LCMAPS: Warning: output truncated for uid.\n");
         /* lcmaps_log(debug_level, "uid                   : %d  [%d/%d]\n", credData.uid[i], i+1, credData.cntUid); */
     }
     for (i = 0; i < credData.cntPriGid; i++)
     {
-        if ((bufsize - strlen(buffer)) <= snprintf (&buffer[strlen(buffer)], bufsize - strlen(buffer), ",pgid:\'%d\'", 
-                                                                                                       credData.priGid[i]))
+	buflen=strlen(buffer);
+	rc=snprintf (&buffer[buflen], bufsize - buflen, ",pgid:\'%d\'", credData.priGid[i]);
+	if (rc<0)
+	    lcmaps_log(LOG_INFO, "LCMAPS: Warning: error printing pgid: %s\n",strerror(errno));
+	else if ((size_t)rc>=bufsize-buflen)
             lcmaps_log(LOG_INFO, "LCMAPS: Warning: output truncated for pgid.\n");
         /* lcmaps_log(debug_level, "pgid                  : %d  [%d/%d]\n", credData.priGid[i], i+1, credData.cntPriGid); */
     }
     for (i = 0; i < credData.cntSecGid; i++)
     {
-        if ((bufsize - strlen(buffer)) <= snprintf (&buffer[strlen(buffer)], bufsize - strlen(buffer), ",sgid:\'%d\'", 
-                                                                                                       credData.secGid[i]))
+	buflen=strlen(buffer);
+	rc=snprintf (&buffer[buflen], bufsize - buflen, ",sgid:\'%d\'", credData.secGid[i]);
+	if (rc<0)
+	    lcmaps_log(LOG_INFO, "LCMAPS: Warning: error printing sgid: %s\n",strerror(errno));
+	else if ((size_t)rc>=bufsize-buflen)
             lcmaps_log(LOG_INFO, "LCMAPS: Warning: output truncated for sgid.\n");
         /* lcmaps_log(debug_level, "sgid                  : %d  [%d/%d]\n", credData.secGid[i], i+1, credData.cntSecGid); */
     }
@@ -481,7 +503,7 @@ static int lcmaps_compare_gids(const void * pgid1, const void * pgid2)
 /******************************************************************************
 CVS Information:
     $Source: /srv/home/dennisvd/svn/mw-security/lcmaps/src/pluginmanager/lcmaps_cred_data.c,v $
-    $Date: 2012-03-16 16:16:23 +0100 (Fri, 16 Mar 2012) $
-    $Revision: 16177 $
+    $Date: 2015-02-02 11:46:42 +0100 (Mon, 02 Feb 2015) $
+    $Revision: 18216 $
     $Author: msalle $
 ******************************************************************************/
