@@ -49,9 +49,9 @@
                             Include header files
 ******************************************************************************/
 #include <stdlib.h>
-/* #include <malloc.h> */
 #include <stdio.h>
 #include <string.h>
+
 #include "lcmaps_log.h"
 #include "_lcmaps_db_read.h"
 
@@ -80,12 +80,16 @@
     Characters that terminate pairs in the lcmaps database file. This
     is a combination of whitespace and separators. \internal
  */
+#if 0
 #define PAIR_TERMINATOR_CHARS PAIR_SEP_CHARS WHITESPACE_CHARS
+#endif
 /*!
     Characters that terminate variables and values in the lcmaps database file. This
     is a combination of whitespace and separators. \internal
  */
+#if 0
 #define VARVAL_TERMINATOR_CHARS VARVAL_SEP_CHARS WHITESPACE_CHARS
+#endif
 
 #ifndef NUL
 #define NUL '\0' /*!< \internal */
@@ -172,7 +176,7 @@ static int lcmaps_db_read_entries(
     nlines=0;
     no_entries=0;
 /*    lcmaps_db_fill_entry(no_entries,NULL); */
-    while (fgets(line, sizeof(line), dbstream))
+    while (fgets(line, (int)sizeof(line), dbstream))
     {
         ++nlines;
 
@@ -431,7 +435,7 @@ static int lcmaps_db_parse_pair(
         char ** pval
 )
 {
-    int    len;
+    size_t len;
     char * var;
     char * val;
 
@@ -626,7 +630,7 @@ int lcmaps_db_clean(void)
 /******************************************************************************
 CVS Information:
     $Source: /srv/home/dennisvd/svn/mw-security/lcmaps/src/pluginmanager/lcmaps_db_read.c,v $
-    $Date: 2011-11-06 22:34:18 +0100 (Sun, 06 Nov 2011) $
-    $Revision: 15527 $
-    $Author: okoeroo $
+    $Date: 2014-02-26 16:01:45 +0100 (Wed, 26 Feb 2014) $
+    $Revision: 17529 $
+    $Author: msalle $
 ******************************************************************************/
