@@ -45,6 +45,7 @@
     \author Martijn Steenbakkers for the EU DataGrid.
 */
 
+#define _XOPEN_SOURCE	500
 
 /*****************************************************************************
                             Include header files
@@ -65,12 +66,12 @@
                           Module specific prototypes
 ******************************************************************************/
 #ifndef CONF_LCMAPS_DEBUG_LEVEL
-    #define CONF_LCMAPS_DEBUG_LEVEL 4 /*!< default debugging level */
+    #define CONF_LCMAPS_DEBUG_LEVEL 4 /*!< default log level */
 #endif /* CONF_LCMAPS_DEBUG_LEVEL */
 
 static int lcmaps_debuglevel_to_sysloglevel(int);
-static char * lcmaps_syslog_level_name_to_string(int);
-char * lcmaps_get_time_string(void);
+static const char * lcmaps_syslog_level_name_to_string(int);
+static char * lcmaps_get_time_string(void);
 
 
 /******************************************************************************
@@ -79,7 +80,7 @@ char * lcmaps_get_time_string(void);
 static FILE *  lcmaps_logfp=NULL; /*!< stream associated with logfile \internal */
 static int     logging_usrlog=0; /*!< flag to do user logging \internal */
 static int     logging_syslog=0; /*!< flag to use syslog \internal */
-static int     lcmaps_debug_level=LOG_INFO; /*!< debugging level \internal */
+static int     lcmaps_debug_level=LOG_INFO; /*!< log level \internal */
 static char *  extra_logstr = NULL; /*!< string to be included in every log statement \internal */ 
 static int     should_close_lcmaps_logfp = 0; /*!< Flag to check if the log stream should be closed \internal */ 
 static int     detected_old_plugin = 0; /*!< Flag that indicates that an old plugin has been detected */
@@ -116,7 +117,7 @@ Parameters:
                 syslog level
 Returns:        char * name or UNKNOWN
 ******************************************************************************/
-static char * lcmaps_syslog_level_name_to_string(int syslog_level)
+static const char * lcmaps_syslog_level_name_to_string(int syslog_level)
 {
     switch (syslog_level) {
         case LOG_EMERG   : return "LOG_EMERG";
@@ -151,7 +152,7 @@ Returns:        0 succes
     \brief Start logging
 
     This function should only be used by the LCMAPS itself.
-    It opens the logfile and tries to set the debugging level in the following order:
+    It opens the logfile and tries to set the log level in the following order:
     -# Try if CONF_LCMAPS_DEBUG_LEVEL > 0
     -# Try if environment variable LCMAPS_DEBUG_LEVEL is set and if it is an integer > 0
     -# Otherwise set lcmaps_debug_level = LOG_INFO;
@@ -167,45 +168,55 @@ Returns:        0 succes
 int
 lcmaps_log_open(char * path, FILE * fp, unsigned short logtype )
 {
-    int    j = 0;
+    size_t j = 0;
+    long   longval;
+    char * logfile;
     int    conf_lcmaps_debug_level = 4; /* The LCMAPS debug messages setting in a range of 0-5, needs to be mapped onto Syslog levels. */
     char * debug_env = NULL;
     char * logstr_env = NULL;
 
     detected_old_plugin = 0; /* Initializer */
 
-    if ((logtype & DO_SYSLOG) == DO_SYSLOG)
-    {
+    /* syslog part */
+    if ((logtype & DO_SYSLOG) == DO_SYSLOG) {
         logging_syslog=1;
     }
-    if ((logtype & DO_USRLOG) == DO_USRLOG)
-    {
-        logging_usrlog=1;
-        if (fp != NULL)
-        {
-            /* File already opened, should not be closed at the end */
-            lcmaps_logfp=fp;
-            should_close_lcmaps_logfp = 0;
-        }
-        else if (path != NULL)
-        {
-            /* Try to append to file */
-            if ((lcmaps_logfp = fopen(path, "a")) == NULL)
-            {
-                syslog(LOG_CRIT, "lcmaps_log_open(): Cannot open logfile %s: %s\n", path, strerror(errno));
-                if (logging_syslog)
-                {
-                    syslog(LOG_ERR, "lcmaps_log_open(): Cannot open logfile %s\n", path);
-                }
-                return 1;
-            }
-            should_close_lcmaps_logfp = 1;
-        }
-        else
-        {
-            syslog(LOG_ERR, "lcmaps_log_open(): Specified to 'log to file', but no open file pointer or path to logfile was provided.\n");
-            return 1;
-        }
+
+    /* logfile part: either fp is given or need filename */
+    if ((logtype & DO_USRLOG) == DO_USRLOG) {
+	if (lcmaps_logfp!=NULL)	{
+	    lcmaps_log(LOG_DEBUG,
+		    "%s() has already been called before.\n", __func__);
+	    return 0;
+	}
+   
+	/* filepointer or filename ? */
+	if (fp != NULL) {
+	    /* File already opened, should not be closed at the end */
+	    lcmaps_logfp=fp;
+	    should_close_lcmaps_logfp = 0;
+	    logging_usrlog=1;
+	} else { /* Need a filename: check argument and env variable */
+	    logfile=(path ? path : getenv("LCMAPS_LOG_FILE"));
+	    if (logfile != NULL) {
+		/* Try to append to file */
+		if ((lcmaps_logfp = fopen(logfile, "a")) != NULL) {
+		    should_close_lcmaps_logfp = 1;
+		    logging_usrlog=1;
+		} else {
+		    logging_usrlog=0;
+		    logging_syslog=1;
+		    syslog(LOG_ERR,
+			"%s(): Cannot open logfile %s, will use syslog: %s\n",
+			__func__, logfile, strerror(errno));
+		}
+	    } else {
+		logging_usrlog=0;
+		logging_syslog=1;
+		/* This is not an error: better not log at all as we haven't yet
+		 * set loglevel */
+	    }
+	}
     }
 
     /* Get the configuration for LCMAPS log leveling
@@ -217,22 +228,26 @@ lcmaps_log_open(char * path, FILE * fp, unsigned short logtype )
         /* convert into integer */
         for (j = 0; j < strlen(debug_env); j++) {
             if (!isdigit((debug_env)[j])) {
-                syslog(LOG_ERR, "lcmaps_log_open(): found non-digit in environment variable in \"LCMAPS_DEBUG_LEVEL\" = %s\n", debug_env);
+                syslog(LOG_ERR, "%s(): found non-digit in environment variable in \"LCMAPS_DEBUG_LEVEL\" = %s\n", __func__, debug_env);
                 return 1;
             }
         }
-        conf_lcmaps_debug_level = strtol(debug_env, NULL, 10);
-        if (conf_lcmaps_debug_level < 0) {
-            syslog(LOG_ERR, "lcmaps_log_open(): environment variable value in \"LCMAPS_DEBUG_LEVEL\" should be 0 <= x <= 5.\n");
+	errno=0;
+	longval=strtol(debug_env, NULL, 10);
+	if (errno!=0 || longval<0 || longval>5)	{
+	    syslog(LOG_ERR, "%s(): environment variable value in \"LCMAPS_DEBUG_LEVEL\" should be 0 <= x <= 5.\n", __func__);
             return 1;
         }
+	conf_lcmaps_debug_level=(int)longval;
     } else {
         /* Default debug_level, even when nothing else is provided */
         conf_lcmaps_debug_level = (int) CONF_LCMAPS_DEBUG_LEVEL;
     }
 
     lcmaps_debug_level = lcmaps_debuglevel_to_sysloglevel(conf_lcmaps_debug_level);
-    lcmaps_log(LOG_DEBUG,"lcmaps_log_open(): setting debugging level to %d (LCMAPS_DEBUG_LEVEL), which translates to Syslog level \"%s\".\n", conf_lcmaps_debug_level, lcmaps_syslog_level_name_to_string(lcmaps_debug_level));
+    lcmaps_log(LOG_DEBUG,"%s(): setting log level to %d (LCMAPS_DEBUG_LEVEL), which translates to Syslog level \"%s\".\n", 
+	    __func__, conf_lcmaps_debug_level,
+	    lcmaps_syslog_level_name_to_string(lcmaps_debug_level));
 
     /*
      * Check if there is an extra log string
@@ -270,14 +285,13 @@ Returns:        0 succes
     priority is 0.
 
     \param prty syslog priority (if 0 don't syslog).
-    \param fmt  string format
-    \param ...  variable argument list
+    \param fmt  string format followed by variable argument list
 
     \retval 0 succes.
     \retval 1 failure.
 */
 int
-lcmaps_log(int prty, char * fmt, ...)
+lcmaps_log(int prty, const char * fmt, ...)
 {
     va_list pvar;
     char    buf[MAX_LOG_BUFFER_SIZE];
@@ -300,18 +314,25 @@ lcmaps_log(int prty, char * fmt, ...)
 	    buf[i]='?';
     }
 
-    if ( (res >= MAX_LOG_BUFFER_SIZE) || (res < 0) )
-    {
-        syslog(LOG_ERR, "lcmaps_log(): log string too long (> %d)\n", MAX_LOG_BUFFER_SIZE);
+    /* Check return value */
+    if ( res < 0 )  {
+	lcmaps_log(LOG_ERR, "lcmaps_log() error: %s\n",strerror(errno));
+	return 1;
     }
+	
+    /* truncate too long entries */
+    if ((size_t)res >= MAX_LOG_BUFFER_SIZE)
+	snprintf(buf+MAX_LOG_BUFFER_SIZE-5,(size_t)5,"...\n");
+
     /* Log to an opened file pointer */
     if (logging_usrlog)
     {
-        if (lcmaps_logfp == NULL)
-        {
-            /* fprintf(stderr,"lcmaps_log() error: cannot open file descriptor\n"); */
-            syslog(LOG_CRIT, "lcmaps_log() error: cannot open file descriptor");
-        }
+        if (lcmaps_logfp == NULL)   {
+            syslog(LOG_ERR, "lcmaps_log() error: cannot open file descriptor");
+	    /* switch over to syslog */
+	    logging_usrlog=0;
+	    logging_syslog=1;
+	}
         else
         {
             /* Get the (mandatory) prefixed datetime stamps */
@@ -324,15 +345,15 @@ lcmaps_log(int prty, char * fmt, ...)
 
             if (extra_logstr == NULL) {
                 if (lcmaps_log_ident) {
-                    fprintf(lcmaps_logfp,"%s:lcmaps[%d] %11s: %s: %s", lcmaps_log_ident, (int)getpid(), lcmaps_syslog_level_name_to_string(prty), datetime, buf);
+                    fprintf(lcmaps_logfp,"%s:lcmaps[%ld] %11s: %s: %s", lcmaps_log_ident, (long)getpid(), lcmaps_syslog_level_name_to_string(prty), datetime, buf);
                 } else {
-                    fprintf(lcmaps_logfp,"lcmaps[%d] %11s: %s: %s", (int)getpid(), lcmaps_syslog_level_name_to_string(prty), datetime, buf);
+                    fprintf(lcmaps_logfp,"lcmaps[%ld] %11s: %s: %s", (long)getpid(), lcmaps_syslog_level_name_to_string(prty), datetime, buf);
                 }
             } else {
                 if (lcmaps_log_ident) {
-                    fprintf(lcmaps_logfp,"%s:lcmaps[%d] %11s: %s: %s: %s", lcmaps_log_ident, (int)getpid(), lcmaps_syslog_level_name_to_string(prty), datetime, extra_logstr, buf);
+                    fprintf(lcmaps_logfp,"%s:lcmaps[%ld] %11s: %s: %s: %s", lcmaps_log_ident, (long)getpid(), lcmaps_syslog_level_name_to_string(prty), datetime, extra_logstr, buf);
                 } else {
-                    fprintf(lcmaps_logfp,"lcmaps[%d] %11s: %s: %s: %s", (int)getpid(), lcmaps_syslog_level_name_to_string(prty), datetime, extra_logstr, buf);
+                    fprintf(lcmaps_logfp,"lcmaps[%ld] %11s: %s: %s: %s", (long)getpid(), lcmaps_syslog_level_name_to_string(prty), datetime, extra_logstr, buf);
                 }
             }
             fflush(lcmaps_logfp);
@@ -354,9 +375,9 @@ lcmaps_log(int prty, char * fmt, ...)
 #if 0
         /* Mention the log priority explicitly, for debugging purposes */
         if (extra_logstr == NULL) {
-            syslog(prty, "lcmaps[%d] %11s: %s",      (int)getpid(), lcmaps_syslog_level_name_to_string(prty), buf);
+            syslog(prty, "lcmaps[%ld] %11s: %s",      (long)getpid(), lcmaps_syslog_level_name_to_string(prty), buf);
         } else {
-            syslog(prty, "lcmaps[%d] %11s: %s: %s",  (int)getpid(), lcmaps_syslog_level_name_to_string(prty), extra_logstr, buf);
+            syslog(prty, "lcmaps[%ld] %11s: %s: %s",  (long)getpid(), lcmaps_syslog_level_name_to_string(prty), extra_logstr, buf);
         }
 #else
         if (extra_logstr == NULL) {
@@ -391,14 +412,13 @@ Returns:        0 succes
     provided debug_lvl <= DEBUG_LEVEL (default is 0).
 
     \param debug_lvl debugging level
-    \param fmt       string format
-    \param ...       variable argument list
+    \param fmt       string format followed by variable argument list
 
     \retval 0 succes.
     \retval 1 failure.
 */
 int
-lcmaps_log_debug(int debug_lvl, char * fmt, ...)
+lcmaps_log_debug(int debug_lvl, const char * fmt, ...)
 {
     va_list pvar;
     char    buf[MAX_LOG_BUFFER_SIZE];
@@ -423,11 +443,17 @@ lcmaps_log_debug(int debug_lvl, char * fmt, ...)
     va_start(pvar, fmt);
     res=vsnprintf(buf,MAX_LOG_BUFFER_SIZE,fmt,pvar);
     va_end(pvar);
-    if ( (res >= MAX_LOG_BUFFER_SIZE) || (res < 0) )
-    {
-        fprintf(stderr,"lcmaps_log(): log string too long (> %d)\n", MAX_LOG_BUFFER_SIZE);
+    
+    /* Check return value */
+    if ( res < 0 )  {
+	lcmaps_log(LOG_ERR, "lcmaps_log_debug() error: %s\n",strerror(errno));
+	return 1;
     }
 
+    /* truncate too long entries */
+    if ((size_t)res >= MAX_LOG_BUFFER_SIZE)
+	snprintf(buf+MAX_LOG_BUFFER_SIZE-5,(size_t)5,"...\n");
+
     /* LOG_DEBUG, no exceptions */
     /* NOTE: Make sure NOT to use buf directly, but do via a format string */
     return lcmaps_log(LOG_DEBUG,"%s",buf);
@@ -463,7 +489,7 @@ Returns:        0 succes
     \retval 1 failure.
 */
 int
-lcmaps_log_a_string(int prty, char * fmt, char * the_string)
+lcmaps_log_a_string(int prty, const char * fmt, const char * the_string)
 {
     return lcmaps_log(prty, fmt, the_string);
 }
@@ -494,7 +520,7 @@ Returns:        0 succes
     \retval 1 failure.
 */
 int
-lcmaps_log_a_string_debug(int debug_lvl, char * fmt, char * the_string)
+lcmaps_log_a_string_debug(int debug_lvl, const char * fmt, const char * the_string)
 {
     return lcmaps_log_debug(debug_lvl, fmt, the_string);
 }
@@ -559,14 +585,13 @@ Returns:        0 succes
     priority is 0.
  
     \param prty syslog priority (if 0 don't syslog).
-    \param fmt  string format
-    \param ...  variable argument list
+    \param fmt  string format followed by variable argument list
  
     \retval 0 succes.
     \retval 1 failure.
 */
 int
-lcmaps_log_time(int prty, char * fmt, ...)
+lcmaps_log_time(int prty, const char * fmt, ...)
 {
     va_list     pvar;
     char        buf[MAX_LOG_BUFFER_SIZE];
@@ -576,11 +601,16 @@ lcmaps_log_time(int prty, char * fmt, ...)
     va_start(pvar, fmt);
     res=vsnprintf(buf,MAX_LOG_BUFFER_SIZE,fmt,pvar);
     va_end(pvar);
-    if ( (res >= MAX_LOG_BUFFER_SIZE) || (res < 0) )
-    {
-        fprintf(stderr,"lcmaps_log_time(): log string too long (> %d)\n", MAX_LOG_BUFFER_SIZE);
+    
+    /* Check return value */
+    if ( res < 0 )  {
+	lcmaps_log(LOG_ERR, "lcmaps_log_time() error: %s\n",strerror(errno));
+	return 1;
     }
 
+    /* truncate too long entries */
+    if ((size_t)res >= MAX_LOG_BUFFER_SIZE)
+	snprintf(buf+MAX_LOG_BUFFER_SIZE-5,(size_t)5,"...\n");
 
     /* Do regular logging, time prefix is added by lcmaps_log() when logging to
      * file */
@@ -610,7 +640,7 @@ Returns:       !NULL succes
     \retval !NULL succes.
     \retval NULL failure.
 */
-char * lcmaps_get_time_string(void)
+static char * lcmaps_get_time_string(void)
 {
     char      * datetime;
     time_t      mclock;
@@ -626,7 +656,7 @@ char * lcmaps_get_time_string(void)
     datetime = malloc(sizeof(char) * 21);
 
     /* This string will always return 19 characters and \0 */
-    snprintf(datetime, 21, "%04d-%02d-%02d.%02d:%02d:%02dZ",
+    snprintf(datetime, (size_t)21, "%04d-%02d-%02d.%02d:%02d:%02dZ",
                        tmp->tm_year + 1900, tmp->tm_mon + 1, tmp->tm_mday,
                        tmp->tm_hour, tmp->tm_min, tmp->tm_sec);
 
@@ -636,7 +666,7 @@ char * lcmaps_get_time_string(void)
 /******************************************************************************
 CVS Information:
     $Source: /srv/home/dennisvd/svn/mw-security/lcmaps/src/pluginmanager/lcmaps_log.c,v $
-    $Date: 2012-03-22 10:33:06 +0100 (Thu, 22 Mar 2012) $
-    $Revision: 16234 $
+    $Date: 2015-02-02 11:46:42 +0100 (Mon, 02 Feb 2015) $
+    $Revision: 18216 $
     $Author: msalle $
 ******************************************************************************/
