@@ -53,13 +53,12 @@
     -# lcmaps_resetCredentialData(): reset the Credential Data. (used by Evaluation Manager)
 */
 
-#ifndef LCMAPS_PLUGINMANAGER_C
-#define LCMAPS_PLUGINMANAGER_C
-
 /*****************************************************************************
                             Include header files
 ******************************************************************************/
 #include "lcmaps_config.h"
+
+/* Needed for NULL */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -69,27 +68,25 @@
 #include <errno.h>
 #include <sys/stat.h>
 
-#if HAVE_MALLOC_H
-#    include <malloc.h>
-#endif
-
 /* Programming interface to dynamic linking loader */
-#if HAVE_DLFCN_H
+#ifdef HAVE_DLFCN_H
 #    include <dlfcn.h>
 #endif
 
 /* LCMAPS includes */
-#include "lcmaps_types.h"
-#include "lcmaps_log.h"
-#include "lcmaps_arguments.h"
-#include "lcmaps_defines.h"
-#include "_lcmaps_utils.h"
-#include "_lcmaps_db_read.h"
-#include "_lcmaps_runvars.h"
-#include "_lcmaps_cred_data.h"
-#include "_lcmaps_pluginmanager.h"
+ #include "lcmaps_types.h"
+ #include "lcmaps_log.h"
+ #include "lcmaps_arguments.h"
+ #include "lcmaps_defines.h"
+ #include "lcmaps_plugin_typedefs.h"
+ #include "_lcmaps_utils.h"
+/*#include "_lcmaps_db_read.h"*/
+ #include "grid_credential_handling/_lcmaps_runvars.h"
+ #include "_lcmaps_cred_data.h"
+ #include "_lcmaps_pluginmanager.h"
 /* interface to evaluationmanager */
-#include "../evaluationmanager/evaluationmanager.h"
+ #include "../evaluationmanager/evaluationmanager.h"
+ #include "../evaluationmanager/pdl.h"
 
 /******************************************************************************
                              Define constants
@@ -100,49 +97,26 @@
 #endif
 
 /* LCMAPS authorization module definitions */
-#define MAXPROCS    5 /*!< maximum number of interface symbols in plugin modules \internal */
 #define USE_EVALUATIONMANAGER 1
 
 
 /******************************************************************************
                                Type definitions
 ******************************************************************************/
-/*!
-    \enum lcmaps_proctype_e
-    \brief This enumeration type gives the different plugin symbol/function types.
-    \internal
-*/
-enum lcmaps_proctype_e
-{
-    INITPROC,   /*!< this value corresponds to the plugin initialization function */
-    RUNPROC,    /*!< this value corresponds to the plugin run function (get credentials) */
-    TERMPROC,   /*!< this value corresponds to the plugin termination function */
-    INTROPROC,  /*!< this value corresponds to the plugin introspect function */
-    VERIFYPROC, /*!< this value corresponds to the plugin verification function */
-    ENDOFPROCS  /*< this is the last enumeration value */
-};
-
-/*!
-    \typedef lcmaps_proc_t
-    \brief this type corresponds to the types of the plugin interface functions
-    \internal
-*/
-typedef int (*lcmaps_proc_t)();
-
 /*!
     \typedef lcmaps_plugindl_t
-    \brief the type definition of the lcmaps plugin module structure
-    \internal
-*/
-/*!
     \struct lcmaps_plugindl_s
-    \brief the lcmaps plugin module structure
+    \brief the type definition of the lcmaps plugin module structure
     \internal
 */
 typedef struct lcmaps_plugindl_s
 {
     void *                      handle;                            /*!< dlopen handle to plugin module */
-    lcmaps_proc_t               procs[MAXPROCS];                   /*!< list of handles to interface functions of plugin */
+    plugin_initialize_t		*plugin_initialize;
+    plugin_introspect_t		*plugin_introspect;
+    plugin_run_t		*plugin_run;
+    plugin_verify_t		*plugin_verify;
+    plugin_terminate_t		*plugin_terminate;
     char                        pluginshortname[LCMAPS_MAXPATHLEN+1];   /*!< name of plugin without path */
     char                        pluginabsname[LCMAPS_MAXPATHLEN+1];   /*!< absolute name of plugin */
     char                        pluginargs[LCMAPS_MAXARGSTRING+1]; /*!< argument string */
@@ -158,7 +132,7 @@ lcmaps_plugindl_t;
                           Module specific prototypes
 ******************************************************************************/
 static lcmaps_plugindl_t * PluginInit(lcmaps_db_entry_t *, lcmaps_plugindl_t **);
-static lcmaps_proc_t       get_procsymbol(void *, char *);
+static void *		   get_procsymbol(void *, const char *);
 #ifdef LCMAPS_DEBUG
 static int                 print_lcmaps_plugin(int, lcmaps_plugindl_t *);
 #endif
@@ -174,10 +148,10 @@ static char * lcmaps_findplugin(char *);
                        Define module specific variables
 ******************************************************************************/
 static unsigned int         lcmaps_mode                    = LCMAPS_NORMAL_MODE; /*!< \internal */
-static char *               lcmaps_db_file_default         = NULL; /*!< \internal */
-static char *               lcmaps_dir                     = NULL; /*!< \internal */
+static const char *         lcmaps_db_file_default         = NULL; /*!< \internal */
+static const char *         lcmaps_dir                     = NULL; /*!< \internal */
 static lcmaps_plugindl_t *  plugin_list                    = NULL; /*!< \internal */
-static void *               lcmaps_stored_sigpipe_handler  = NULL; /*!< \internal */
+static void                 (*lcmaps_stored_sigpipe_handler)(int)  = NULL; /*!< \internal */
 
 /******************************************************************************
 Function:       fexist()
@@ -472,10 +446,10 @@ int lcmaps_startPluginManager(
 #else
     lcmaps_db_clean();
 #endif
-    if (clean_plugin_list(&plugin_list)!=0)
-    {
-        lcmaps_log(LOG_ERR,"lcmaps.mod-lcmaps_startPluginManager() error: could not clean up plugin list\n");
-    }
+
+    /* Stop the evaluationmanager again, cleaning up e.g. lex/yacc memory */
+    lcmaps_stopPluginManager();
+
     if (lcmaps_db_file) free(lcmaps_db_file);
 
     /* It is necessary to call lcmaps_free_resources() here in case LCMAPS
@@ -530,10 +504,15 @@ static lcmaps_plugindl_t * PluginInit(
     char *                 pname=NULL;
     char *                 args=NULL;
     void *                 plugin_handle;
-    lcmaps_proc_t          plugin_procs[MAXPROCS];
     lcmaps_plugindl_t *    pplugin=NULL;
     int                    i;
     int                    retval;
+    plugin_initialize_t	   *plugin_initialize=NULL;
+    plugin_introspect_t	   *plugin_introspect=NULL;
+    plugin_run_t	   *plugin_run=NULL;
+    plugin_verify_t	   *plugin_verify=NULL;
+    plugin_terminate_t	   *plugin_terminate=NULL;
+
 
     name = db_handle->pluginname;
     args = db_handle->pluginargs;
@@ -555,75 +534,56 @@ static lcmaps_plugindl_t * PluginInit(
     }
 
     /* Check symbols */
-    for (i=0; i < MAXPROCS; ++i)
+    plugin_initialize=(plugin_initialize_t*)get_procsymbol(plugin_handle,"plugin_initialize");
+    if (plugin_initialize == NULL)
     {
-        switch (i)
-        {
-            case INITPROC:
-                plugin_procs[i]=get_procsymbol(plugin_handle,"plugin_initialize");
-                if (plugin_procs[i] == NULL)
-                {
-                    lcmaps_log(LOG_ERR,"lcmaps.mod-PluginInit(): plugin %s not compliant. Missing function \"plugin_initialize()\"\n", name);
-                    goto fail_PluginInit;
-                }
-                else
-                {
-                    lcmaps_log_debug(3,"lcmaps.mod-PluginInit(): found \"plugin_initialize()\"\n");
-                }
-                break;
-            case RUNPROC:
-                plugin_procs[i]=get_procsymbol(plugin_handle,"plugin_run");
-                if (plugin_procs[i] == NULL)
-                {
-                    lcmaps_log(LOG_ERR,"lcmaps.mod-PluginInit(): plugin %s not compliant. Missing function \"plugin_run()\"\n", name);
-                    goto fail_PluginInit;
-                }
-                else
-                {
-                    lcmaps_log_debug(3,"lcmaps.mod-PluginInit(): found \"plugin_run()\"\n");
-                }
-                break;
-            case TERMPROC:
-                plugin_procs[i]=get_procsymbol(plugin_handle,"plugin_terminate");
-                if (plugin_procs[i] == NULL)
-                {
-                    lcmaps_log(LOG_ERR,"lcmaps.mod-PluginInit(): plugin %s not compliant. Missing function \"plugin_terminate()\"\n", name);
-                    goto fail_PluginInit;
-                }
-                else
-                {
-                    lcmaps_log_debug(3,"lcmaps.mod-PluginInit(): found \"plugin_terminate()\"\n");
-                }
-                break;
-            case INTROPROC:
-                plugin_procs[i]=get_procsymbol(plugin_handle,"plugin_introspect");
-                if (plugin_procs[i] == NULL)
-                {
-                    lcmaps_log(LOG_ERR,"lcmaps.mod-PluginInit(): plugin %s not compliant. Missing function \"plugin_introspect()\"\n", name);
-                    goto fail_PluginInit;
-                }
-                else
-                {
-                    lcmaps_log_debug(3,"lcmaps.mod-PluginInit(): found \"plugin_introspect()\"\n");
-                }
-                break;
-            case VERIFYPROC:
-                plugin_procs[i]=get_procsymbol(plugin_handle,"plugin_verify");
-                if (plugin_procs[i] == NULL)
-                {
-                    lcmaps_log(LOG_INFO,"lcmaps.mod-PluginInit(): no \"plugin_verify()\" method found in %s: old style plugin\n", name);
-                }
-                else
-                {
-                    lcmaps_log_debug(3,"lcmaps.mod-PluginInit(): found \"plugin_verify()\"\n");
-                }
-                break;
-            default:
-                /* do nothing */
-                plugin_procs[i]=NULL;
-                break;
-        }
+	lcmaps_log(LOG_ERR,"lcmaps.mod-PluginInit(): plugin %s not compliant. Missing function \"plugin_initialize()\"\n", name);
+	goto fail_PluginInit;
     }
+    else
+    {
+	lcmaps_log_debug(3,"lcmaps.mod-PluginInit(): found \"plugin_initialize()\"\n");
+    }
+    plugin_run=(plugin_run_t*)get_procsymbol(plugin_handle,"plugin_run");
+    if (plugin_run == NULL)
+    {
+	lcmaps_log(LOG_ERR,"lcmaps.mod-PluginInit(): plugin %s not compliant. Missing function \"plugin_run()\"\n", name);
+	goto fail_PluginInit;
+    }
+    else
+    {
+	lcmaps_log_debug(3,"lcmaps.mod-PluginInit(): found \"plugin_run()\"\n");
+    }
+    plugin_terminate=(plugin_terminate_t*)get_procsymbol(plugin_handle,"plugin_terminate");
+    if (plugin_terminate == NULL)
+    {
+	lcmaps_log(LOG_ERR,"lcmaps.mod-PluginInit(): plugin %s not compliant. Missing function \"plugin_terminate()\"\n", name);
+	goto fail_PluginInit;
+    }
+    else
+    {
+	lcmaps_log_debug(3,"lcmaps.mod-PluginInit(): found \"plugin_terminate()\"\n");
+    }
+    plugin_introspect=(plugin_introspect_t*)get_procsymbol(plugin_handle,"plugin_introspect");
+    if (plugin_introspect == NULL)
+    {
+	lcmaps_log(LOG_ERR,"lcmaps.mod-PluginInit(): plugin %s not compliant. Missing function \"plugin_introspect()\"\n", name);
+	goto fail_PluginInit;
+    }
+    else
+    {
+	lcmaps_log_debug(3,"lcmaps.mod-PluginInit(): found \"plugin_introspect()\"\n");
+    }
+    plugin_verify=(plugin_verify_t*)get_procsymbol(plugin_handle,"plugin_verify");
+    if (plugin_verify == NULL)
+    {
+	lcmaps_log(LOG_INFO,"lcmaps.mod-PluginInit(): no \"plugin_verify()\" method found in %s: old style plugin\n", name);
+    }
+    else
+    {
+	lcmaps_log_debug(3,"lcmaps.mod-PluginInit(): found \"plugin_verify()\"\n");
+    }
+
     /* Make a new entry at end of plugin list */
     if (!*list)
     {
@@ -647,8 +607,11 @@ static lcmaps_plugindl_t * PluginInit(
     pplugin->handle=plugin_handle;
     pplugin->run_argc=0;
     pplugin->run_argv=NULL;
-    for (i=0; i < MAXPROCS; ++i)
-        pplugin->procs[i]=plugin_procs[i];
+    pplugin->plugin_initialize	= plugin_initialize;
+    pplugin->plugin_introspect	= plugin_introspect;
+    pplugin->plugin_run		= plugin_run;
+    pplugin->plugin_verify	= plugin_verify;
+    pplugin->plugin_terminate	= plugin_terminate;
 
     if (pname != NULL)
     {
@@ -684,7 +647,7 @@ static lcmaps_plugindl_t * PluginInit(
 
 
     /* Run plugin_initialize() */
-    retval=(*(pplugin->procs[INITPROC]))(pplugin->init_argc,pplugin->init_argv);
+    retval=(*(pplugin->plugin_initialize))(pplugin->init_argc,pplugin->init_argv);
     if (retval != LCMAPS_MOD_SUCCESS )
     {
         lcmaps_log_debug(1,"lcmaps.mod-PluginInit(): \"plugin_initialize()\" failed\n");
@@ -696,7 +659,7 @@ static lcmaps_plugindl_t * PluginInit(
     }
 
     /* Run plugin_introspect() */
-    retval=(*(pplugin->procs[INTROPROC]))(&(pplugin->run_argc),&(pplugin->run_argv));
+    retval=(*(pplugin->plugin_introspect))(&(pplugin->run_argc),&(pplugin->run_argv));
     if (retval != LCMAPS_MOD_SUCCESS )
     {
         lcmaps_log(LOG_ERR,"lcmaps.mod-PluginInit(): \"plugin_introspect()\" failed\n");
@@ -756,10 +719,14 @@ static int parse_args_plugin(
         int * xargc
 )
 {
-    int len=0;
+    size_t len=0;
     int i;
     int rc;
 
+    /* Protect against empty name */
+    if (name==NULL)
+	return 1;
+
     /* set modulename */
     len=strlen(name);
     if ( (len > 0) && (len < LCMAPS_MAXPATHLEN) )
@@ -809,14 +776,15 @@ Returns:
     \return handle to procedure symbol or NUll
     \internal
 */
-static lcmaps_proc_t get_procsymbol(
+static void *get_procsymbol(
         void * handle,
-        char * symname
+        const char * symname
 )
 {
-    lcmaps_proc_t symhandle;
+    void *symhandle=NULL;
     char *error;
 
+    dlerror();
     symhandle=dlsym(handle,symname);
     if ((error = dlerror()) != NULL)
     {
@@ -857,7 +825,7 @@ static int print_lcmaps_plugin(
 )
 {
     char ** pstring = NULL;
-    char *  argType = NULL;
+    const char *  argType = NULL;
     int  *  pint    = NULL;
     int     i       = 0;
     void * value    = NULL;
@@ -867,11 +835,11 @@ static int print_lcmaps_plugin(
     lcmaps_log_debug(debug_lvl,"plugin address                      : 0x%x\n",plugin);
     lcmaps_log_debug(debug_lvl,"plugin size                         : %d\n",sizeof(lcmaps_plugindl_t));
     lcmaps_log_debug(debug_lvl,"plugin handle                       : 0x%x\n",plugin->handle);
-    lcmaps_log_debug(debug_lvl,"plugin_initialize()                 : 0x%x\n",plugin->procs[INITPROC]);
-    lcmaps_log_debug(debug_lvl,"plugin_run()                        : 0x%x\n",plugin->procs[RUNPROC]);
-    lcmaps_log_debug(debug_lvl,"plugin_verify()                     : 0x%x\n",plugin->procs[VERIFYPROC]);
-    lcmaps_log_debug(debug_lvl,"plugin_terminate()                  : 0x%x\n",plugin->procs[TERMPROC]);
-    lcmaps_log_debug(debug_lvl,"plugin_introspect()                 : 0x%x\n",plugin->procs[INTROPROC]);
+    lcmaps_log_debug(debug_lvl,"plugin_initialize()                 : 0x%x\n",plugin->plugin_initialize);
+    lcmaps_log_debug(debug_lvl,"plugin_run()                        : 0x%x\n",plugin->plugin_run);
+    lcmaps_log_debug(debug_lvl,"plugin_verify()                     : 0x%x\n",plugin->plugin_verify);
+    lcmaps_log_debug(debug_lvl,"plugin_terminate()                  : 0x%x\n",plugin->plugin_terminate);
+    lcmaps_log_debug(debug_lvl,"plugin_introspect()                 : 0x%x\n",plugin->plugin_introspect);
     lcmaps_log_debug(debug_lvl,"plugin init_argc                    : %d\n",plugin->init_argc);
     for (i=0; i < plugin->init_argc; i++)
         lcmaps_log_debug(debug_lvl,"plugin init_argv[%2d]                : %s\n",i,(plugin->init_argv)[i]);
@@ -958,7 +926,6 @@ Returns:
         lcmaps_plugindl_t ** list
         )
     \brief clean (free) the list of plugins and call the plugin termination functions
-    \param list
     \param list      pointer to list of plugins which has to be freeed.
     \retval 0 succes.
     \retval 1 failure.
@@ -977,7 +944,7 @@ static int clean_plugin_list(
         lcmaps_plugindl_t * plugin_next;
         int               i;
 
-        rc = plugin_entry->procs[TERMPROC]();
+        rc = (*(plugin_entry->plugin_terminate))();
         if (rc != LCMAPS_MOD_SUCCESS)
         {
             lcmaps_log(LOG_WARNING,"lcmaps.mod-clean_plugin_list(): failed to terminate plugin module %s\n", plugin_entry->pluginabsname);
@@ -1085,15 +1052,15 @@ int lcmaps_runPluginManager(
     while (plugin_entry)
     {
         void * pvalue=NULL;
-        char * argName=NULL;
-        char * argType=NULL;
+        const char * argName=NULL;
+        const char * argType=NULL;
         int iarg=0;
 
         /*
          * Now that we know that we're running in verification mode or not,
          * check for the plugin_verify() method
          */
-        if ((lcmaps_mode == LCMAPS_VERIFICATION_MODE) && (plugin_entry->procs[VERIFYPROC] == NULL))
+        if ((lcmaps_mode == LCMAPS_VERIFICATION_MODE) && (plugin_entry->plugin_verify == NULL))
         {
             lcmaps_log(LOG_ERR, "lcmaps.mod-lcmaps_runPluginManager(): Cannot find required \"plugin_verify()\" method in plugin \"%s\" (failure)\n", plugin_entry->pluginabsname);
             return 1;
@@ -1259,11 +1226,11 @@ int lcmaps_runPlugin(
     lcmaps_log_debug(3, "lcmaps.mod-lcmaps_runPlugin(): running plugin %s\n",plugin_entry->pluginabsname);
     if (lcmaps_mode == LCMAPS_VERIFICATION_MODE)
     {
-        rc = plugin_entry->procs[VERIFYPROC](plugin_entry->run_argc, plugin_entry->run_argv);
+        rc = (*(plugin_entry->plugin_verify))(plugin_entry->run_argc, plugin_entry->run_argv);
     }
     else /* LCMAPS_NORMAL_MODE */
     {
-        rc = plugin_entry->procs[RUNPROC](plugin_entry->run_argc, plugin_entry->run_argv);
+        rc = (*(plugin_entry->plugin_run))(plugin_entry->run_argc, plugin_entry->run_argv);
     }
 
     return rc;
@@ -1417,5 +1384,3 @@ void lcmaps_sigpipe_handler(int sig)
         lcmaps_log(LOG_NOTICE, "Caught signal SIGPIPE\n");
     }
 }
-
-#endif /* LCMAPS_PLUGINMANAGER_C */
