@@ -52,14 +52,17 @@
     -# lcmaps_stringVoData(): cast a VoData structure into a string
 */
 
+#define _XOPEN_SOURCE	500
+
 /******************************************************************************
                              Include header files
 ******************************************************************************/
 
 #include <stdio.h>
 #include <stdlib.h>
-/* #include <malloc.h> */
 #include <string.h>
+#include <sys/types.h>
+#include <errno.h>
 
 #include "lcmaps_vo_data.h"
 #include "lcmaps_log.h"
@@ -389,7 +392,7 @@ lcmaps_printVoData(
 {
     if (vo_data)
     {
-        lcmaps_log_debug(debug_level,"lcmaps_printVoData(): address of vo data struct: %p\n", vo_data);
+        lcmaps_log_debug(debug_level,"lcmaps_printVoData(): address of vo data struct: %p\n", (const void*)vo_data);
         lcmaps_log_debug(debug_level,"lcmaps_printVoData():                        VO: %s\n", vo_data->vo);
         lcmaps_log_debug(debug_level,"lcmaps_printVoData():                     GROUP: %s\n", vo_data->group);
         lcmaps_log_debug(debug_level,"lcmaps_printVoData():                  SUBGROUP: %s\n", vo_data->subgroup);
@@ -473,7 +476,7 @@ lcmaps_stringVoData(
     /* Skip over leading whitespace */
     strptr=vo_data->vo;
     strptr += strspn(strptr, VO_DATA_WHITESPACE_CHARS);
-    if ( (*strptr!='\0') && (strncmp(strptr,"NULL",4)) )
+    if ( (*strptr!='\0') && (strncmp(strptr,"NULL",(size_t)4)) )
     {
         /* Convention for solaris 2.8 and glibc-2.1 and higher:
          * totalchars is the number of chars written (excluding \0) if enough space had been 
@@ -481,14 +484,15 @@ lcmaps_stringVoData(
          * negative: error
          */
         totalchars=snprintf(bufptr,(size_t)buflen,"/VO=%s",strptr);
-        if ( (totalchars+1) > buflen )
+        if ( totalchars < 0 )
         {
-            lcmaps_log(LOG_ERR,"lcmaps_stringVoData(): could not write all characters into buffer for VO. Excess of characters: %d\n",totalchars+1-buflen);
+            lcmaps_log(LOG_ERR,"lcmaps_stringVoData(): error in snprintf(): %s\n",
+		    strerror(errno));
             return -1;
         }
-        else if ( totalchars < 0 )
+	else if ( (totalchars+1) > buflen )
         {
-            lcmaps_log(LOG_ERR,"lcmaps_stringVoData(): error in snprintf()\n");
+            lcmaps_log(LOG_ERR,"lcmaps_stringVoData(): could not write all characters into buffer for VO. Excess of characters: %d\n",totalchars+1-buflen);
             return -1;
         }
         else
@@ -512,17 +516,18 @@ lcmaps_stringVoData(
     /* Skip over leading whitespace */
     strptr=vo_data->group;
     strptr += strspn(strptr, VO_DATA_WHITESPACE_CHARS);
-    if ( (*strptr!='\0') && (strncmp(strptr,"NULL",4)) )
+    if ( (*strptr!='\0') && (strncmp(strptr,"NULL",(size_t)4)) )
     {
         totalchars=snprintf(bufptr,(size_t)buflen,"/GROUP=%s",strptr);
-        if ( (totalchars+1) > buflen )
+        if ( totalchars < 0 )
         {
-            lcmaps_log(LOG_ERR,"lcmaps_stringVoData(): could not write all characters into buffer for GROUP. Excess of characters: %d\n",totalchars+1-buflen);
+            lcmaps_log(LOG_ERR,"lcmaps_stringVoData(): error in snprintf(): %s\n",
+		    strerror(errno));
             return -1;
         }
-        else if ( totalchars < 0 )
+	else if ( (totalchars+1) > buflen )
         {
-            lcmaps_log(LOG_ERR,"lcmaps_stringVoData(): error in snprintf()\n");
+            lcmaps_log(LOG_ERR,"lcmaps_stringVoData(): could not write all characters into buffer for GROUP. Excess of characters: %d\n",totalchars+1-buflen);
             return -1;
         }
         else
@@ -545,17 +550,18 @@ lcmaps_stringVoData(
         /* Skip over leading whitespace */
         strptr=vo_data->role;
         strptr += strspn(strptr, VO_DATA_WHITESPACE_CHARS);
-        if ( (*strptr!='\0') && (strncmp(strptr,"NULL",4)) )
+        if ( (*strptr!='\0') && (strncmp(strptr,"NULL",(size_t)4)) )
         {
             totalchars=snprintf(bufptr,(size_t)buflen,"/ROLE=%s",strptr);
-            if ( (totalchars+1) > buflen )
+            if ( totalchars < 0 )
             {
-                lcmaps_log(LOG_ERR,"lcmaps_stringVoData(): could not write all characters into buffer for ROLE. Excess of characters: %d\n",totalchars+1-buflen);
+                lcmaps_log(LOG_ERR,"lcmaps_stringVoData(): error in snprintf(): %s\n",
+			strerror(errno));
                 return -1;
             }
-            else if ( totalchars < 0 )
+	    else if ( (totalchars+1) > buflen )
             {
-                lcmaps_log(LOG_ERR,"lcmaps_stringVoData(): error in snprintf()\n");
+                lcmaps_log(LOG_ERR,"lcmaps_stringVoData(): could not write all characters into buffer for ROLE. Excess of characters: %d\n",totalchars+1-buflen);
                 return -1;
             }
             else
@@ -572,17 +578,18 @@ lcmaps_stringVoData(
         /* Skip over leading whitespace */
         strptr=vo_data->capability;
         strptr += strspn(strptr, VO_DATA_WHITESPACE_CHARS);
-        if ( (*strptr!='\0') && (strncmp(strptr,"NULL",4)) )
+        if ( (*strptr!='\0') && (strncmp(strptr,"NULL",(size_t)4)) )
         {
             totalchars=snprintf(bufptr,(size_t)buflen,"/CAPABILITY=%s",strptr);
-            if ( (totalchars+1) > buflen )
+            if ( totalchars < 0 )
             {
-                lcmaps_log(LOG_ERR,"lcmaps_stringVoData(): could not write all characters into buffer for CAPABILITY. Excess of characters: %d\n",totalchars+1-buflen);
+                lcmaps_log(LOG_ERR,"lcmaps_stringVoData(): error in snprintf(): %s\n",
+			strerror(errno));
                 return -1;
             }
-            else if ( totalchars < 0 )
+            else if ( (totalchars+1) > buflen )
             {
-                lcmaps_log(LOG_ERR,"lcmaps_stringVoData(): error in snprintf()\n");
+                lcmaps_log(LOG_ERR,"lcmaps_stringVoData(): could not write all characters into buffer for CAPABILITY. Excess of characters: %d\n",totalchars+1-buflen);
                 return -1;
             }
             else
@@ -652,14 +659,14 @@ lcmaps_createVoMapping(
 
     if (vo_data_string)	{
 	if ( (newVoMapping->vostring = strdup(vo_data_string))==NULL )	{
-	    lcmaps_log(LOG_ERR, "%s: Out of memory\n");
+	    lcmaps_log(LOG_ERR, "%s: Out of memory\n", __func__);
 	    return NULL;
 	}
     }
 	    
     if (vo_data_string)	{
 	if ( (newVoMapping->groupname = strdup(groupname))==NULL )	{
-	    lcmaps_log(LOG_ERR, "%s: Out of memory\n");
+	    lcmaps_log(LOG_ERR, "%s: Out of memory\n", __func__);
 	    return NULL;
 	}
     }
@@ -877,7 +884,7 @@ lcmaps_printVoMapping(
 /******************************************************************************
 CVS Information:
     $Source: /srv/home/dennisvd/svn/mw-security/lcmaps/src/pluginmanager/lcmaps_vo_data.c,v $
-    $Date: 2012-03-16 16:16:23 +0100 (Fri, 16 Mar 2012) $
-    $Revision: 16177 $
+    $Date: 2014-07-08 10:39:48 +0200 (Tue, 08 Jul 2014) $
+    $Revision: 17848 $
     $Author: msalle $
 ******************************************************************************/
